@@ -188,7 +188,7 @@ typedef struct _insig
     void *g_outlet;             /* outlet for raw data */
 #endif
     t_float *g_inbuf;           /* buffered input samples */
-    t_float *g_invec;           /* new input samples */
+    t_sample *g_invec;           /* new input samples */
 } t_insig;
 
 typedef struct _bonk
@@ -247,8 +247,14 @@ typedef struct _bonk
 static void *bonk_new(t_symbol *s, long ac, t_atom *av);
 static void bonk_tick(t_bonk *x);
 static void bonk_doit(t_bonk *x);
+static void bonk_perform_generic(t_bonk *x, int n);
 static t_int *bonk_perform(t_int *w);
+static void bonk_perform64(t_bonk *x, t_object *dsp64, double **ins,
+                           long numins, double **outs, long numouts,
+                           long sampleframes, long flags, void *userparam);
 static void bonk_dsp(t_bonk *x, t_signal **sp);
+static void bonk_dsp64(t_bonk *x, t_object *dsp64, short *count,
+                       double samplerate, long maxvectorsize, long flags);
 void bonk_assist(t_bonk *x, void *b, long m, long a, char *s);
 static void bonk_free(t_bonk *x);
 void bonk_setup(void);
@@ -282,6 +288,7 @@ t_float qrsqrt(t_float f);
 double clock_getsystime();
 double clock_gettimesince(double prevsystime);
 char *strcpy(char *s1, const char *s2);
+#define SETFLOAT A_SETFLOAT
 #endif
 
 static void bonk_tick(t_bonk *x);
@@ -389,7 +396,7 @@ static void bonk_freefilterbank(t_filterbank *b)
     int i;
     if (bonk_filterbanklist == b)
         bonk_filterbanklist = b->b_next;
-    else for (b2 = bonk_filterbanklist; b3 = b2->b_next; b2 = b3)
+    else for (b2 = bonk_filterbanklist; (b3 = b2->b_next); b2 = b3)
         if (b3 == b)
     {
         b2->b_next = b3->b_next;
@@ -738,10 +745,7 @@ static void bonk_doit(t_bonk *x)
     x->x_debouncevel *= x->x_debouncedecay;
 }
 
-static t_int *bonk_perform(t_int *w)
-{
-    t_bonk *x = (t_bonk *)(w[1]);
-    int n = (int)(w[2]);
+static void bonk_perform_generic(t_bonk *x, int n) {
     int onset = 0;
     if (x->x_countdown >= n)
         x->x_countdown -= n;
@@ -763,12 +767,12 @@ static t_int *bonk_perform(t_int *w)
             for (i = 0, gp = x->x_insig; i < ninsig; i++, gp++)
             {
                 t_float *fp = gp->g_inbuf + infill;
-                t_float *in1 = gp->g_invec + onset;
+                t_sample *in1 = gp->g_invec + onset;
                 for (j = 0; j < m; j++)
                     *fp++ = *in1++;
             }
             infill += m;
-            x->x_infill = infill;   
+            x->x_infill = infill;
             if (infill == x->x_npoints)
             {
                 bonk_doit(x);
@@ -784,7 +788,7 @@ static t_int *bonk_perform(t_int *w)
                     for (n = 0, gp = x->x_insig; n < ninsig; n++, gp++)
                         for (i = overlap, fp1 = gp->g_inbuf,
                              fp2 = fp1 + x->x_period; i--;)
-                                *fp1++ = *fp2++;
+                            *fp1++ = *fp2++;
                     x->x_infill = overlap;
                 }
                 else x->x_infill = 0;
@@ -793,6 +797,13 @@ static t_int *bonk_perform(t_int *w)
             onset += m;
         }
     }
+}
+
+static t_int *bonk_perform(t_int *w)
+{
+    t_bonk *x = (t_bonk *)(w[1]);
+    int n = (int)(w[2]);
+    bonk_perform_generic(x, n);
     return (w+3);
 }
 
@@ -971,9 +982,10 @@ static void bonk_bang(t_bonk *x)
 #ifdef PD
 static void bonk_read(t_bonk *x, t_symbol *s)
 {
-    t_float vec[MAXNFILTERS];
+    float vec[MAXNFILTERS];
     int i, ntemplate = 0, remaining;
-    t_float *fp, *fp2;
+    float *fp;
+    t_float *fp2;
 
     /* fbar: canvas_open code taken from g_array.c */
     FILE *fd;
@@ -1002,7 +1014,7 @@ static void bonk_read(t_bonk *x, t_symbol *s)
         ntemplate++;
     }
 nomore:
-    if (remaining = (ntemplate % x->x_ninsig))
+    if ((remaining = (ntemplate % x->x_ninsig)))
     {
         post("bonk_read: %d templates not a multiple of %d; dropping extras");
         x->x_template = (t_template *)t_resizebytes(x->x_template,
@@ -1017,6 +1029,30 @@ nomore:
 #endif
 
 #ifdef MSP
+static void bonk_perform64(t_bonk *x, t_object *dsp64, double **ins,
+                           long numins, double **outs, long numouts,
+                           long sampleframes, long flags, void *userparam)
+{
+    int n = sampleframes;
+
+    int i = sampleframes, ninsig = x->x_ninsig;
+    t_insig *gp;
+
+    for (i = 0, gp = x->x_insig; i < ninsig; i++, gp++)
+        gp->g_invec = ins[0];
+
+    bonk_perform_generic(x, n);
+}
+
+static void bonk_dsp64(t_bonk *x, t_object *dsp64, short *count,
+                       double samplerate, long maxvectorsize, long flags)
+{
+
+    x->x_sr = samplerate;
+    object_method(dsp64, gensym("dsp_add64"), x, bonk_perform64, 0, NULL);
+}
+
+
 static void bonk_read(t_bonk *x, t_symbol *s)
 {
     defer(x, (method)bonk_doread, s, 0, NULL);
@@ -1024,7 +1060,7 @@ static void bonk_read(t_bonk *x, t_symbol *s)
 
 static void bonk_doread(t_bonk *x, t_symbol *s)
 {
-    long filetype = 'TEXT', outtype;
+    t_fourcc filetype = 'TEXT', outtype;
     char filename[512];
     short path;
     
@@ -1051,7 +1087,7 @@ static void bonk_openfile(t_bonk *x, char *filename, short path) {
     char **texthandle;
     char *tokptr;
     
-    long size;
+    t_ptr_size size;
 
     if (path_opensysfile(filename, path, &fh, READ_PERM)) {
         object_error((t_object *) x, "error opening %s", filename);
@@ -1084,7 +1120,7 @@ static void bonk_openfile(t_bonk *x, char *filename, short path) {
         ntemplate++;
     }
 nomore:
-    if (remaining = (ntemplate % x->x_ninsig))
+    if ((remaining = (ntemplate % x->x_ninsig)))
     {
         post("bonk_read: %d templates not a multiple of %d; dropping extras");
         x->x_template = (t_template *)t_resizebytes(x->x_template,
@@ -1137,7 +1173,7 @@ static void bonk_write(t_bonk *x, t_symbol *s)
 
 static void bonk_dowrite(t_bonk *x, t_symbol *s)
 {
-    long filetype = 'TEXT', outtype;
+    t_fourcc filetype = 'TEXT', outtype;
     char filename[MAX_FILENAME_CHARS];
     short path;
     
@@ -1159,7 +1195,7 @@ void bonk_writefile(t_bonk *x, char *filename, short path)
     t_template *tp = x->x_template;
     t_float *fp;
     long err;
-    long buflen;
+    t_ptr_size buflen;
     
     t_filehandle fh;
     
@@ -1415,6 +1451,7 @@ int main()
         class_addattr(c, attr);
     
         class_addmethod(c, (method)bonk_dsp, "dsp", A_CANT, 0);
+        class_addmethod(c, (method)bonk_dsp64, "dsp64", A_CANT, 0);
         class_addmethod(c, (method)bonk_bang, "bang", A_CANT, 0);
         class_addmethod(c, (method)bonk_forget, "forget", 0);
         class_addmethod(c, (method)bonk_thresh, "thresh", A_FLOAT, A_FLOAT, 0);
@@ -1440,7 +1477,7 @@ static void *bonk_new(t_symbol *s, long ac, t_atom *av)
     short j;
     t_bonk *x;
         
-    if (x = (t_bonk *)object_alloc(bonk_class)) {
+    if ((x = (t_bonk *)object_alloc(bonk_class))) {
         
         t_insig *g;
 
