@@ -75,6 +75,9 @@ static char pd_tilde_dllextent[] = ".d_fat",
 static char pd_tilde_dllextent[] = ".m_i386", pd_tilde_dllextent2[] = ".dll";
 #endif
 
+#define FOOFOO
+#include "binarymsg.c"
+
 /* ------------------------ pd_tilde~ ----------------------------- */
 
 #define MSGBUFSIZE 65536
@@ -94,13 +97,12 @@ typedef struct _pd_tilde
 #endif /* MSP */
     FILE *x_infd;
     FILE *x_outfd;
-    char *x_msgbuf;
-    int x_msgbufsize;
-    int x_infill;
+    t_binbuf *x_binbuf;
     int x_childpid;
     int x_ninsig;
     int x_noutsig;
     int x_fifo;
+    int x_binary;
     t_float x_sr;
     t_symbol *x_pddir;
     t_symbol *x_schedlibdir;
@@ -137,56 +139,62 @@ static void pd_tilde_close(t_pd_tilde *x)
 #else
         waitpid(x->x_childpid, 0, 0);
 #endif
-    if (x->x_msgbuf)
-        free(x->x_msgbuf);
+    binbuf_clear(x->x_binbuf);
     x->x_infd = x->x_outfd = 0;
     x->x_childpid = -1;
-    x->x_msgbuf = 0;
-    x->x_msgbufsize = 0;
 }
 
 static void pd_tilde_readmessages(t_pd_tilde *x)
 {
-    int gotsomething = 0, setclock = 0, wasempty = (x->x_infill == 0);
-    FILE *infd = x->x_infd;
-    while (1)
+    t_atom at;
+    binbuf_clear(x->x_binbuf);
+    if (x->x_binary)
     {
-        int c = getc(infd);
-        if (c == EOF)
+        int nonempty = 0;
+        while (1)
         {
-            PDERROR "pd~: read failed: %s", strerror(errno));
-            pd_tilde_close(x);
-            break;
-        }
-        if (x->x_infill >= x->x_msgbufsize)
-        {
-            char *z = realloc(x->x_msgbuf, x->x_msgbufsize+MSGBUFSIZE);
-            if (!z)
-            {
-                PDERROR "pd~: failed to grow input buffer");
-                pd_tilde_close(x);
+            pd_tilde_getatom(&at, x->x_infd);
+            if (!nonempty && at.a_type == A_SEMI)
                 break;
-            }
-            x->x_msgbuf = z;
-            x->x_msgbufsize += MSGBUFSIZE;
+            nonempty = (at.a_type != A_SEMI);
+            binbuf_add(x->x_binbuf, 1, &at);
         }
-        x->x_msgbuf[x->x_infill++] = c;
-        if (c == ';')
-        {
-            if (!gotsomething)
-                break;
-            gotsomething = 0;
-        }
-        else if (!isspace(c))
-            gotsomething = setclock = 1;
     }
-    if (setclock)
-        clock_delay(x->x_clock, 0);
-    else if (wasempty)
-        x->x_infill = 0;
+    else    /* ASCII */
+    {
+        t_binbuf *tmpb = binbuf_new();
+        while (1)
+        {
+            char msgbuf[MAXPDSTRING];
+            int c, infill = 0, n;
+            t_atom *vec;
+            while (isspace((c = getc(x->x_infd))) && c != EOF)
+                ;
+            if (c == EOF)
+                return;
+            do
+                msgbuf[infill++] = c;
+            while (!isspace((c = getc(x->x_infd))) && c != ';' && c != EOF) ;
+            binbuf_text(tmpb, msgbuf, infill);
+            n = binbuf_getnatom(tmpb);
+            vec = binbuf_getvec(tmpb);
+            binbuf_add(x->x_binbuf, n, vec);
+            if (!n)
+            {
+                bug("pd~");
+                break;  /* shouldn't happen */
+            }
+            if (vec[0].a_type == A_SEMI)
+                break;
+        }
+        binbuf_free(tmpb);
+        /* if (binbuf_getnatom(x->x_binbuf) > 1)
+            binbuf_print(x->x_binbuf); */
+    }
+    clock_delay(x->x_clock, 0);
 }
 
-#define FIXEDARG 11
+#define FIXEDARG 13
 #define MAXARG 100
 #ifdef _WIN32
 #define EXTENT ".com"
@@ -255,14 +263,16 @@ static void pd_tilde_donew(t_pd_tilde *x, char *pddir, char *schedlibdir,
     execargv[0] = pdexecbuf;
     execargv[1] = "-schedlib";
     execargv[2] = schedbuf;
-    execargv[3] = "-path";
-    execargv[4] = patchdir;
-    execargv[5] = "-inchannels";
-    execargv[6] = ninsigstr;
-    execargv[7] = "-outchannels";
-    execargv[8] = noutsigstr;
-    execargv[9] = "-r";
-    execargv[10] = sampleratestr;
+    execargv[3] = "-extraflags";
+    execargv[4] = (x->x_binary ? "b" : "a");
+    execargv[5] = "-path";
+    execargv[6] = patchdir;
+    execargv[7] = "-inchannels";
+    execargv[8] = ninsigstr;
+    execargv[9] = "-outchannels";
+    execargv[10] = noutsigstr;
+    execargv[11] = "-r";
+    execargv[12] = sampleratestr;
 
         /* convert atom arguments to strings (temporarily allocating space) */
     for (i = 0; i < argc; i++)
@@ -372,28 +382,24 @@ static void pd_tilde_donew(t_pd_tilde *x, char *pddir, char *schedlibdir,
     x->x_infd = fdopen(pipe2[0], "r");
     x->x_childpid = pid;
     for (i = 0; i < fifo; i++)
-        fprintf(x->x_outfd, "%s", ";\n0;\n");
-    fflush(x->x_outfd);
-    if (!(x->x_msgbuf = calloc(MSGBUFSIZE, 1)))
+        if (x->x_binary)
     {
-        PDERROR "pd~: can't allocate message buffer");
-        goto fail3;
+        putc(A_SEMI, x->x_outfd);
+        pd_tilde_putfloat(0, x->x_outfd);
+        putc(A_SEMI, x->x_outfd);
     }
-    x->x_msgbufsize = MSGBUFSIZE;
-    x->x_infill = 0;
+    else fprintf(x->x_outfd, "%s", ";\n0;\n");
+
+    fflush(x->x_outfd);
+    binbuf_clear(x->x_binbuf);
     pd_tilde_readmessages(x);
     return;
+#ifndef _WIN32
 fail3:
     close(pipe2[0]);
     close(pipe2[1]);
     if (x->x_childpid > 0)
-#ifdef _WIN32
-    {
-        int termstat;
-        _cwait(&termstat, x->x_childpid, WAIT_CHILD);
-    }
-#else
-        waitpid(x->x_childpid, 0, 0);
+    waitpid(x->x_childpid, 0, 0);
 #endif
 fail2:
     close(pipe1[0]);
@@ -407,31 +413,49 @@ fail1:
 static t_int *pd_tilde_perform(t_int *w)
 {
     t_pd_tilde *x = (t_pd_tilde *)(w[1]);
-    int n = (int)(w[2]), i, j, numbuffill = 0, c;
+    int n = (int)(w[2]), i, j, nsigs, numbuffill = 0, c;
     char numbuf[80];
     FILE *infd = x->x_infd;
     if (!infd)
         goto zeroit;
-    fprintf(x->x_outfd, ";\n");
-    if (!x->x_ninsig)
-        fprintf(x->x_outfd, "0\n");
-    else for (i = 0; i < x->x_ninsig; i++)
+    if (x->x_binary)
     {
-        t_sample *fp = x->x_insig[i];
-        for (j = 0; j < n; j++)
-            fprintf(x->x_outfd, "%g\n", *fp++);
-        for (; j < DEFDACBLKSIZE; j++)
+        putc(A_SEMI, x->x_outfd);
+        if (!x->x_ninsig)
+            pd_tilde_putfloat(0, x->x_outfd);
+        else for (i = 0; i < x->x_ninsig; i++)
+        {
+            t_sample *fp = x->x_insig[i];
+            for (j = 0; j < n; j++)
+                pd_tilde_putfloat(*fp++, x->x_outfd);
+            for (; j < DEFDACBLKSIZE; j++)
+                pd_tilde_putfloat(0, x->x_outfd);
+        }
+        putc(A_SEMI, x->x_outfd);
+    }
+    else
+    {
+        fprintf(x->x_outfd, ";\n");
+        if (!x->x_ninsig)
             fprintf(x->x_outfd, "0\n");
+        else for (i = 0; i < x->x_ninsig; i++)
+        {
+            t_sample *fp = x->x_insig[i];
+            for (j = 0; j < n; j++)
+                fprintf(x->x_outfd, "%g\n", *fp++);
+            for (; j < DEFDACBLKSIZE; j++)
+                fprintf(x->x_outfd, "0\n");
+        }
+        fprintf(x->x_outfd, ";\n");
     }
-    fprintf(x->x_outfd, ";\n");
     fflush(x->x_outfd);
-    i = j = 0;
-    while (1)
+    nsigs = j = 0;
+    if (x->x_binary)
     {
         while (1)
         {
-            c = getc(infd);
-            if (c == EOF)
+            t_atom at;
+            if (!pd_tilde_getatom(&at, infd))
             {
                 if (errno)
                     PDERROR "pd~: %s", strerror(errno));
@@ -439,40 +463,70 @@ static t_int *pd_tilde_perform(t_int *w)
                 pd_tilde_close(x);
                 goto zeroit;
             }
-            else if (!isspace(c) && c != ';')
-            {
-                if (numbuffill < (80-1))
-                    numbuf[numbuffill++] = c;
-            }
-            else
-            {
-                t_sample z;
-                if (numbuffill)
-                {
-                    numbuf[numbuffill] = 0;
-#if PD_FLOATSIZE == 32
-                    if (sscanf(numbuf, "%f", &z) < 1)
-#else
-                    if (sscanf(numbuf, "%lf", &z) < 1)
-#endif
-                        continue;
-                    if (i < x->x_noutsig)
-                        x->x_outsig[i][j] = z;
-                    if (++j >= DEFDACBLKSIZE)
-                        j = 0, i++;
-                }
-                numbuffill = 0;
+            if (at.a_type == A_SEMI)
                 break;
+            else if (at.a_type == A_FLOAT)
+            {
+                if (nsigs < x->x_noutsig)
+                    x->x_outsig[nsigs][j] = at.a_w.w_float;
+                if (++j >= DEFDACBLKSIZE)
+                    j = 0, nsigs++;
             }
+            else PDERROR "pd~: subprocess returned malformed audio");
         }
-        /* message terminated */
-        if (c == ';')
-            break;
     }
-    for (; i < x->x_noutsig; i++, j = 0)
+    else
+    {
+        while (1)
+        {
+            while (1)
+            {
+                c = getc(infd);
+                if (c == EOF)
+                {
+                    if (errno)
+                        PDERROR "pd~: %s", strerror(errno));
+                    else PDERROR "pd~: subprocess exited");
+                    pd_tilde_close(x);
+                    goto zeroit;
+                }
+                else if (!isspace(c) && c != ';')
+                {
+                    if (numbuffill < (80-1))
+                        numbuf[numbuffill++] = c;
+                }
+                else
+                {
+                    t_sample z;
+                    if (numbuffill)
+                    {
+                        numbuf[numbuffill] = 0;
+#if PD_FLOATSIZE == 32
+                        if (sscanf(numbuf, "%f", &z) < 1)
+#else
+                        if (sscanf(numbuf, "%lf", &z) < 1)
+#endif
+                            continue;
+                        if (nsigs < x->x_noutsig)
+                            x->x_outsig[nsigs][j] = z;
+                        if (++j >= DEFDACBLKSIZE)
+                            j = 0, nsigs++;
+                    }
+                    numbuffill = 0;
+                    break;
+                }
+            }
+            /* message terminated */
+            if (c == ';')
+                break;
+        }
+    }
+    if (nsigs < x->x_noutsig)
+        post("sigs %d, j %d", nsigs, j);
+    for (; nsigs < x->x_noutsig; nsigs++, j = 0)
     {
         for (; j < DEFDACBLKSIZE; j++)
-            x->x_outsig[i][j] = 0;
+            x->x_outsig[nsigs][j] = 0;
     }
     pd_tilde_readmessages(x);
     return (w+3);
@@ -572,14 +626,9 @@ static void pd_tilde_tick(t_pd_tilde *x)
 {
     int messstart = 0, i, n;
     t_atom *vec;
-    t_binbuf *b;
-    if (!x->x_msgbuf)
-        return;
-    b = binbuf_new();
-    binbuf_text(b, x->x_msgbuf, x->x_infill);
     /* binbuf_print(b); */
-    n = binbuf_getnatom(b);
-    vec = binbuf_getvec(b);
+    n = binbuf_getnatom(x->x_binbuf);
+    vec = binbuf_getvec(x->x_binbuf);
     for (i = 0; i < n; i++)
     {
         if (vec[i].a_type == A_SEMI)
@@ -592,8 +641,7 @@ static void pd_tilde_tick(t_pd_tilde *x)
             messstart = i+1;
         }
     }
-    binbuf_free(b);
-    x->x_infill = 0;
+    binbuf_clear(x->x_binbuf);
 }
 
 static void pd_tilde_anything(t_pd_tilde *x, t_symbol *s,
@@ -602,19 +650,34 @@ static void pd_tilde_anything(t_pd_tilde *x, t_symbol *s,
     char msgbuf[MAXPDSTRING];
     if (!x->x_outfd)
         return;
-    fprintf(x->x_outfd, "%s ", s->s_name);
-    while (argc--)
+    if (x->x_binary)
     {
-        atom_string(argv++, msgbuf, MAXPDSTRING);
-        fprintf(x->x_outfd, "%s ", msgbuf);
+        pd_tilde_putsymbol(s, x->x_outfd);
+        for (; argc--; argv++)
+        {
+            if (argv->a_type == A_FLOAT)
+                pd_tilde_putfloat(argv->a_w.w_float, x->x_outfd);
+            else if (argv->a_type == A_SYMBOL)
+                pd_tilde_putsymbol(argv->a_w.w_symbol, x->x_outfd);
+        }
+        putc(A_SEMI, x->x_outfd);
+    }
+    else
+    {
+        fprintf(x->x_outfd, "%s ", s->s_name);
+        while (argc--)
+        {
+            atom_string(argv++, msgbuf, MAXPDSTRING);
+            fprintf(x->x_outfd, "%s ", msgbuf);
+        }
+        fprintf(x->x_outfd, ";\n");
     }
-    fprintf(x->x_outfd, ";\n");
 }
 
 static void *pd_tilde_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_pd_tilde *x = (t_pd_tilde *)pd_new(pd_tilde_class);
-    int ninsig = 2, noutsig = 2, j, fifo = 5;
+    int ninsig = 2, noutsig = 2, j, fifo = 5, binary = 1;
     t_float sr = sys_getsr();
     t_sample **g;
     t_symbol *pddir = sys_libdir,
@@ -653,6 +716,11 @@ static void *pd_tilde_new(t_symbol *s, int argc, t_atom *argv)
             scheddir = atom_getsymbolarg(1, argc, argv);
             argc -= 2; argv += 2;
         }
+        else if (!strcmp(firstarg->s_name, "-ascii"))
+        {
+            binary = 0;
+            argc--; argv++;
+        }
         else break;
     }
 
@@ -677,8 +745,9 @@ static void *pd_tilde_new(t_symbol *s, int argc, t_atom *argv)
     x->x_outfd = 0;
     x->x_outfd = 0;
     x->x_childpid = -1;
-    x->x_msgbuf = 0;
     x->x_canvas = canvas_getcurrent();
+    x->x_binbuf = binbuf_new();
+    x->x_binary = binary;
     for (j = 1, g = x->x_insig; j < ninsig; j++, g++)
         inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
     x->x_outlet1 = outlet_new(&x->x_obj, 0);
@@ -702,7 +771,7 @@ void pd_tilde_setup(void)
 }
 #endif
 
-/* -------------------------- MSP glue ------------------------- */
+/* -------------------------- Max/MSP glue ------------------------- */
 #ifdef MSP
 
 #define LOTS 10000
@@ -712,13 +781,8 @@ static void pd_tilde_tick(t_pd_tilde *x)
     int messstart = 0, i, n = 0;
     t_atom vec[LOTS];
     long z1 = 0, z2 = 0;
-    void *b;
-    if (!x->x_msgbuf)
-        return;
-    b = binbuf_new();
-    binbuf_text(b, &x->x_msgbuf, x->x_infill);
     /* binbuf_print(b); */
-    while (!binbuf_getatom(b, &z1, &z2, vec+n))
+    while (!binbuf_getatom(x->x_binbuf, &z1, &z2, vec+n))
     if (++n >= LOTS)
         break;
     for (i = 0; i < n; i++)
@@ -741,8 +805,7 @@ static void pd_tilde_tick(t_pd_tilde *x)
             messstart = i+1;
         }
     }
-    binbuf_free(b);
-    x->x_infill = 0;
+    binbuf_clear(b);
 }
 
 static void pd_tilde_anything(t_pd_tilde *x, t_symbol *s,
@@ -859,7 +922,8 @@ static void *pd_tilde_new(t_symbol *s, long ac, t_atom *av)
         x->x_outfd = 0;
         x->x_outfd = 0;
         x->x_childpid = -1;
-        x->x_msgbuf = 0;
+        x->x_binbuf = binbuf_new();
+        x->x_binary = binary;
     }
     return (x);
 }
