@@ -774,7 +774,7 @@ static t_int *threshold_tilde_perform(t_int *w)
 {
     t_sample *in1 = (t_sample *)(w[1]);
     t_threshold_tilde *x = (t_threshold_tilde *)(w[2]);
-    int n = (t_int)(w[3]);
+    int n = (int)w[3];
     if (x->x_deadwait > 0)
         x->x_deadwait -= x->x_msecpertick;
     else if (x->x_state)
