@@ -5,7 +5,6 @@
 /* g_7_guis.c written by Thomas Musil (c) IEM KUG Graz Austria 2000-2001 */
 /* thanks to Miller Puckette, Guenther Geiger and Krzystof Czaja */
 
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -22,7 +21,6 @@
 #include <unistd.h>
 #endif
 
-
 /* --------------- bng     gui-bang ------------------------- */
 
 t_widgetbehavior bng_widgetbehavior;
@@ -30,85 +28,89 @@ static t_class *bng_class;
 
 /*  widget helper functions  */
 
-
 void bng_draw_update(t_bng *x, t_glist *glist)
 {
     if(glist_isvisible(glist))
     {
-        sys_vgui(".x%lx.c itemconfigure %lxBUT -fill #%6.6x\n", glist_getcanvas(glist), x,
-                 x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol);
+        sys_vgui(".x%lx.c itemconfigure %lxBUT -fill #%6.6x\n",
+            glist_getcanvas(glist), x,
+            (x->x_flashed ? x->x_gui.x_fcol : x->x_gui.x_bcol));
     }
 }
 
 void bng_draw_new(t_bng *x, t_glist *glist)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    int inset = IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c create rectangle %d %d %d %d -width %d -fill #%6.6x -tags %lxBASE\n",
              canvas, xpos, ypos,
              xpos + x->x_gui.x_w, ypos + x->x_gui.x_h,
              IEMGUI_ZOOM(x),
              x->x_gui.x_bcol, x);
+    if(!x->x_gui.x_fsf.x_snd_able)
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]\n",
+             canvas,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h,
+             x, 0);
+    if(!x->x_gui.x_fsf.x_rcv_able)
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
+             canvas,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
     sys_vgui(".x%lx.c create oval %d %d %d %d -width %d -fill #%6.6x -tags %lxBUT\n",
-             canvas, xpos+1, ypos+1,
-             xpos + x->x_gui.x_w-1, ypos + x->x_gui.x_h-1,
+             canvas, xpos + inset, ypos + inset,
+             xpos + x->x_gui.x_w - inset, ypos + x->x_gui.x_h - inset,
              IEMGUI_ZOOM(x),
-             x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol, x);
+             (x->x_flashed ? x->x_gui.x_fcol : x->x_gui.x_bcol), x);
     sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
              -font {{%s} -%d %s} -fill #%6.6x -tags [list %lxLABEL label text]\n",
-             canvas, xpos+x->x_gui.x_ldx * zoomlabel,
-             ypos+x->x_gui.x_ldy * zoomlabel,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
-             x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
+             canvas, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
+             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
              x->x_gui.x_lcol, x);
-    if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxOUT%d outlet]\n",
-             canvas, xpos,
-             ypos + x->x_gui.x_h+1-2*IEMGUI_ZOOM(x), xpos + IOWIDTH,
-             ypos + x->x_gui.x_h, x, 0);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxIN%d inlet]\n",
-             canvas, xpos, ypos,
-             xpos + IOWIDTH, ypos-1+2*IEMGUI_ZOOM(x), x, 0);
 }
 
 void bng_draw_move(t_bng *x, t_glist *glist)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    int inset = IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c coords %lxBASE %d %d %d %d\n",
              canvas, x, xpos, ypos,
              xpos + x->x_gui.x_w, ypos + x->x_gui.x_h);
-    sys_vgui(".x%lx.c coords %lxBUT %d %d %d %d\n",
-             canvas, x, xpos+1,ypos+1,
-             xpos + x->x_gui.x_w-1, ypos + x->x_gui.x_h-1);
-    sys_vgui(".x%lx.c itemconfigure %lxBUT -fill #%6.6x\n", canvas, x,
-             x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xpos+x->x_gui.x_ldx * zoomlabel,
-             ypos+x->x_gui.x_ldy * zoomlabel);
     if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
-             canvas, x, 0, xpos,
-             ypos + x->x_gui.x_h+1-2*IEMGUI_ZOOM(x), xpos + IOWIDTH,
-             ypos + x->x_gui.x_h);
+             canvas, x, 0,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h);
     if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
-             canvas, x, 0, xpos, ypos,
-             xpos + IOWIDTH, ypos-1+2*IEMGUI_ZOOM(x));
+             canvas, x, 0,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh);
+    sys_vgui(".x%lx.c coords %lxBUT %d %d %d %d\n",
+             canvas, x, xpos + inset, ypos + inset,
+             xpos + x->x_gui.x_w - inset, ypos + x->x_gui.x_h - inset);
+    sys_vgui(".x%lx.c itemconfigure %lxBUT -fill #%6.6x\n", canvas, x,
+             (x->x_flashed ? x->x_gui.x_fcol : x->x_gui.x_bcol));
+    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
+             canvas, x, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x));
 }
 
 void bng_draw_erase(t_bng* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c delete %lxBASE\n", canvas, x);
     sys_vgui(".x%lx.c delete %lxBUT\n", canvas, x);
@@ -121,41 +123,45 @@ void bng_draw_erase(t_bng* x, t_glist* glist)
 
 void bng_draw_config(t_bng* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%6.6x -text {%s} \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
+             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_lcol),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""));
     sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%6.6x\n", canvas, x, x->x_gui.x_bcol);
     sys_vgui(".x%lx.c itemconfigure %lxBUT -fill #%6.6x\n", canvas, x,
-             x->x_flashed?x->x_gui.x_fcol:x->x_gui.x_bcol);
+             (x->x_flashed ? x->x_gui.x_fcol : x->x_gui.x_bcol));
 }
 
 void bng_draw_io(t_bng* x, t_glist* glist, int old_snd_rcv_flags)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxOUT%d\n",
-             canvas, xpos,
-             ypos + x->x_gui.x_h-1, xpos + IOWIDTH,
-             ypos + x->x_gui.x_h, x, 0);
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
+             canvas,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h,
+             x, 0);
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
     if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxIN%d\n",
-             canvas, xpos, ypos,
-             xpos + IOWIDTH, ypos+1, x, 0);
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
+             canvas,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
 }
 
 void bng_draw_select(t_bng* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if(x->x_gui.x_fsf.x_selected)
     {
@@ -191,7 +197,8 @@ void bng_draw(t_bng *x, t_glist *glist, int mode)
 
 /* ------------------------ bng widgetbehaviour----------------------------- */
 
-static void bng_getrect(t_gobj *z, t_glist *glist, int *xp1, int *yp1, int *xp2, int *yp2)
+static void bng_getrect(t_gobj *z, t_glist *glist,
+                        int *xp1, int *yp1, int *xp2, int *yp2)
 {
     t_bng *x = (t_bng *)z;
 
@@ -210,7 +217,7 @@ static void bng_save(t_gobj *z, t_binbuf *b)
     iemgui_save(&x->x_gui, srl, bflcol);
     binbuf_addv(b, "ssiisiiiisssiiiisss", gensym("#X"),gensym("obj"),
                 (int)x->x_gui.x_obj.te_xpix, (int)x->x_gui.x_obj.te_ypix,
-                gensym("bng"), x->x_gui.x_w,
+                gensym("bng"), x->x_gui.x_w/IEMGUI_ZOOM(x),
                 x->x_flashtime_hold, x->x_flashtime_break,
                 iem_symargstoint(&x->x_gui.x_isa),
                 srl[0], srl[1], srl[2],
@@ -253,7 +260,7 @@ static void bng_properties(t_gobj *z, t_glist *owner)
             %s %d %d \
             %d %d \
             #%06x #%06x #%06x\n",
-            x->x_gui.x_w, IEM_GUI_MINSIZE,
+            x->x_gui.x_w/IEMGUI_ZOOM(x), IEM_GUI_MINSIZE,
             x->x_flashtime_break, x->x_flashtime_hold, 2,/*min_max_schedule+clip*/
             -1, x->x_gui.x_isa.x_loadinit, -1, -1,/*no linlog, no multi*/
             srl[0]->s_name, srl[1]->s_name,
@@ -277,7 +284,7 @@ static void bng_set(t_bng *x)
     clock_delay(x->x_clock_hld, holdtime);
 }
 
-static void bng_bout1(t_bng *x)/*wird nur mehr gesendet, wenn snd != rcv*/
+static void bng_bout1(t_bng *x) /*wird nur mehr gesendet, wenn snd != rcv*/
 {
     if(!x->x_gui.x_fsf.x_put_in2out)
     {
@@ -289,7 +296,7 @@ static void bng_bout1(t_bng *x)/*wird nur mehr gesendet, wenn snd != rcv*/
         pd_bang(x->x_gui.x_snd->s_thing);
 }
 
-static void bng_bout2(t_bng *x)/*wird immer gesendet, wenn moeglich*/
+static void bng_bout2(t_bng *x) /*wird immer gesendet, wenn moeglich*/
 {
     if(!x->x_gui.x_fsf.x_put_in2out)
     {
@@ -301,7 +308,7 @@ static void bng_bout2(t_bng *x)/*wird immer gesendet, wenn moeglich*/
         pd_bang(x->x_gui.x_snd->s_thing);
 }
 
-static void bng_bang(t_bng *x)/*wird nur mehr gesendet, wenn snd != rcv*/
+static void bng_bang(t_bng *x) /*wird nur mehr gesendet, wenn snd != rcv*/
 {
     if(!x->x_gui.x_isa.x_locked)
     {
@@ -310,7 +317,7 @@ static void bng_bang(t_bng *x)/*wird nur mehr gesendet, wenn snd != rcv*/
     }
 }
 
-static void bng_bang2(t_bng *x)/*wird immer gesendet, wenn moeglich*/
+static void bng_bang2(t_bng *x) /*wird immer gesendet, wenn moeglich*/
 {
     if(!x->x_gui.x_isa.x_locked)
     {
@@ -322,12 +329,12 @@ static void bng_bang2(t_bng *x)/*wird immer gesendet, wenn moeglich*/
 static void bng_dialog(t_bng *x, t_symbol *s, int argc, t_atom *argv)
 {
     t_symbol *srl[3];
-    int a = (int)atom_getintarg(0, argc, argv);
-    int fthold = (int)atom_getintarg(2, argc, argv);
-    int ftbreak = (int)atom_getintarg(3, argc, argv);
+    int a = (int)atom_getfloatarg(0, argc, argv);
+    int fthold = (int)atom_getfloatarg(2, argc, argv);
+    int ftbreak = (int)atom_getfloatarg(3, argc, argv);
     int sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
 
-    x->x_gui.x_w = iemgui_clip_size(a);
+    x->x_gui.x_w = iemgui_clip_size(a) * IEMGUI_ZOOM(x);
     x->x_gui.x_h = x->x_gui.x_w;
     bng_check_minmax(x, ftbreak, fthold);
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
@@ -359,9 +366,7 @@ static void bng_pointer(t_bng *x, t_gpointer *gp)
 {bng_bang2(x);}
 
 static void bng_list(t_bng *x, t_symbol *s, int ac, t_atom *av)
-{
-    bng_bang2(x);
-}
+{bng_bang2(x);}
 
 static void bng_anything(t_bng *x, t_symbol *s, int argc, t_atom *argv)
 {bng_bang2(x);}
@@ -377,7 +382,7 @@ static void bng_loadbang(t_bng *x, t_floatarg action)
 
 static void bng_size(t_bng *x, t_symbol *s, int ac, t_atom *av)
 {
-    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
+    x->x_gui.x_w = iemgui_clip_size((int)atom_getfloatarg(0, ac, av)) * IEMGUI_ZOOM(x);
     x->x_gui.x_h = x->x_gui.x_w;
     iemgui_size((void *)x, &x->x_gui);
 }
@@ -390,8 +395,8 @@ static void bng_pos(t_bng *x, t_symbol *s, int ac, t_atom *av)
 
 static void bng_flashtime(t_bng *x, t_symbol *s, int ac, t_atom *av)
 {
-    bng_check_minmax(x, (int)atom_getintarg(0, ac, av),
-                     (int)atom_getintarg(1, ac, av));
+    bng_check_minmax(x, (int)atom_getfloatarg(0, ac, av),
+                        (int)atom_getfloatarg(1, ac, av));
 }
 
 static void bng_color(t_bng *x, t_symbol *s, int ac, t_atom *av)
@@ -414,7 +419,7 @@ static void bng_label_font(t_bng *x, t_symbol *s, int ac, t_atom *av)
 
 static void bng_init(t_bng *x, t_floatarg f)
 {
-    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
+    x->x_gui.x_isa.x_loadinit = (f == 0.0) ? 0 : 1;
 }
 
 static void bng_tick_hld(t_bng *x)
@@ -431,11 +436,11 @@ static void bng_tick_lck(t_bng *x)
 static void *bng_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_bng *x = (t_bng *)pd_new(bng_class);
-    int a=IEM_GUI_DEFAULTSIZE;
-    int ldx=17, ldy=7;
-    int fs=10;
-    int ftbreak=IEM_BNG_DEFAULTBREAKFLASHTIME,
-        fthold=IEM_BNG_DEFAULTHOLDFLASHTIME;
+    int a = IEM_GUI_DEFAULTSIZE;
+    int ldx = 17, ldy = 7;
+    int fs = 10;
+    int ftbreak = IEM_BNG_DEFAULTBREAKFLASHTIME,
+        fthold = IEM_BNG_DEFAULTHOLDFLASHTIME;
     char str[144];
 
     iem_inttosymargs(&x->x_gui.x_isa, 0);
@@ -455,15 +460,15 @@ static void *bng_new(t_symbol *s, int argc, t_atom *argv)
        &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10))
     {
 
-        a = (int)atom_getintarg(0, argc, argv);
-        fthold = (int)atom_getintarg(1, argc, argv);
-        ftbreak = (int)atom_getintarg(2, argc, argv);
-        iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(3, argc, argv));
+        a = (int)atom_getfloatarg(0, argc, argv);
+        fthold = (int)atom_getfloatarg(1, argc, argv);
+        ftbreak = (int)atom_getfloatarg(2, argc, argv);
+        iem_inttosymargs(&x->x_gui.x_isa, atom_getfloatarg(3, argc, argv));
         iemgui_new_getnames(&x->x_gui, 4, argv);
-        ldx = (int)atom_getintarg(7, argc, argv);
-        ldy = (int)atom_getintarg(8, argc, argv);
-        iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(9, argc, argv));
-        fs = (int)atom_getintarg(10, argc, argv);
+        ldx = (int)atom_getfloatarg(7, argc, argv);
+        ldy = (int)atom_getfloatarg(8, argc, argv);
+        iem_inttofstyle(&x->x_gui.x_fsf, atom_getfloatarg(9, argc, argv));
+        fs = (int)atom_getfloatarg(10, argc, argv);
         iemgui_all_loadcolors(&x->x_gui, argv+11, argv+12, argv+13);
     }
     else iemgui_new_getnames(&x->x_gui, 4, 0);
@@ -499,6 +504,7 @@ static void *bng_new(t_symbol *s, int argc, t_atom *argv)
     x->x_lastflashtime = clock_getsystime();
     x->x_clock_hld = clock_new(x, (t_method)bng_tick_hld);
     x->x_clock_lck = clock_new(x, (t_method)bng_tick_lck);
+    iemgui_newzoom(&x->x_gui);
     outlet_new(&x->x_gui.x_obj, &s_bang);
     return (x);
 }
@@ -515,32 +521,43 @@ static void bng_ff(t_bng *x)
 void g_bang_setup(void)
 {
     bng_class = class_new(gensym("bng"), (t_newmethod)bng_new,
-                          (t_method)bng_ff, sizeof(t_bng), 0, A_GIMME, 0);
+        (t_method)bng_ff, sizeof(t_bng), 0, A_GIMME, 0);
     class_addbang(bng_class, bng_bang);
     class_addfloat(bng_class, bng_float);
     class_addsymbol(bng_class, bng_symbol);
     class_addpointer(bng_class, bng_pointer);
     class_addlist(bng_class, bng_list);
     class_addanything(bng_class, bng_anything);
-    class_addmethod(bng_class, (t_method)bng_click, gensym("click"),
-                    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
-    class_addmethod(bng_class, (t_method)bng_dialog, gensym("dialog"),
-                    A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_loadbang, gensym("loadbang"),
-        A_DEFFLOAT, 0);
-    class_addmethod(bng_class, (t_method)bng_size, gensym("size"), A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_delta, gensym("delta"), A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_pos, gensym("pos"), A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_flashtime, gensym("flashtime"), A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_color, gensym("color"), A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_send, gensym("send"), A_DEFSYM, 0);
-    class_addmethod(bng_class, (t_method)bng_receive, gensym("receive"), A_DEFSYM, 0);
-    class_addmethod(bng_class, (t_method)bng_label, gensym("label"), A_DEFSYM, 0);
-    class_addmethod(bng_class, (t_method)bng_label_pos, gensym("label_pos"), A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_label_font, gensym("label_font"), A_GIMME, 0);
-    class_addmethod(bng_class, (t_method)bng_init, gensym("init"), A_FLOAT, 0);
-    class_addmethod(bng_class, (t_method)iemgui_zoom, gensym("zoom"),
-        A_CANT, 0);
+    class_addmethod(bng_class, (t_method)bng_click,
+        gensym("click"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
+    class_addmethod(bng_class, (t_method)bng_dialog,
+        gensym("dialog"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_loadbang,
+        gensym("loadbang"), A_DEFFLOAT, 0);
+    class_addmethod(bng_class, (t_method)bng_size,
+        gensym("size"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_delta,
+        gensym("delta"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_pos,
+        gensym("pos"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_flashtime,
+        gensym("flashtime"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_color,
+        gensym("color"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_send,
+        gensym("send"), A_DEFSYM, 0);
+    class_addmethod(bng_class, (t_method)bng_receive,
+        gensym("receive"), A_DEFSYM, 0);
+    class_addmethod(bng_class, (t_method)bng_label,
+        gensym("label"), A_DEFSYM, 0);
+    class_addmethod(bng_class, (t_method)bng_label_pos,
+        gensym("label_pos"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_label_font,
+        gensym("label_font"), A_GIMME, 0);
+    class_addmethod(bng_class, (t_method)bng_init,
+        gensym("init"), A_FLOAT, 0);
+    class_addmethod(bng_class, (t_method)iemgui_zoom,
+        gensym("zoom"), A_CANT, 0);
     bng_widgetbehavior.w_getrectfn = bng_getrect;
     bng_widgetbehavior.w_displacefn = iemgui_displace;
     bng_widgetbehavior.w_selectfn = iemgui_select;
