@@ -5,7 +5,6 @@
 /* g_7_guis.c written by Thomas Musil (c) IEM KUG Graz Austria 2000-2001 */
 /* thanks to Miller Puckette, Guenther Geiger and Krzystof Czaja */
 
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -22,8 +21,10 @@
 #include <unistd.h>
 #endif
 
+#define LMARGIN 3
+#define RMARGIN 2
 
-/* ------------ hsl    gui-horicontal  slider ----------------------- */
+/* ------------ hsl gui-horizontal  slider ----------------------- */
 
 t_widgetbehavior hslider_widgetbehavior;
 static t_class *hslider_class;
@@ -35,83 +36,88 @@ static void hslider_draw_update(t_gobj *client, t_glist *glist)
     t_hslider *x = (t_hslider *)client;
     if (glist_isvisible(glist))
     {
-        int r = text_xpix(&x->x_gui.x_obj, glist) + (x->x_val + 50)/100;
-        int ypos=text_ypix(&x->x_gui.x_obj, glist);
-        t_canvas *canvas=glist_getcanvas(glist);
+        int r = text_xpix(&x->x_gui.x_obj, glist) + ((x->x_val + 50)/100);
+        int ypos = text_ypix(&x->x_gui.x_obj, glist);
+        t_canvas *canvas = glist_getcanvas(glist);
         sys_vgui(".x%lx.c coords %lxKNOB %d %d %d %d\n",
-                 canvas, x, r, ypos+1,
-                 r, ypos + x->x_gui.x_h);
+                 canvas, x, r, ypos + IEMGUI_ZOOM(x),
+                 r, ypos + x->x_gui.x_h - IEMGUI_ZOOM(x));
     }
 }
 
 static void hslider_draw_new(t_hslider *x, t_glist *glist)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    int lmargin = LMARGIN * IEMGUI_ZOOM(x), rmargin = RMARGIN * IEMGUI_ZOOM(x);
     int r = xpos + (x->x_val + 50)/100;
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c create rectangle %d %d %d %d -width %d -fill #%06x -tags %lxBASE\n",
-             canvas, xpos-3, ypos,
-             xpos + x->x_gui.x_w+2, ypos + x->x_gui.x_h,
+             canvas, xpos - lmargin, ypos,
+             xpos + x->x_gui.x_w + rmargin, ypos + x->x_gui.x_h,
              IEMGUI_ZOOM(x),
              x->x_gui.x_bcol, x);
-    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxKNOB\n",
-             canvas, r, ypos+1, r,
-             ypos + x->x_gui.x_h, 1 + 2 * IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
-    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-             -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
-             canvas, xpos+x->x_gui.x_ldx * zoomlabel,
-             ypos+x->x_gui.x_ldy * zoomlabel,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
-             x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_lcol, x);
     if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]\n",
-             canvas, xpos-3, ypos + x->x_gui.x_h+1-2*IEMGUI_ZOOM(x),
-             xpos+4, ypos + x->x_gui.x_h, x, 0);
+             canvas,
+             xpos - lmargin, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos - lmargin + iow, ypos + x->x_gui.x_h,
+             x, 0);
     if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
-             canvas, xpos-3, ypos,
-             xpos+4, ypos-1+2*IEMGUI_ZOOM(x), x, 0);
+             canvas,
+             xpos - lmargin, ypos,
+             xpos - lmargin + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
+    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
+             -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
+             canvas, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
+             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             x->x_gui.x_lcol, x);
+    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxKNOB\n",
+             canvas, r, ypos + IEMGUI_ZOOM(x),
+             r, ypos + x->x_gui.x_h - IEMGUI_ZOOM(x),
+             1 + 2 * IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
 }
 
 static void hslider_draw_move(t_hslider *x, t_glist *glist)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    int lmargin = LMARGIN * IEMGUI_ZOOM(x), rmargin = RMARGIN * IEMGUI_ZOOM(x);
     int r = xpos + (x->x_val + 50)/100;
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c coords %lxBASE %d %d %d %d\n",
              canvas, x,
-             xpos-3, ypos,
-             xpos + x->x_gui.x_w+2, ypos + x->x_gui.x_h);
-    sys_vgui(".x%lx.c coords %lxKNOB %d %d %d %d\n",
-             canvas, x, r, ypos+1,
-             r, ypos + x->x_gui.x_h);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xpos+x->x_gui.x_ldx * zoomlabel,
-             ypos+x->x_gui.x_ldy * zoomlabel);
+             xpos - lmargin, ypos,
+             xpos + x->x_gui.x_w + rmargin, ypos + x->x_gui.x_h);
     if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
              canvas, x, 0,
-             xpos-3, ypos + x->x_gui.x_h+1-2*IEMGUI_ZOOM(x),
-             xpos+4, ypos + x->x_gui.x_h);
+             xpos - lmargin, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos - lmargin + iow, ypos + x->x_gui.x_h);
     if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
              canvas, x, 0,
-             xpos-3, ypos,
-             xpos+4, ypos-1+2*IEMGUI_ZOOM(x));
+             xpos - lmargin, ypos,
+             xpos - lmargin + iow, ypos - IEMGUI_ZOOM(x) + ioh);
+    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
+             canvas, x, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x));
+    sys_vgui(".x%lx.c coords %lxKNOB %d %d %d %d\n",
+             canvas, x, r, ypos + IEMGUI_ZOOM(x),
+             r, ypos + x->x_gui.x_h - IEMGUI_ZOOM(x));
 }
 
-static void hslider_draw_erase(t_hslider* x,t_glist* glist)
+static void hslider_draw_erase(t_hslider* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c delete %lxBASE\n", canvas, x);
     sys_vgui(".x%lx.c delete %lxKNOB\n", canvas, x);
@@ -122,41 +128,55 @@ static void hslider_draw_erase(t_hslider* x,t_glist* glist)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
 }
 
-static void hslider_draw_config(t_hslider* x,t_glist* glist)
+static void hslider_draw_config(t_hslider* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
+             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_lcol),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""));
     sys_vgui(".x%lx.c itemconfigure %lxKNOB -fill #%06x\n", canvas, x, x->x_gui.x_fcol);
     sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%06x\n", canvas, x, x->x_gui.x_bcol);
 }
 
-static void hslider_draw_io(t_hslider* x,t_glist* glist, int old_snd_rcv_flags)
+static void hslider_draw_io(t_hslider* x, t_glist* glist, int old_snd_rcv_flags)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    int lmargin = LMARGIN * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxOUT%d\n",
-             canvas, xpos-3, ypos + x->x_gui.x_h-1,
-             xpos+4, ypos + x->x_gui.x_h, x, 0);
+    {
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
+             canvas,
+             xpos - lmargin, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos - lmargin + iow, ypos + x->x_gui.x_h,
+             x, 0);
+        /* keep knob above outlet */
+        sys_vgui(".x%lx.c raise %lxKNOB %lxOUT%d\n", canvas, x, x, 0);
+    }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
     if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxIN%d\n",
-             canvas, xpos-3, ypos,
-             xpos+4, ypos+1, x, 0);
+    {
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
+             canvas,
+             xpos - lmargin, ypos,
+             xpos - lmargin + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
+        /* keep label and knob above inlet */
+        sys_vgui(".x%lx.c raise %lxKNOB %lxIN%d\n", canvas, x, x, 0);
+    }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
 }
 
-static void hslider_draw_select(t_hslider* x,t_glist* glist)
+static void hslider_draw_select(t_hslider* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if(x->x_gui.x_fsf.x_selected)
     {
@@ -196,9 +216,9 @@ static void hslider_getrect(t_gobj *z, t_glist *glist,
 {
     t_hslider* x = (t_hslider*)z;
 
-    *xp1 = text_xpix(&x->x_gui.x_obj, glist) - 3;
+    *xp1 = text_xpix(&x->x_gui.x_obj, glist) - LMARGIN*glist_getzoom(glist);
     *yp1 = text_ypix(&x->x_gui.x_obj, glist);
-    *xp2 = *xp1 + x->x_gui.x_w + 5;
+    *xp2 = *xp1 + x->x_gui.x_w + (LMARGIN + RMARGIN)*glist_getzoom(glist);
     *yp2 = *yp1 + x->x_gui.x_h;
 }
 
@@ -209,9 +229,9 @@ static void hslider_save(t_gobj *z, t_binbuf *b)
     t_symbol *srl[3];
 
     iemgui_save(&x->x_gui, srl, bflcol);
-    binbuf_addv(b, "ssiisiiffiisssiiiisssii", gensym("#X"),gensym("obj"),
+    binbuf_addv(b, "ssiisiiffiisssiiiisssii", gensym("#X"), gensym("obj"),
                 (int)x->x_gui.x_obj.te_xpix, (int)x->x_gui.x_obj.te_ypix,
-                gensym("hsl"), x->x_gui.x_w, x->x_gui.x_h,
+                gensym("hsl"), x->x_gui.x_w/IEMGUI_ZOOM(x), x->x_gui.x_h/IEMGUI_ZOOM(x),
                 (t_float)x->x_min, (t_float)x->x_max,
                 x->x_lin0_log1, iem_symargstoint(&x->x_gui.x_isa),
                 srl[0], srl[1], srl[2],
@@ -224,8 +244,8 @@ static void hslider_save(t_gobj *z, t_binbuf *b)
 
 void hslider_check_width(t_hslider *x, int w)
 {
-    if(w < IEM_SL_MINSIZE)
-        w = IEM_SL_MINSIZE;
+    if(w < IEM_SL_MINSIZE * IEMGUI_ZOOM(x))
+        w = IEM_SL_MINSIZE * IEMGUI_ZOOM(x);
     x->x_gui.x_w = w;
     if(x->x_val > (x->x_gui.x_w*100 - 100))
     {
@@ -233,16 +253,16 @@ void hslider_check_width(t_hslider *x, int w)
         x->x_val = x->x_pos;
     }
     if(x->x_lin0_log1)
-        x->x_k = log(x->x_max/x->x_min)/(double)(x->x_gui.x_w - 1);
+        x->x_k = log(x->x_max/x->x_min) / (double)(x->x_gui.x_w/IEMGUI_ZOOM(x) - 1);
     else
-        x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);
+        x->x_k = (x->x_max - x->x_min) / (double)(x->x_gui.x_w/IEMGUI_ZOOM(x) - 1);
 }
 
 void hslider_check_minmax(t_hslider *x, double min, double max)
 {
     if(x->x_lin0_log1)
     {
-        if((min == 0.0)&&(max == 0.0))
+        if((min == 0.0) && (max == 0.0))
             max = 1.0;
         if(max > 0.0)
         {
@@ -258,9 +278,9 @@ void hslider_check_minmax(t_hslider *x, double min, double max)
     x->x_min = min;
     x->x_max = max;
     if(x->x_lin0_log1)
-        x->x_k = log(x->x_max/x->x_min)/(double)(x->x_gui.x_w - 1);
+        x->x_k = log(x->x_max/x->x_min) / (double)(x->x_gui.x_w/IEMGUI_ZOOM(x) - 1);
     else
-        x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);
+        x->x_k = (x->x_max - x->x_min) / (double)(x->x_gui.x_w/IEMGUI_ZOOM(x) - 1);
 }
 
 static void hslider_properties(t_gobj *z, t_glist *owner)
@@ -278,7 +298,7 @@ static void hslider_properties(t_gobj *z, t_glist *owner)
             %s %d %d \
             %d %d \
             #%06x #%06x #%06x\n",
-            x->x_gui.x_w, IEM_SL_MINSIZE, x->x_gui.x_h, IEM_GUI_MINSIZE,
+            x->x_gui.x_w/IEMGUI_ZOOM(x), IEM_SL_MINSIZE, x->x_gui.x_h/IEMGUI_ZOOM(x), IEM_GUI_MINSIZE,
             x->x_min, x->x_max, 0.0,/*no_schedule*/
             x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, x->x_steady, -1,/*no multi, but iem-characteristic*/
             srl[0]->s_name, srl[1]->s_name,
@@ -309,10 +329,10 @@ static void hslider_set(t_hslider *x, t_floatarg f)    /* bugfix */
             f = x->x_min;
     }
     if(x->x_lin0_log1)
-        g = log(f/x->x_min)/x->x_k;
+        g = log(f/x->x_min) / x->x_k;
     else
         g = (f - x->x_min) / x->x_k;
-    x->x_val = (int)(100.0*g + 0.49999);
+    x->x_val = IEMGUI_ZOOM(x) * (int)(100.0*g + 0.49999);
     x->x_pos = x->x_val;
     if(x->x_val != old)
         (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
@@ -322,9 +342,12 @@ static void hslider_set(t_hslider *x, t_floatarg f)    /* bugfix */
 static t_float hslider_getfval(t_hslider *x)
 {
     t_float fval;
+    int zoomval = (x->x_gui.x_fsf.x_finemoved) ?
+        x->x_val/IEMGUI_ZOOM(x) : (x->x_val / (100*IEMGUI_ZOOM(x))) * 100;
+
     if (x->x_lin0_log1)
-        fval = x->x_min*exp(x->x_k*(double)(x->x_val)*0.01);
-    else fval = (double)(x->x_val)*0.01*x->x_k + x->x_min;
+        fval = x->x_min * exp(x->x_k * (double)(zoomval) * 0.01);
+    else fval = (double)(zoomval) * 0.01 * x->x_k + x->x_min;
     if ((fval < 1.0e-10) && (fval > -1.0e-10))
         fval = 0.0;
     return (fval);
@@ -345,12 +368,12 @@ static void hslider_bang(t_hslider *x)
 static void hslider_dialog(t_hslider *x, t_symbol *s, int argc, t_atom *argv)
 {
     t_symbol *srl[3];
-    int w = (int)atom_getintarg(0, argc, argv);
-    int h = (int)atom_getintarg(1, argc, argv);
+    int w = (int)atom_getfloatarg(0, argc, argv) * IEMGUI_ZOOM(x);
+    int h = (int)atom_getfloatarg(1, argc, argv);
     double min = (double)atom_getfloatarg(2, argc, argv);
     double max = (double)atom_getfloatarg(3, argc, argv);
-    int lilo = (int)atom_getintarg(4, argc, argv);
-    int steady = (int)atom_getintarg(17, argc, argv);
+    int lilo = (int)atom_getfloatarg(4, argc, argv);
+    int steady = (int)atom_getfloatarg(17, argc, argv);
     int sr_flags;
 
     if(lilo != 0) lilo = 1;
@@ -360,7 +383,7 @@ static void hslider_dialog(t_hslider *x, t_symbol *s, int argc, t_atom *argv)
     else
         x->x_steady = 0;
     sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
-    x->x_gui.x_h = iemgui_clip_size(h);
+    x->x_gui.x_h = iemgui_clip_size(h) * IEMGUI_ZOOM(x);
     hslider_check_width(x, w);
     hslider_check_minmax(x, min, max);
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
@@ -376,19 +399,19 @@ static void hslider_motion(t_hslider *x, t_floatarg dx, t_floatarg dy)
     if(x->x_gui.x_fsf.x_finemoved)
         x->x_pos += (int)dx;
     else
-        x->x_pos += 100*(int)dx;
+        x->x_pos += 100 * (int)dx;
     x->x_val = x->x_pos;
     if(x->x_val > (100*x->x_gui.x_w - 100))
     {
         x->x_val = 100*x->x_gui.x_w - 100;
         x->x_pos += 50;
-        x->x_pos -= x->x_pos%100;
+        x->x_pos -= x->x_pos % 100;
     }
     if(x->x_val < 0)
     {
         x->x_val = 0;
         x->x_pos -= 50;
-        x->x_pos -= x->x_pos%100;
+        x->x_pos -= x->x_pos % 100;
     }
     x->x_fval = hslider_getfval(x);
     if (old != x->x_val)
@@ -422,8 +445,8 @@ static int hslider_newclick(t_gobj *z, struct _glist *glist,
 
     if(doit)
     {
-        hslider_click( x, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift,
-                       0, (t_floatarg)alt);
+        hslider_click(x, (t_floatarg)xpix, (t_floatarg)ypix, (t_floatarg)shift,
+                      0, (t_floatarg)alt);
         if(shift)
             x->x_gui.x_fsf.x_finemoved = 1;
         else
@@ -434,9 +457,9 @@ static int hslider_newclick(t_gobj *z, struct _glist *glist,
 
 static void hslider_size(t_hslider *x, t_symbol *s, int ac, t_atom *av)
 {
-    hslider_check_width(x, (int)atom_getintarg(0, ac, av));
+    hslider_check_width(x, (int)atom_getfloatarg(0, ac, av)*IEMGUI_ZOOM(x));
     if(ac > 1)
-        x->x_gui.x_h = iemgui_clip_size((int)atom_getintarg(1, ac, av));
+        x->x_gui.x_h = iemgui_clip_size((int)atom_getfloatarg(1, ac, av))*IEMGUI_ZOOM(x);
     iemgui_size((void *)x, &x->x_gui);
 }
 
@@ -449,7 +472,7 @@ static void hslider_pos(t_hslider *x, t_symbol *s, int ac, t_atom *av)
 static void hslider_range(t_hslider *x, t_symbol *s, int ac, t_atom *av)
 {
     hslider_check_minmax(x, (double)atom_getfloatarg(0, ac, av),
-                         (double)atom_getfloatarg(1, ac, av));
+                            (double)atom_getfloatarg(1, ac, av));
 }
 
 static void hslider_color(t_hslider *x, t_symbol *s, int ac, t_atom *av)
@@ -479,23 +502,29 @@ static void hslider_log(t_hslider *x)
 static void hslider_lin(t_hslider *x)
 {
     x->x_lin0_log1 = 0;
-    x->x_k = (x->x_max - x->x_min)/(double)(x->x_gui.x_w - 1);
+    x->x_k = (x->x_max - x->x_min) / (double)(x->x_gui.x_w/IEMGUI_ZOOM(x) - 1);
 }
 
 static void hslider_init(t_hslider *x, t_floatarg f)
 {
-    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
+    x->x_gui.x_isa.x_loadinit = (f == 0.0) ? 0 : 1;
 }
 
 static void hslider_steady(t_hslider *x, t_floatarg f)
 {
-    x->x_steady = (f==0.0)?0:1;
+    x->x_steady = (f == 0.0) ? 0 : 1;
+}
+
+static void hslider_zoom(t_hslider *x, t_floatarg f)
+{
+    /* scale current pixel value */
+    x->x_val = (IEMGUI_ZOOM(x) == 2 ? (x->x_val)/2 : (x->x_val)*2);
+    x->x_pos = x->x_val;
+    iemgui_zoom(&x->x_gui, f);
 }
 
 static void hslider_float(t_hslider *x, t_floatarg f)
 {
-    double out;
-
     hslider_set(x, f);
     if(x->x_gui.x_fsf.x_put_in2out)
         hslider_bang(x);
@@ -513,10 +542,10 @@ static void hslider_loadbang(t_hslider *x, t_floatarg action)
 static void *hslider_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_hslider *x = (t_hslider *)pd_new(hslider_class);
-    int w=IEM_SL_DEFAULTSIZE, h=IEM_GUI_DEFAULTSIZE;
-    int lilo=0, ldx=-2, ldy=-8, f=0, steady=1;
-    int fs=10;
-    double min=0.0, max=(double)(IEM_SL_DEFAULTSIZE-1);
+    int w = IEM_SL_DEFAULTSIZE, h = IEM_GUI_DEFAULTSIZE;
+    int lilo = 0, ldx = -2, ldy = -8, f = 0, steady = 1;
+    int fs = 10;
+    double min = 0.0, max = (double)(IEM_SL_DEFAULTSIZE-1);
     char str[144];
     float v = 0;
 
@@ -536,29 +565,26 @@ static void *hslider_new(t_symbol *s, int argc, t_atom *argv)
        &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)
        &&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,16))
     {
-        w = (int)atom_getintarg(0, argc, argv);
-        h = (int)atom_getintarg(1, argc, argv);
+        w = (int)atom_getfloatarg(0, argc, argv);
+        h = (int)atom_getfloatarg(1, argc, argv);
         min = (double)atom_getfloatarg(2, argc, argv);
         max = (double)atom_getfloatarg(3, argc, argv);
-        lilo = (int)atom_getintarg(4, argc, argv);
-        iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(5, argc, argv));
+        lilo = (int)atom_getfloatarg(4, argc, argv);
+        iem_inttosymargs(&x->x_gui.x_isa, atom_getfloatarg(5, argc, argv));
         iemgui_new_getnames(&x->x_gui, 6, argv);
-        ldx = (int)atom_getintarg(9, argc, argv);
-        ldy = (int)atom_getintarg(10, argc, argv);
-        iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(11, argc, argv));
-        fs = (int)atom_getintarg(12, argc, argv);
+        ldx = (int)atom_getfloatarg(9, argc, argv);
+        ldy = (int)atom_getfloatarg(10, argc, argv);
+        iem_inttofstyle(&x->x_gui.x_fsf, atom_getfloatarg(11, argc, argv));
+        fs = (int)atom_getfloatarg(12, argc, argv);
         iemgui_all_loadcolors(&x->x_gui, argv+13, argv+14, argv+15);
         v = atom_getfloatarg(16, argc, argv);
     }
     else iemgui_new_getnames(&x->x_gui, 6, 0);
     if((argc == 18)&&IS_A_FLOAT(argv,17))
-        steady = (int)atom_getintarg(17, argc, argv);
-
+        steady = (int)atom_getfloatarg(17, argc, argv);
     x->x_gui.x_draw = (t_iemfunptr)hslider_draw;
-
     x->x_gui.x_fsf.x_snd_able = 1;
     x->x_gui.x_fsf.x_rcv_able = 1;
-
     x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
     if (x->x_gui.x_isa.x_loadinit)
         x->x_val = v;
@@ -585,8 +611,9 @@ static void *hslider_new(t_symbol *s, int argc, t_atom *argv)
     x->x_gui.x_fontsize = fs;
     x->x_gui.x_h = iemgui_clip_size(h);
     hslider_check_width(x, w);
-    hslider_check_minmax(x, min, max);
     iemgui_verify_snd_ne_rcv(&x->x_gui);
+    iemgui_newzoom(&x->x_gui);
+    hslider_check_minmax(x, min, max);
     outlet_new(&x->x_gui.x_obj, &s_float);
     x->x_fval = hslider_getfval(x);
     return (x);
@@ -602,36 +629,52 @@ static void hslider_free(t_hslider *x)
 void g_hslider_setup(void)
 {
     hslider_class = class_new(gensym("hsl"), (t_newmethod)hslider_new,
-                              (t_method)hslider_free, sizeof(t_hslider), 0, A_GIMME, 0);
+        (t_method)hslider_free, sizeof(t_hslider), 0, A_GIMME, 0);
 #ifndef GGEE_HSLIDER_COMPATIBLE
     class_addcreator((t_newmethod)hslider_new, gensym("hslider"), A_GIMME, 0);
 #endif
-    class_addbang(hslider_class,hslider_bang);
-    class_addfloat(hslider_class,hslider_float);
-    class_addmethod(hslider_class, (t_method)hslider_click, gensym("click"),
-                    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
-    class_addmethod(hslider_class, (t_method)hslider_motion, gensym("motion"),
-                    A_FLOAT, A_FLOAT, 0);
-    class_addmethod(hslider_class, (t_method)hslider_dialog, gensym("dialog"), A_GIMME, 0);
+    class_addbang(hslider_class, hslider_bang);
+    class_addfloat(hslider_class, hslider_float);
+    class_addmethod(hslider_class, (t_method)hslider_click,
+        gensym("click"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
+    class_addmethod(hslider_class, (t_method)hslider_motion,
+        gensym("motion"), A_FLOAT, A_FLOAT, 0);
+    class_addmethod(hslider_class, (t_method)hslider_dialog,
+        gensym("dialog"), A_GIMME, 0);
     class_addmethod(hslider_class, (t_method)hslider_loadbang,
         gensym("loadbang"), A_DEFFLOAT, 0);
-    class_addmethod(hslider_class, (t_method)hslider_set, gensym("set"), A_FLOAT, 0);
-    class_addmethod(hslider_class, (t_method)hslider_size, gensym("size"), A_GIMME, 0);
-    class_addmethod(hslider_class, (t_method)hslider_delta, gensym("delta"), A_GIMME, 0);
-    class_addmethod(hslider_class, (t_method)hslider_pos, gensym("pos"), A_GIMME, 0);
-    class_addmethod(hslider_class, (t_method)hslider_range, gensym("range"), A_GIMME, 0);
-    class_addmethod(hslider_class, (t_method)hslider_color, gensym("color"), A_GIMME, 0);
-    class_addmethod(hslider_class, (t_method)hslider_send, gensym("send"), A_DEFSYM, 0);
-    class_addmethod(hslider_class, (t_method)hslider_receive, gensym("receive"), A_DEFSYM, 0);
-    class_addmethod(hslider_class, (t_method)hslider_label, gensym("label"), A_DEFSYM, 0);
-    class_addmethod(hslider_class, (t_method)hslider_label_pos, gensym("label_pos"), A_GIMME, 0);
-    class_addmethod(hslider_class, (t_method)hslider_label_font, gensym("label_font"), A_GIMME, 0);
-    class_addmethod(hslider_class, (t_method)hslider_log, gensym("log"), 0);
-    class_addmethod(hslider_class, (t_method)hslider_lin, gensym("lin"), 0);
-    class_addmethod(hslider_class, (t_method)hslider_init, gensym("init"), A_FLOAT, 0);
-    class_addmethod(hslider_class, (t_method)hslider_steady, gensym("steady"), A_FLOAT, 0);
-    class_addmethod(hslider_class, (t_method)iemgui_zoom, gensym("zoom"),
-        A_CANT, 0);
+    class_addmethod(hslider_class, (t_method)hslider_set,
+        gensym("set"), A_FLOAT, 0);
+    class_addmethod(hslider_class, (t_method)hslider_size,
+        gensym("size"), A_GIMME, 0);
+    class_addmethod(hslider_class, (t_method)hslider_delta,
+        gensym("delta"), A_GIMME, 0);
+    class_addmethod(hslider_class, (t_method)hslider_pos,
+        gensym("pos"), A_GIMME, 0);
+    class_addmethod(hslider_class, (t_method)hslider_range,
+        gensym("range"), A_GIMME, 0);
+    class_addmethod(hslider_class, (t_method)hslider_color,
+        gensym("color"), A_GIMME, 0);
+    class_addmethod(hslider_class, (t_method)hslider_send,
+        gensym("send"), A_DEFSYM, 0);
+    class_addmethod(hslider_class, (t_method)hslider_receive,
+        gensym("receive"), A_DEFSYM, 0);
+    class_addmethod(hslider_class, (t_method)hslider_label,
+        gensym("label"), A_DEFSYM, 0);
+    class_addmethod(hslider_class, (t_method)hslider_label_pos,
+        gensym("label_pos"), A_GIMME, 0);
+    class_addmethod(hslider_class, (t_method)hslider_label_font,
+        gensym("label_font"), A_GIMME, 0);
+    class_addmethod(hslider_class, (t_method)hslider_log,
+        gensym("log"), 0);
+    class_addmethod(hslider_class, (t_method)hslider_lin,
+        gensym("lin"), 0);
+    class_addmethod(hslider_class, (t_method)hslider_init,
+        gensym("init"), A_FLOAT, 0);
+    class_addmethod(hslider_class, (t_method)hslider_steady,
+        gensym("steady"), A_FLOAT, 0);
+    class_addmethod(hslider_class, (t_method)hslider_zoom,
+        gensym("zoom"), A_CANT, 0);
     hslider_widgetbehavior.w_getrectfn =    hslider_getrect;
     hslider_widgetbehavior.w_displacefn =   iemgui_displace;
     hslider_widgetbehavior.w_selectfn =     iemgui_select;
