@@ -20,15 +20,15 @@
 #include <unistd.h>
 #endif
 
-/*------------------ global varaibles -------------------------*/
-
+#define MINDIGITS 1
+#define MINFONT   4
 
 /*------------------ global functions -------------------------*/
 
 static void my_numbox_key(void *z, t_floatarg fkey);
 static void my_numbox_draw_update(t_gobj *client, t_glist *glist);
 
-/* ------------ nmx gui-my number box ----------------------- */
+/* ------------ nbx gui-my number box ----------------------- */
 
 t_widgetbehavior my_numbox_widgetbehavior;
 static t_class *my_numbox_class;
@@ -59,22 +59,22 @@ void my_numbox_clip(t_my_numbox *x)
 
 void my_numbox_calc_fontwidth(t_my_numbox *x)
 {
-    int w, f=31;
+    int w, f = 31;
 
     if(x->x_gui.x_fsf.x_font_style == 1)
         f = 27;
     else if(x->x_gui.x_fsf.x_font_style == 2)
         f = 25;
 
-    w = x->x_gui.x_fontsize * f * x->x_gui.x_w;
+    w = x->x_gui.x_fontsize * f * x->x_numwidth;
     w /= 36;
-    x->x_numwidth = w + (x->x_gui.x_h / 2) + 4;
+    x->x_gui.x_w = (w + (x->x_gui.x_h/2)/IEMGUI_ZOOM(x) + 4) * IEMGUI_ZOOM(x);
 }
 
 void my_numbox_ftoa(t_my_numbox *x)
 {
-    double f=x->x_val;
-    int bufsize, is_exp=0, i, idecimal;
+    double f = x->x_val;
+    int bufsize, is_exp = 0, i, idecimal;
 
     sprintf(x->x_buf, "%g", f);
     bufsize = strlen(x->x_buf);
@@ -84,46 +84,46 @@ void my_numbox_ftoa(t_my_numbox *x)
         if((x->x_buf[i] == 'e') || (x->x_buf[i] == 'E'))
             is_exp = 1;
     }
-    if(bufsize > x->x_gui.x_w)/* if to reduce */
+    if(bufsize > x->x_numwidth)/* if to reduce */
     {
         if(is_exp)
         {
-            if(x->x_gui.x_w <= 5)
+            if(x->x_numwidth <= 5)
             {
                 x->x_buf[0] = (f < 0.0 ? '-' : '+');
                 x->x_buf[1] = 0;
             }
             i = bufsize - 4;
-            for(idecimal=0; idecimal < i; idecimal++)
+            for(idecimal = 0; idecimal < i; idecimal++)
                 if(x->x_buf[idecimal] == '.')
                     break;
-            if(idecimal > (x->x_gui.x_w - 4))
+            if(idecimal > (x->x_numwidth - 4))
             {
                 x->x_buf[0] = (f < 0.0 ? '-' : '+');
                 x->x_buf[1] = 0;
             }
             else
             {
-                int new_exp_index=x->x_gui.x_w-4, old_exp_index=bufsize-4;
+                int new_exp_index = x->x_numwidth - 4;
+                int old_exp_index = bufsize - 4;
 
-                for(i=0; i < 4; i++, new_exp_index++, old_exp_index++)
+                for(i = 0; i < 4; i++, new_exp_index++, old_exp_index++)
                     x->x_buf[new_exp_index] = x->x_buf[old_exp_index];
-                x->x_buf[x->x_gui.x_w] = 0;
+                x->x_buf[x->x_numwidth] = 0;
             }
-
         }
         else
         {
-            for(idecimal=0; idecimal < bufsize; idecimal++)
+            for(idecimal = 0; idecimal < bufsize; idecimal++)
                 if(x->x_buf[idecimal] == '.')
                     break;
-            if(idecimal > x->x_gui.x_w)
+            if(idecimal > x->x_numwidth)
             {
                 x->x_buf[0] = (f < 0.0 ? '-' : '+');
                 x->x_buf[1] = 0;
             }
             else
-                x->x_buf[x->x_gui.x_w] = 0;
+                x->x_buf[x->x_numwidth] = 0;
         }
     }
 }
@@ -131,42 +131,40 @@ void my_numbox_ftoa(t_my_numbox *x)
 static void my_numbox_draw_update(t_gobj *client, t_glist *glist)
 {
     t_my_numbox *x = (t_my_numbox *)client;
-    if (glist_isvisible(glist))
+    if(glist_isvisible(glist))
     {
         if(x->x_gui.x_fsf.x_change)
         {
             if(x->x_buf[0])
             {
-                char *cp=x->x_buf;
+                char *cp = x->x_buf;
                 int sl = strlen(x->x_buf);
 
                 x->x_buf[sl] = '>';
                 x->x_buf[sl+1] = 0;
-                if(sl >= x->x_gui.x_w)
-                    cp += sl - x->x_gui.x_w + 1;
-                sys_vgui(
-                    ".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
-                         glist_getcanvas(glist), x, IEM_GUI_COLOR_EDITED, cp);
+                if(sl >= x->x_numwidth)
+                    cp += sl - x->x_numwidth + 1;
+                sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
+                         glist_getcanvas(glist), x,
+                         IEM_GUI_COLOR_EDITED, cp);
                 x->x_buf[sl] = 0;
             }
             else
             {
                 my_numbox_ftoa(x);
-                sys_vgui(
-                    ".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
-                    glist_getcanvas(glist), x, IEM_GUI_COLOR_EDITED, x->x_buf);
+                sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
+                         glist_getcanvas(glist), x,
+                         IEM_GUI_COLOR_EDITED, x->x_buf);
                 x->x_buf[0] = 0;
             }
         }
         else
         {
             my_numbox_ftoa(x);
-            sys_vgui(
-                ".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
-                glist_getcanvas(glist), x,
-                x->x_gui.x_fsf.x_selected?
-                    IEM_GUI_COLOR_SELECTED:x->x_gui.x_fcol,
-                x->x_buf);
+            sys_vgui(".x%lx.c itemconfigure %lxNUMBER -fill #%06x -text {%s} \n",
+                     glist_getcanvas(glist), x,
+                     (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_fcol),
+                     x->x_buf);
             x->x_buf[0] = 0;
         }
     }
@@ -174,89 +172,99 @@ static void my_numbox_draw_update(t_gobj *client, t_glist *glist)
 
 static void my_numbox_draw_new(t_my_numbox *x, t_glist *glist)
 {
-    int half=x->x_gui.x_h/2, d=1+x->x_gui.x_h/34;
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int w = x->x_gui.x_w, half = x->x_gui.x_h/2;
+    int d = IEMGUI_ZOOM(x) + x->x_gui.x_h/(34*IEMGUI_ZOOM(x));
+    int corner = x->x_gui.x_h/4;
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
-    sys_vgui(
-".x%lx.c create polygon %d %d %d %d %d %d %d %d %d %d -outline #%06x \
--fill #%06x -tags %lxBASE1\n",
-             canvas, xpos, ypos,
-             xpos + x->x_numwidth-4, ypos,
-             xpos + x->x_numwidth, ypos+4,
-             xpos + x->x_numwidth, ypos + x->x_gui.x_h,
-             xpos, ypos + x->x_gui.x_h,
-             IEM_GUI_COLOR_NORMAL, x->x_gui.x_bcol, x);
-    sys_vgui(
-        ".x%lx.c create line %d %d %d %d %d %d -fill #%06x -tags %lxBASE2\n",
-        canvas, xpos, ypos,
-        xpos + half, ypos + half,
-        xpos, ypos + x->x_gui.x_h,
-        x->x_gui.x_fcol, x);
-    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-        -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
-        canvas, xpos+x->x_gui.x_ldx,
-        ypos+x->x_gui.x_ldy,
-        strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
-        x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_lcol, x);
-    my_numbox_ftoa(x);
-    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-        -font {{%s} -%d %s} -fill #%06x -tags %lxNUMBER\n",
-        canvas, xpos+half+2, ypos+half+d,
-        x->x_buf, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-        x->x_gui.x_fcol, x);
-    if(!x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxOUT%d outlet]\n",
-             canvas,
-             xpos, ypos + x->x_gui.x_h-1,
-             xpos+IOWIDTH, ypos + x->x_gui.x_h,
-             x, 0);
-    if(!x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags [list %lxIN%d inlet]\n",
+    sys_vgui(".x%lx.c create polygon %d %d %d %d %d %d %d %d %d %d %d %d "
+             "-width %d -outline #%06x -fill #%06x -tags %lxBASE1\n",
              canvas,
              xpos, ypos,
-             xpos+IOWIDTH, ypos+1,
+             xpos + w - corner, ypos,
+             xpos + w, ypos + corner,
+             xpos + w, ypos + x->x_gui.x_h,
+             xpos, ypos + x->x_gui.x_h,
+             xpos, ypos,
+             IEMGUI_ZOOM(x), IEM_GUI_COLOR_NORMAL, x->x_gui.x_bcol, x);
+    sys_vgui(".x%lx.c create line %d %d %d %d %d %d -width %d -fill #%06x -tags %lxBASE2\n",
+             canvas,
+             xpos + IEMGUI_ZOOM(x), ypos + IEMGUI_ZOOM(x),
+             xpos + half, ypos + half,
+             xpos + IEMGUI_ZOOM(x), ypos + x->x_gui.x_h - IEMGUI_ZOOM(x),
+             IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
+    if(!x->x_gui.x_fsf.x_snd_able)
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]\n",
+             canvas,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h,
              x, 0);
+    if(!x->x_gui.x_fsf.x_rcv_able)
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
+             canvas,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
+    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
+             canvas, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
+             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             x->x_gui.x_lcol, x);
+    my_numbox_ftoa(x);
+    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w -font {{%s} -%d %s} -fill #%06x -tags %lxNUMBER\n",
+             canvas, xpos + half + 2*IEMGUI_ZOOM(x), ypos + half + d,
+             x->x_buf, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x),
+             sys_fontweight, (x->x_gui.x_fsf.x_change ? IEM_GUI_COLOR_EDITED : x->x_gui.x_fcol), x);
 }
 
 static void my_numbox_draw_move(t_my_numbox *x, t_glist *glist)
 {
-    int half = x->x_gui.x_h/2, d=1+x->x_gui.x_h/34;
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int w = x->x_gui.x_w, half = x->x_gui.x_h/2;
+    int d = IEMGUI_ZOOM(x) + x->x_gui.x_h / (34 * IEMGUI_ZOOM(x));
+    int corner = x->x_gui.x_h/4;
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
-    sys_vgui(".x%lx.c coords %lxBASE1 %d %d %d %d %d %d %d %d %d %d\n",
-             canvas, x, xpos, ypos,
-             xpos + x->x_numwidth-4, ypos,
-             xpos + x->x_numwidth, ypos+4,
-             xpos + x->x_numwidth, ypos + x->x_gui.x_h,
-             xpos, ypos + x->x_gui.x_h);
+    sys_vgui(".x%lx.c coords %lxBASE1 %d %d %d %d %d %d %d %d %d %d %d %d\n",
+             canvas, x,
+             xpos, ypos,
+             xpos + w - corner, ypos,
+             xpos + w, ypos + corner,
+             xpos + w, ypos + x->x_gui.x_h,
+             xpos, ypos + x->x_gui.x_h,
+             xpos, ypos);
     sys_vgui(".x%lx.c coords %lxBASE2 %d %d %d %d %d %d\n",
-             canvas, x, xpos, ypos,
+             canvas, x,
+             xpos + IEMGUI_ZOOM(x), ypos + IEMGUI_ZOOM(x),
              xpos + half, ypos + half,
-             xpos, ypos + x->x_gui.x_h);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xpos+x->x_gui.x_ldx, ypos+x->x_gui.x_ldy);
-    sys_vgui(".x%lx.c coords %lxNUMBER %d %d\n",
-             canvas, x, xpos+half+2, ypos+half+d);
-   if(!x->x_gui.x_fsf.x_snd_able)
+             xpos + IEMGUI_ZOOM(x), ypos + x->x_gui.x_h - IEMGUI_ZOOM(x));
+    if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
              canvas, x, 0,
-             xpos, ypos + x->x_gui.x_h-1,
-             xpos+IOWIDTH, ypos + x->x_gui.x_h);
-   if(!x->x_gui.x_fsf.x_rcv_able)
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h);
+    if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
              canvas, x, 0,
              xpos, ypos,
-             xpos+IOWIDTH, ypos+1);
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh);
+    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
+             canvas, x,
+             xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x));
+    sys_vgui(".x%lx.c coords %lxNUMBER %d %d\n",
+             canvas, x, xpos + half + 2*IEMGUI_ZOOM(x), ypos + half + d);
 }
 
-static void my_numbox_draw_erase(t_my_numbox* x,t_glist* glist)
+static void my_numbox_draw_erase(t_my_numbox* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c delete %lxBASE1\n", canvas, x);
     sys_vgui(".x%lx.c delete %lxBASE2\n", canvas, x);
@@ -268,42 +276,43 @@ static void my_numbox_draw_erase(t_my_numbox* x,t_glist* glist)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
 }
 
-static void my_numbox_draw_config(t_my_numbox* x,t_glist* glist)
+static void my_numbox_draw_config(t_my_numbox* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
+             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_lcol),
+             strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name:"");
     sys_vgui(".x%lx.c itemconfigure %lxNUMBER -font {{%s} -%d %s} -fill #%06x \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_fcol);
+             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_fcol));
     sys_vgui(".x%lx.c itemconfigure %lxBASE1 -fill #%06x\n", canvas,
              x, x->x_gui.x_bcol);
-    sys_vgui(".x%lx.c itemconfigure %lxBASE2 -fill #%06x\n", canvas,
-             x, x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_fcol);
+    sys_vgui(".x%lx.c itemconfigure %lxBASE2 -fill #%06x\n", canvas, x,
+             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_fcol));
 }
 
 static void my_numbox_draw_io(t_my_numbox* x,t_glist* glist, int old_snd_rcv_flags)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxOUT%d\n",
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
              canvas,
-             xpos, ypos + x->x_gui.x_h-1,
-             xpos+IOWIDTH, ypos + x->x_gui.x_h,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h,
              x, 0);
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
     if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxIN%d\n",
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
              canvas,
              xpos, ypos,
-             xpos+IOWIDTH, ypos+1,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
              x, 0);
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
@@ -311,7 +320,7 @@ static void my_numbox_draw_io(t_my_numbox* x,t_glist* glist, int old_snd_rcv_fla
 
 static void my_numbox_draw_select(t_my_numbox *x, t_glist *glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if(x->x_gui.x_fsf.x_selected)
     {
@@ -364,7 +373,6 @@ void my_numbox_draw(t_my_numbox *x, t_glist *glist, int mode)
 
 /* ------------------------ nbx widgetbehaviour----------------------------- */
 
-
 static void my_numbox_getrect(t_gobj *z, t_glist *glist,
                               int *xp1, int *yp1, int *xp2, int *yp2)
 {
@@ -372,7 +380,7 @@ static void my_numbox_getrect(t_gobj *z, t_glist *glist,
 
     *xp1 = text_xpix(&x->x_gui.x_obj, glist);
     *yp1 = text_ypix(&x->x_gui.x_obj, glist);
-    *xp2 = *xp1 + x->x_numwidth;
+    *xp2 = *xp1 + x->x_gui.x_w;
     *yp2 = *yp1 + x->x_gui.x_h;
 }
 
@@ -389,9 +397,9 @@ static void my_numbox_save(t_gobj *z, t_binbuf *b)
         clock_unset(x->x_clock_reset);
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
     }
-    binbuf_addv(b, "ssiisiiffiisssiiiisssfi", gensym("#X"),gensym("obj"),
+    binbuf_addv(b, "ssiisiiffiisssiiiisssfi", gensym("#X"), gensym("obj"),
                 (int)x->x_gui.x_obj.te_xpix, (int)x->x_gui.x_obj.te_ypix,
-                gensym("nbx"), x->x_gui.x_w, x->x_gui.x_h,
+                gensym("nbx"), x->x_numwidth, x->x_gui.x_h/IEMGUI_ZOOM(x),
                 (t_float)x->x_min, (t_float)x->x_max,
                 x->x_lin0_log1, iem_symargstoint(&x->x_gui.x_isa),
                 srl[0], srl[1], srl[2],
@@ -404,21 +412,21 @@ static void my_numbox_save(t_gobj *z, t_binbuf *b)
 
 int my_numbox_check_minmax(t_my_numbox *x, double min, double max)
 {
-    int ret=0;
+    int ret = 0;
 
     if(x->x_lin0_log1)
     {
-        if((min == 0.0)&&(max == 0.0))
+        if((min == 0.0) && (max == 0.0))
             max = 1.0;
         if(max > 0.0)
         {
             if(min <= 0.0)
-                min = 0.01*max;
+                min = 0.01 * max;
         }
         else
         {
             if(min > 0.0)
-                max = 0.01*min;
+                max = 0.01 * min;
         }
     }
     x->x_min = min;
@@ -434,7 +442,7 @@ int my_numbox_check_minmax(t_my_numbox *x, double min, double max)
         ret = 1;
     }
     if(x->x_lin0_log1)
-        x->x_k = exp(log(x->x_max/x->x_min)/(double)(x->x_log_height));
+        x->x_k = exp(log(x->x_max/x->x_min) / (double)(x->x_log_height));
     else
         x->x_k = 1.0;
     return(ret);
@@ -452,7 +460,6 @@ static void my_numbox_properties(t_gobj *z, t_glist *owner)
         x->x_gui.x_fsf.x_change = 0;
         clock_unset(x->x_clock_reset);
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
-
     }
     sprintf(buf, "pdtk_iemgui_dialog %%s |nbx| \
             -------dimensions(digits)(pix):------- %d %d width: %d %d height: \
@@ -462,7 +469,7 @@ static void my_numbox_properties(t_gobj *z, t_glist *owner)
             %s %d %d \
             %d %d \
             #%06x #%06x #%06x\n",
-            x->x_gui.x_w, 1, x->x_gui.x_h, 8,
+            x->x_numwidth, MINDIGITS, x->x_gui.x_h/IEMGUI_ZOOM(x), IEM_GUI_MINSIZE,
             x->x_min, x->x_max, 0,/*no_schedule*/
             x->x_lin0_log1, x->x_gui.x_isa.x_loadinit, -1,
                 x->x_log_height, /*no multi, but iem-characteristic*/
@@ -485,23 +492,23 @@ static void my_numbox_dialog(t_my_numbox *x, t_symbol *s, int argc,
     t_atom *argv)
 {
     t_symbol *srl[3];
-    int w = (int)atom_getintarg(0, argc, argv);
-    int h = (int)atom_getintarg(1, argc, argv);
+    int w = (int)atom_getfloatarg(0, argc, argv);
+    int h = (int)atom_getfloatarg(1, argc, argv);
     double min = (double)atom_getfloatarg(2, argc, argv);
     double max = (double)atom_getfloatarg(3, argc, argv);
-    int lilo = (int)atom_getintarg(4, argc, argv);
-    int log_height = (int)atom_getintarg(6, argc, argv);
+    int lilo = (int)atom_getfloatarg(4, argc, argv);
+    int log_height = (int)atom_getfloatarg(6, argc, argv);
     int sr_flags;
 
     if(lilo != 0) lilo = 1;
     x->x_lin0_log1 = lilo;
     sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
-    if(w < 1)
-        w = 1;
-    x->x_gui.x_w = w;
-    if(h < 8)
-        h = 8;
-    x->x_gui.x_h = h;
+    if(w < MINDIGITS)
+        w = MINDIGITS;
+    x->x_numwidth = w;
+    if(h < IEM_GUI_MINSIZE)
+        h = IEM_GUI_MINSIZE;
+    x->x_gui.x_h = h * IEMGUI_ZOOM(x);
     if(log_height < 10)
         log_height = 10;
     x->x_log_height = log_height;
@@ -518,7 +525,7 @@ static void my_numbox_dialog(t_my_numbox *x, t_symbol *s, int argc,
 
 static void my_numbox_motion(t_my_numbox *x, t_floatarg dx, t_floatarg dy)
 {
-    double k2=1.0;
+    double k2 = 1.0;
 
     if(x->x_gui.x_fsf.x_finemoved)
         k2 = 0.01;
@@ -590,7 +597,6 @@ static void my_numbox_log_height(t_my_numbox *x, t_floatarg lh)
         x->x_k = exp(log(x->x_max/x->x_min)/(double)(x->x_log_height));
     else
         x->x_k = 1.0;
-
 }
 
 static void my_numbox_float(t_my_numbox *x, t_floatarg f)
@@ -604,16 +610,16 @@ static void my_numbox_size(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
 {
     int h, w;
 
-    w = (int)atom_getintarg(0, ac, av);
-    if(w < 1)
-        w = 1;
-    x->x_gui.x_w = w;
+    w = (int)atom_getfloatarg(0, ac, av);
+    if(w < MINDIGITS)
+        w = MINDIGITS;
+    x->x_numwidth = w;
     if(ac > 1)
     {
-        h = (int)atom_getintarg(1, ac, av);
-        if(h < 8)
-            h = 8;
-        x->x_gui.x_h = h;
+        h = (int)atom_getfloatarg(1, ac, av);
+        if(h < IEM_GUI_MINSIZE)
+            h = IEM_GUI_MINSIZE;
+        x->x_gui.x_h = h * IEMGUI_ZOOM(x);
     }
     my_numbox_calc_fontwidth(x);
     iemgui_size((void *)x, &x->x_gui);
@@ -628,7 +634,7 @@ static void my_numbox_pos(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
 static void my_numbox_range(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
 {
     if(my_numbox_check_minmax(x, (double)atom_getfloatarg(0, ac, av),
-                              (double)atom_getfloatarg(1, ac, av)))
+                                 (double)atom_getfloatarg(1, ac, av)))
     {
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
         /*my_numbox_bang(x);*/
@@ -653,12 +659,12 @@ static void my_numbox_label_pos(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
 static void my_numbox_label_font(t_my_numbox *x,
     t_symbol *s, int ac, t_atom *av)
 {
-    int f = (int)atom_getintarg(1, ac, av);
+    int f = (int)atom_getfloatarg(1, ac, av);
 
     if(f < 4)
         f = 4;
     x->x_gui.x_fontsize = f;
-    f = (int)atom_getintarg(0, ac, av);
+    f = (int)atom_getfloatarg(0, ac, av);
     if((f < 0) || (f > 2))
         f = 0;
     x->x_gui.x_fsf.x_font_style = f;
@@ -683,12 +689,12 @@ static void my_numbox_lin(t_my_numbox *x)
 
 static void my_numbox_init(t_my_numbox *x, t_floatarg f)
 {
-    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
+    x->x_gui.x_isa.x_loadinit = (f == 0.0) ? 0 : 1;
 }
 
 static void my_numbox_loadbang(t_my_numbox *x, t_floatarg action)
 {
-    if (action == LB_LOAD && x->x_gui.x_isa.x_loadinit)
+    if(action == LB_LOAD && x->x_gui.x_isa.x_loadinit)
     {
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
         my_numbox_bang(x);
@@ -698,19 +704,19 @@ static void my_numbox_loadbang(t_my_numbox *x, t_floatarg action)
 static void my_numbox_key(void *z, t_floatarg fkey)
 {
     t_my_numbox *x = z;
-    char c=fkey;
+    char c = fkey;
     char buf[3];
     buf[1] = 0;
 
-    if (c == 0)
+    if(c == 0)
     {
         x->x_gui.x_fsf.x_change = 0;
         clock_unset(x->x_clock_reset);
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
         return;
     }
-    if(((c>='0')&&(c<='9'))||(c=='.')||(c=='-')||
-        (c=='e')||(c=='+')||(c=='E'))
+    if(((c >= '0') && (c <= '9')) || (c == '.') || (c == '-') ||
+        (c == 'e') || (c == '+') || (c == 'E'))
     {
         if(strlen(x->x_buf) < (IEMGUI_MAX_NUM_LEN-2))
         {
@@ -719,16 +725,16 @@ static void my_numbox_key(void *z, t_floatarg fkey)
             sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
         }
     }
-    else if((c=='\b')||(c==127))
+    else if((c == '\b') || (c == 127))
     {
-        int sl=strlen(x->x_buf)-1;
+        int sl = strlen(x->x_buf) - 1;
 
         if(sl < 0)
             sl = 0;
         x->x_buf[sl] = 0;
         sys_queuegui(x, x->x_gui.x_glist, my_numbox_draw_update);
     }
-    else if((c=='\n')||(c==13))
+    else if((c == '\n') || (c == 13))
     {
         x->x_val = atof(x->x_buf);
         x->x_buf[0] = 0;
@@ -743,7 +749,10 @@ static void my_numbox_key(void *z, t_floatarg fkey)
 
 static void my_numbox_list(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
 {
-    if (IS_A_FLOAT(av,0))
+    if(!ac) {
+        my_numbox_bang(x);
+    }
+    else if(IS_A_FLOAT(av, 0))
     {
         my_numbox_set(x, atom_getfloatarg(0, ac, av));
         my_numbox_bang(x);
@@ -753,11 +762,11 @@ static void my_numbox_list(t_my_numbox *x, t_symbol *s, int ac, t_atom *av)
 static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_my_numbox *x = (t_my_numbox *)pd_new(my_numbox_class);
-    int w=5, h=14;
-    int lilo=0, f=0, ldx=0, ldy=-8;
-    int fs=10;
-    int log_height=256;
-    double min=-1.0e+37, max=1.0e+37,v=0.0;
+    int w = 5, h = 14;
+    int lilo = 0, f = 0, ldx = 0, ldy = -8;
+    int fs = 10;
+    int log_height = 256;
+    double min = -1.0e+37, max = 1.0e+37, v = 0.0;
     char str[144];
 
     x->x_gui.x_bcol = 0xFCFCFC;
@@ -773,24 +782,24 @@ static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
        &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)
        &&IS_A_FLOAT(argv,11)&&IS_A_FLOAT(argv,12)&&IS_A_FLOAT(argv,16))
     {
-        w = (int)atom_getintarg(0, argc, argv);
-        h = (int)atom_getintarg(1, argc, argv);
+        w = (int)atom_getfloatarg(0, argc, argv);
+        h = (int)atom_getfloatarg(1, argc, argv);
         min = (double)atom_getfloatarg(2, argc, argv);
         max = (double)atom_getfloatarg(3, argc, argv);
-        lilo = (int)atom_getintarg(4, argc, argv);
-        iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(5, argc, argv));
+        lilo = (int)atom_getfloatarg(4, argc, argv);
+        iem_inttosymargs(&x->x_gui.x_isa, atom_getfloatarg(5, argc, argv));
         iemgui_new_getnames(&x->x_gui, 6, argv);
-        ldx = (int)atom_getintarg(9, argc, argv);
-        ldy = (int)atom_getintarg(10, argc, argv);
-        iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(11, argc, argv));
-        fs = (int)atom_getintarg(12, argc, argv);
+        ldx = (int)atom_getfloatarg(9, argc, argv);
+        ldy = (int)atom_getfloatarg(10, argc, argv);
+        iem_inttofstyle(&x->x_gui.x_fsf, atom_getfloatarg(11, argc, argv));
+        fs = (int)atom_getfloatarg(12, argc, argv);
         iemgui_all_loadcolors(&x->x_gui, argv+13, argv+14, argv+15);
         v = atom_getfloatarg(16, argc, argv);
     }
     else iemgui_new_getnames(&x->x_gui, 6, 0);
     if((argc == 18)&&IS_A_FLOAT(argv,17))
     {
-        log_height = (int)atom_getintarg(17, argc, argv);
+        log_height = (int)atom_getfloatarg(17, argc, argv);
     }
     x->x_gui.x_draw = (t_iemfunptr)my_numbox_draw;
     x->x_gui.x_fsf.x_snd_able = 1;
@@ -805,34 +814,35 @@ static void *my_numbox_new(t_symbol *s, int argc, t_atom *argv)
     if(log_height < 10)
         log_height = 10;
     x->x_log_height = log_height;
-    if (!strcmp(x->x_gui.x_snd->s_name, "empty"))
+    if(!strcmp(x->x_gui.x_snd->s_name, "empty"))
         x->x_gui.x_fsf.x_snd_able = 0;
-    if (!strcmp(x->x_gui.x_rcv->s_name, "empty"))
+    if(!strcmp(x->x_gui.x_rcv->s_name, "empty"))
         x->x_gui.x_fsf.x_rcv_able = 0;
     if(x->x_gui.x_fsf.x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
     else if(x->x_gui.x_fsf.x_font_style == 2) strcpy(x->x_gui.x_font, "times");
     else { x->x_gui.x_fsf.x_font_style = 0;
         strcpy(x->x_gui.x_font, sys_font); }
-    if (x->x_gui.x_fsf.x_rcv_able)
+    if(x->x_gui.x_fsf.x_rcv_able)
         pd_bind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
     x->x_gui.x_ldx = ldx;
     x->x_gui.x_ldy = ldy;
-    if(fs < 4)
-        fs = 4;
+    if(fs < MINFONT)
+        fs = MINFONT;
     x->x_gui.x_fontsize = fs;
-    if(w < 1)
-        w = 1;
-    x->x_gui.x_w = w;
-    if(h < 8)
-        h = 8;
+    if(w < MINDIGITS)
+        w = MINDIGITS;
+    x->x_numwidth = w;
+    if(h < IEM_GUI_MINSIZE)
+        h = IEM_GUI_MINSIZE;
     x->x_gui.x_h = h;
     x->x_buf[0] = 0;
-    my_numbox_calc_fontwidth(x);
     my_numbox_check_minmax(x, min, max);
     iemgui_verify_snd_ne_rcv(&x->x_gui);
     x->x_clock_reset = clock_new(x, (t_method)my_numbox_tick_reset);
     x->x_clock_wait = clock_new(x, (t_method)my_numbox_tick_wait);
     x->x_gui.x_fsf.x_change = 0;
+    iemgui_newzoom(&x->x_gui);
+    my_numbox_calc_fontwidth(x);
     outlet_new(&x->x_gui.x_obj, &s_float);
     return (x);
 }
@@ -850,10 +860,9 @@ void g_numbox_setup(void)
 {
     my_numbox_class = class_new(gensym("nbx"), (t_newmethod)my_numbox_new,
         (t_method)my_numbox_free, sizeof(t_my_numbox), 0, A_GIMME, 0);
-    class_addcreator((t_newmethod)my_numbox_new, gensym("my_numbox"),
-        A_GIMME, 0);
-    class_addbang(my_numbox_class,my_numbox_bang);
-    class_addfloat(my_numbox_class,my_numbox_float);
+    class_addcreator((t_newmethod)my_numbox_new, gensym("my_numbox"), A_GIMME, 0);
+    class_addbang(my_numbox_class, my_numbox_bang);
+    class_addfloat(my_numbox_class, my_numbox_float);
     class_addlist(my_numbox_class, my_numbox_list);
     class_addmethod(my_numbox_class, (t_method)my_numbox_click,
         gensym("click"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
@@ -893,6 +902,8 @@ void g_numbox_setup(void)
         gensym("init"), A_FLOAT, 0);
     class_addmethod(my_numbox_class, (t_method)my_numbox_log_height,
         gensym("log_height"), A_FLOAT, 0);
+    class_addmethod(my_numbox_class, (t_method)iemgui_zoom,
+        gensym("zoom"), A_CANT, 0);
     my_numbox_widgetbehavior.w_getrectfn =    my_numbox_getrect;
     my_numbox_widgetbehavior.w_displacefn =   iemgui_displace;
     my_numbox_widgetbehavior.w_selectfn =     iemgui_select;
