@@ -33,85 +33,109 @@ void toggle_draw_update(t_toggle *x, t_glist *glist)
 {
     if(glist_isvisible(glist))
     {
-        t_canvas *canvas=glist_getcanvas(glist);
+        t_canvas *canvas = glist_getcanvas(glist);
 
         sys_vgui(".x%lx.c itemconfigure %lxX1 -fill #%06x\n", canvas, x,
-                 (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol);
+                 (x->x_on != 0.0) ? x->x_gui.x_fcol : x->x_gui.x_bcol);
         sys_vgui(".x%lx.c itemconfigure %lxX2 -fill #%06x\n", canvas, x,
-                 (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol);
+                 (x->x_on != 0.0) ? x->x_gui.x_fcol : x->x_gui.x_bcol);
     }
 }
 
 void toggle_draw_new(t_toggle *x, t_glist *glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int w=1, xx=text_xpix(&x->x_gui.x_obj, glist), yy=text_ypix(&x->x_gui.x_obj, glist);
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
-    if(x->x_gui.x_w >= 30)
-        w = 2;
-    if(x->x_gui.x_w >= 60)
-        w = 3;
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    int crossw = 1, w = x->x_gui.x_w / IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
+    if(w >= 30)
+        crossw = 2;
+    if(w >= 60)
+        crossw = 3;
+    crossw *= IEMGUI_ZOOM(x);
+
     sys_vgui(".x%lx.c create rectangle %d %d %d %d -width %d -fill #%06x -tags %lxBASE\n",
-             canvas, xx, yy, xx + x->x_gui.x_w, yy + x->x_gui.x_h,
+             canvas, xpos, ypos,
+             xpos + x->x_gui.x_w, ypos + x->x_gui.x_h,
              IEMGUI_ZOOM(x),
              x->x_gui.x_bcol, x);
     sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxX1\n",
-             canvas, xx+w+1, yy+w+1, xx + x->x_gui.x_w-w, yy + x->x_gui.x_h-w, w,
-             (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol, x);
+             canvas,
+             xpos + crossw + IEMGUI_ZOOM(x), ypos + crossw + IEMGUI_ZOOM(x),
+             xpos + x->x_gui.x_w - crossw - IEMGUI_ZOOM(x), ypos + x->x_gui.x_h - crossw - IEMGUI_ZOOM(x),
+             crossw, (x->x_on != 0.0) ? x->x_gui.x_fcol : x->x_gui.x_bcol, x);
     sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxX2\n",
-             canvas, xx+w+1, yy + x->x_gui.x_h-w-1, xx + x->x_gui.x_w-w, yy+w, w,
-             (x->x_on!=0.0)?x->x_gui.x_fcol:x->x_gui.x_bcol, x);
-    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-             -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
-             canvas, xx+x->x_gui.x_ldx * zoomlabel,
-             yy+x->x_gui.x_ldy * zoomlabel,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
-             x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_lcol, x);
+             canvas,
+             xpos + crossw + IEMGUI_ZOOM(x), ypos + x->x_gui.x_h - crossw - IEMGUI_ZOOM(x),
+             xpos + x->x_gui.x_w - crossw - IEMGUI_ZOOM(x), ypos + crossw + IEMGUI_ZOOM(x),
+             crossw, (x->x_on != 0.0) ? x->x_gui.x_fcol : x->x_gui.x_bcol, x);
     if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]\n",
-             canvas, xx, yy + x->x_gui.x_h+1-2*IEMGUI_ZOOM(x),
-                xx + IOWIDTH, yy + x->x_gui.x_h, x, 0);
+             canvas,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h,
+             x, 0);
     if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
-             canvas, xx, yy, xx + IOWIDTH, yy-1+2*IEMGUI_ZOOM(x), x, 0);
+             canvas,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
+    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
+             -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
+             canvas, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
+             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             x->x_gui.x_lcol, x);
 }
 
 void toggle_draw_move(t_toggle *x, t_glist *glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int w=1, xx=text_xpix(&x->x_gui.x_obj, glist), yy=text_ypix(&x->x_gui.x_obj, glist);
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
+    int crossw = 1, w = x->x_gui.x_w / IEMGUI_ZOOM(x);
+    if(w >= 30)
+        crossw = 2;
+    if(w >= 60)
+        crossw = 3;
+    crossw *= IEMGUI_ZOOM(x);
 
-    if(x->x_gui.x_w >= 30)
-        w = 2;
-
-    if(x->x_gui.x_w >= 60)
-        w = 3;
     sys_vgui(".x%lx.c coords %lxBASE %d %d %d %d\n",
-             canvas, x, xx, yy, xx + x->x_gui.x_w, yy + x->x_gui.x_h);
-    sys_vgui(".x%lx.c itemconfigure %lxX1 -width %d\n", canvas, x, w);
+             canvas, x, xpos, ypos,
+             xpos + x->x_gui.x_w, ypos + x->x_gui.x_h);
+    sys_vgui(".x%lx.c itemconfigure %lxX1 -width %d\n", canvas, x, crossw);
     sys_vgui(".x%lx.c coords %lxX1 %d %d %d %d\n",
-             canvas, x, xx+w+1, yy+w+1, xx + x->x_gui.x_w-w, yy + x->x_gui.x_h-w);
-    sys_vgui(".x%lx.c itemconfigure %lxX2 -width %d\n", canvas, x, w);
+             canvas, x,
+             xpos + crossw + IEMGUI_ZOOM(x), ypos + crossw + IEMGUI_ZOOM(x),
+             xpos + x->x_gui.x_w - crossw, ypos + x->x_gui.x_h - crossw);
+    sys_vgui(".x%lx.c itemconfigure %lxX2 -width %d\n", canvas, x, crossw);
     sys_vgui(".x%lx.c coords %lxX2 %d %d %d %d\n",
-             canvas, x, xx+w+1, yy + x->x_gui.x_h-w-1, xx + x->x_gui.x_w-w, yy+w);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xx+x->x_gui.x_ldx * zoomlabel,
-             yy+x->x_gui.x_ldy * zoomlabel);
+             canvas, x,
+             xpos + crossw + IEMGUI_ZOOM(x), ypos + x->x_gui.x_h - crossw - IEMGUI_ZOOM(x),
+             xpos + x->x_gui.x_w - crossw, ypos + crossw);
     if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
-             canvas, x, 0, xx, yy + x->x_gui.x_h+1-2*IEMGUI_ZOOM(x), xx + IOWIDTH, yy + x->x_gui.x_h);
+             canvas, x, 0,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h);
     if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
-             canvas, x, 0, xx, yy, xx + IOWIDTH, yy-1+2*IEMGUI_ZOOM(x));
+             canvas, x, 0,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh);
+    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
+             canvas, x,
+             xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos + x->x_gui.x_ldy * IEMGUI_ZOOM(x));
 }
 
 void toggle_draw_erase(t_toggle* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c delete %lxBASE\n", canvas, x);
     sys_vgui(".x%lx.c delete %lxX1\n", canvas, x);
@@ -125,44 +149,48 @@ void toggle_draw_erase(t_toggle* x, t_glist* glist)
 
 void toggle_draw_config(t_toggle* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
+             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             (x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_lcol),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""));
     sys_vgui(".x%lx.c itemconfigure %lxBASE -fill #%06x\n", canvas, x,
              x->x_gui.x_bcol);
     sys_vgui(".x%lx.c itemconfigure %lxX1 -fill #%06x\n", canvas, x,
-             x->x_on?x->x_gui.x_fcol:x->x_gui.x_bcol);
+             x->x_on ? x->x_gui.x_fcol : x->x_gui.x_bcol);
     sys_vgui(".x%lx.c itemconfigure %lxX2 -fill #%06x\n", canvas, x,
-             x->x_on?x->x_gui.x_fcol:x->x_gui.x_bcol);
+             x->x_on ? x->x_gui.x_fcol : x->x_gui.x_bcol);
 }
 
 void toggle_draw_io(t_toggle* x, t_glist* glist, int old_snd_rcv_flags)
 {
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
-    t_canvas *canvas=glist_getcanvas(glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxOUT%d\n",
-             canvas, xpos,
-             ypos + x->x_gui.x_h-1, xpos + IOWIDTH,
-             ypos + x->x_gui.x_h, x, 0);
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
+             canvas,
+             xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+             xpos + iow, ypos + x->x_gui.x_h,
+             x, 0);
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
     if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxIN%d\n",
-             canvas, xpos, ypos,
-             xpos + IOWIDTH, ypos+1, x, 0);
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
+             canvas,
+             xpos, ypos,
+             xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
 }
 
 void toggle_draw_select(t_toggle* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if(x->x_gui.x_fsf.x_selected)
     {
@@ -196,7 +224,8 @@ void toggle_draw(t_toggle *x, t_glist *glist, int mode)
 
 /* ------------------------ tgl widgetbehaviour----------------------------- */
 
-static void toggle_getrect(t_gobj *z, t_glist *glist, int *xp1, int *yp1, int *xp2, int *yp2)
+static void toggle_getrect(t_gobj *z, t_glist *glist,
+                           int *xp1, int *yp1, int *xp2, int *yp2)
 {
     t_toggle *x = (t_toggle *)z;
 
@@ -213,10 +242,10 @@ static void toggle_save(t_gobj *z, t_binbuf *b)
     t_symbol *srl[3];
 
     iemgui_save(&x->x_gui, srl, bflcol);
-    binbuf_addv(b, "ssiisiisssiiiisssff", gensym("#X"),gensym("obj"),
+    binbuf_addv(b, "ssiisiisssiiiisssff", gensym("#X"), gensym("obj"),
                 (int)x->x_gui.x_obj.te_xpix,
                 (int)x->x_gui.x_obj.te_ypix,
-                gensym("tgl"), x->x_gui.x_w,
+                gensym("tgl"), x->x_gui.x_w/IEMGUI_ZOOM(x),
                 iem_symargstoint(&x->x_gui.x_isa),
                 srl[0], srl[1], srl[2],
                 x->x_gui.x_ldx, x->x_gui.x_ldy,
@@ -240,7 +269,7 @@ static void toggle_properties(t_gobj *z, t_glist *owner)
             %s %d %d \
             %d %d \
             #%06x #%06x #%06x\n",
-            x->x_gui.x_w, IEM_GUI_MINSIZE,
+            x->x_gui.x_w/IEMGUI_ZOOM(x), IEM_GUI_MINSIZE,
             x->x_nonzero, 1.0,/*non_zero-schedule*/
             x->x_gui.x_isa.x_loadinit, -1, -1,/*no multi*/
             srl[0]->s_name, srl[1]->s_name,
@@ -252,7 +281,7 @@ static void toggle_properties(t_gobj *z, t_glist *owner)
 
 static void toggle_bang(t_toggle *x)
 {
-    x->x_on = (x->x_on==0.0)?x->x_nonzero:0.0;
+    x->x_on = (x->x_on == 0.0) ? x->x_nonzero : 0.0;
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
     outlet_float(x->x_gui.x_obj.ob_outlet, x->x_on);
     if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
@@ -262,7 +291,7 @@ static void toggle_bang(t_toggle *x)
 static void toggle_dialog(t_toggle *x, t_symbol *s, int argc, t_atom *argv)
 {
     t_symbol *srl[3];
-    int a = (int)atom_getintarg(0, argc, argv);
+    int a = (int)atom_getfloatarg(0, argc, argv);
     t_float nonzero = (t_float)atom_getfloatarg(2, argc, argv);
     int sr_flags;
 
@@ -272,7 +301,7 @@ static void toggle_dialog(t_toggle *x, t_symbol *s, int argc, t_atom *argv)
     if(x->x_on != 0.0)
         x->x_on = x->x_nonzero;
     sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
-    x->x_gui.x_w = iemgui_clip_size(a);
+    x->x_gui.x_w = iemgui_clip_size(a) * IEMGUI_ZOOM(x);
     x->x_gui.x_h = x->x_gui.x_w;
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_CONFIG);
     (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_IO + sr_flags);
@@ -327,7 +356,7 @@ static void toggle_loadbang(t_toggle *x, t_floatarg action)
 
 static void toggle_size(t_toggle *x, t_symbol *s, int ac, t_atom *av)
 {
-    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
+    x->x_gui.x_w = iemgui_clip_size((int)atom_getfloatarg(0, ac, av)) * IEMGUI_ZOOM(x);
     x->x_gui.x_h = x->x_gui.x_w;
     iemgui_size((void *)x, &x->x_gui);
 }
@@ -358,7 +387,7 @@ static void toggle_label_pos(t_toggle *x, t_symbol *s, int ac, t_atom *av)
 
 static void toggle_init(t_toggle *x, t_floatarg f)
 {
-    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
+    x->x_gui.x_isa.x_loadinit = (f == 0.0) ? 0 : 1;
 }
 
 static void toggle_nonzero(t_toggle *x, t_floatarg f)
@@ -370,10 +399,10 @@ static void toggle_nonzero(t_toggle *x, t_floatarg f)
 static void *toggle_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_toggle *x = (t_toggle *)pd_new(toggle_class);
-    int a=IEM_GUI_DEFAULTSIZE, f=0;
-    int ldx=17, ldy=7;
-    int fs=10;
-    t_float on=0.0, nonzero=1.0;
+    int a = IEM_GUI_DEFAULTSIZE, f = 0;
+    int ldx = 17, ldy = 7;
+    int fs = 10;
+    t_float on = 0.0, nonzero = 1.0;
     char str[144];
 
     iem_inttosymargs(&x->x_gui.x_isa, 0);
@@ -391,13 +420,13 @@ static void *toggle_new(t_symbol *s, int argc, t_atom *argv)
        &&IS_A_FLOAT(argv,5)&&IS_A_FLOAT(argv,6)
        &&IS_A_FLOAT(argv,7)&&IS_A_FLOAT(argv,8)&&IS_A_FLOAT(argv,12))
     {
-        a = (int)atom_getintarg(0, argc, argv);
-        iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(1, argc, argv));
+        a = (int)atom_getfloatarg(0, argc, argv);
+        iem_inttosymargs(&x->x_gui.x_isa, atom_getfloatarg(1, argc, argv));
         iemgui_new_getnames(&x->x_gui, 2, argv);
-        ldx = (int)atom_getintarg(5, argc, argv);
-        ldy = (int)atom_getintarg(6, argc, argv);
-        iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(7, argc, argv));
-        fs = (int)atom_getintarg(8, argc, argv);
+        ldx = (int)atom_getfloatarg(5, argc, argv);
+        ldy = (int)atom_getfloatarg(6, argc, argv);
+        iem_inttofstyle(&x->x_gui.x_fsf, atom_getfloatarg(7, argc, argv));
+        fs = (int)atom_getfloatarg(8, argc, argv);
         iemgui_all_loadcolors(&x->x_gui, argv+9, argv+10, argv+11);
         on = (t_float)atom_getfloatarg(12, argc, argv);
     }
@@ -417,23 +446,22 @@ static void *toggle_new(t_symbol *s, int argc, t_atom *argv)
     else if(x->x_gui.x_fsf.x_font_style == 2) strcpy(x->x_gui.x_font, "times");
     else { x->x_gui.x_fsf.x_font_style = 0;
         strcpy(x->x_gui.x_font, sys_font); }
-    x->x_nonzero = (nonzero!=0.0)?nonzero:1.0;
+    x->x_nonzero = (nonzero != 0.0) ? nonzero : 1.0;
     if(x->x_gui.x_isa.x_loadinit)
-        x->x_on = (on!=0.0)?nonzero:0.0;
+        x->x_on = (on != 0.0) ? nonzero : 0.0;
     else
         x->x_on = 0.0;
     if (x->x_gui.x_fsf.x_rcv_able)
         pd_bind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
     x->x_gui.x_ldx = ldx;
     x->x_gui.x_ldy = ldy;
-
     if(fs < 4)
         fs = 4;
     x->x_gui.x_fontsize = fs;
     x->x_gui.x_w = iemgui_clip_size(a);
     x->x_gui.x_h = x->x_gui.x_w;
-    iemgui_newzoom(&x->x_gui);
     iemgui_verify_snd_ne_rcv(&x->x_gui);
+    iemgui_newzoom(&x->x_gui);
     outlet_new(&x->x_gui.x_obj, &s_float);
     return (x);
 }
