@@ -17,14 +17,6 @@ put out a "float" as in sliders, toggles, etc. */
 #include "g_all_guis.h"
 #include <math.h>
 
-/*------------------ global variables -------------------------*/
-
-
-/*------------------ global functions -------------------------*/
-
-
-
-
 /* ------------- vdl     gui-vertical radio button ---------------------- */
 
 t_widgetbehavior vradio_widgetbehavior;
@@ -51,59 +43,62 @@ void vradio_draw_update(t_gobj *client, t_glist *glist)
 
 void vradio_draw_new(t_vradio *x, t_glist *glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int n=x->x_number, i, dy=x->x_gui.x_h, s4=dy/4;
-    int yy11b=text_ypix(&x->x_gui.x_obj, glist);
-    int yy11=yy11b, yy12=yy11+dy;
-    int yy21=yy11+s4, yy22=yy12-s4;
-    int xx11=text_xpix(&x->x_gui.x_obj, glist), xx12=xx11+dy;
-    int xx21=xx11+s4, xx22=xx12-s4;
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
+    int n = x->x_number, i, dy = x->x_gui.x_h, s4 = dy / 4;
+    int yy11b = text_ypix(&x->x_gui.x_obj, glist);
+    int yy11 = yy11b, yy12 = yy11 + dy;
+    int yy21 = yy11 + s4, yy22 = yy12 - s4;
+    int xx11 = text_xpix(&x->x_gui.x_obj, glist), xx12 = xx11 + dy;
+    int xx21 = xx11 + s4, xx22 = xx12 - s4;
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
-    for(i=0; i<n; i++)
+    for(i = 0; i < n; i++)
     {
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -width %d -fill #%06x -tags %lxBASE%d\n",
                  canvas, xx11, yy11, xx12, yy12, IEMGUI_ZOOM(x),
                  x->x_gui.x_bcol, x, i);
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill #%06x -outline #%06x -tags %lxBUT%d\n",
                  canvas, xx21, yy21, xx22, yy22,
-                 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol,
-                 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol, x, i);
+                 (x->x_on == i) ? x->x_gui.x_fcol : x->x_gui.x_bcol,
+                 (x->x_on == i) ? x->x_gui.x_fcol : x->x_gui.x_bcol, x, i);
         yy11 += dy;
         yy12 += dy;
         yy21 += dy;
         yy22 += dy;
         x->x_drawn = x->x_on;
     }
-    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
-             -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
-             canvas, xx11+x->x_gui.x_ldx * zoomlabel,
-             yy11b+x->x_gui.x_ldy * zoomlabel,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"",
-             x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_lcol, x);
     if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxOUT%d outlet]\n",
-             canvas, xx11, yy11+1-2*IEMGUI_ZOOM(x), xx11 + IOWIDTH, yy11, x, 0);
+             canvas,
+             xx11, yy11 + IEMGUI_ZOOM(x) - ioh,
+             xx11 + iow, yy11,
+             x, 0);
     if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags [list %lxIN%d inlet]\n",
-             canvas, xx11, yy11b, xx11 + IOWIDTH, yy11b-1+2*IEMGUI_ZOOM(x), x, 0);
+             canvas,
+             xx11, yy11b,
+             xx11 + iow, yy11b - IEMGUI_ZOOM(x) + ioh,
+             x, 0);
+    sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
+             canvas, xx11 + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             yy11b + x->x_gui.x_ldy * IEMGUI_ZOOM(x),
+             (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
+             x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             x->x_gui.x_lcol, x);
 }
 
 void vradio_draw_move(t_vradio *x, t_glist *glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int n=x->x_number, i, dy=x->x_gui.x_h, s4=dy/4;
-    int yy11b=text_ypix(&x->x_gui.x_obj, glist);
-    int yy11=yy11b, yy12=yy11+dy;
-    int yy21=yy11+s4, yy22=yy12-s4;
-    int xx11=text_xpix(&x->x_gui.x_obj, glist), xx12=xx11+dy;
-    int xx21=xx11+s4, xx22=xx12-s4;
-    int zoomlabel =
-        1 + (IEMGUI_ZOOM(x)-1) * (x->x_gui.x_ldx >= 0 && x->x_gui.x_ldy >= 0);
+    int n = x->x_number, i, dy = x->x_gui.x_h, s4 = dy / 4;
+    int yy11b = text_ypix(&x->x_gui.x_obj, glist);
+    int yy11 = yy11b, yy12 = yy11 + dy;
+    int yy21 = yy11 + s4, yy22 = yy12 - s4;
+    int xx11 = text_xpix(&x->x_gui.x_obj, glist), xx12 = xx11 + dy;
+    int xx21 = xx11 + s4, xx22 = xx12 - s4;
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
-    for(i=0; i<n; i++)
+    for(i = 0; i < n; i++)
     {
         sys_vgui(".x%lx.c coords %lxBASE%d %d %d %d %d\n",
                  canvas, x, i, xx11, yy11, xx12, yy12);
@@ -115,22 +110,26 @@ void vradio_draw_move(t_vradio *x, t_glist *glist)
         yy22 += dy;
     }
     sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xx11+x->x_gui.x_ldx * zoomlabel,
-             yy11b+x->x_gui.x_ldy * zoomlabel);
+             canvas, x, xx11 + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             yy11b + x->x_gui.x_ldy * IEMGUI_ZOOM(x));
     if(!x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c coords %lxOUT%d %d %d %d %d\n",
-             canvas, x, 0, xx11, yy11+1-2*IEMGUI_ZOOM(x), xx11 + IOWIDTH, yy11);
+             canvas, x, 0,
+             xx11, yy11 + IEMGUI_ZOOM(x) - ioh,
+             xx11 + iow, yy11);
     if(!x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c coords %lxIN%d %d %d %d %d\n",
-             canvas, x, 0, xx11, yy11b, xx11 + IOWIDTH, yy11b-1+2*IEMGUI_ZOOM(x));
+             canvas, x, 0,
+             xx11, yy11b,
+             xx11 + iow, yy11b - IEMGUI_ZOOM(x) + ioh);
 }
 
 void vradio_draw_erase(t_vradio* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int n=x->x_number, i;
+    int n = x->x_number, i;
+    t_canvas *canvas = glist_getcanvas(glist);
 
-    for(i=0; i<n; i++)
+    for(i = 0; i < n; i++)
     {
         sys_vgui(".x%lx.c delete %lxBASE%d\n", canvas, x, i);
         sys_vgui(".x%lx.c delete %lxBUT%d\n", canvas, x, i);
@@ -144,54 +143,64 @@ void vradio_draw_erase(t_vradio* x, t_glist* glist)
 
 void vradio_draw_config(t_vradio* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int n=x->x_number, i;
+    int n = x->x_number, i;
+    t_canvas *canvas = glist_getcanvas(glist);
 
     sys_vgui(".x%lx.c itemconfigure %lxLABEL -font {{%s} -%d %s} -fill #%06x -text {%s} \n",
-             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize, sys_fontweight,
-             x->x_gui.x_fsf.x_selected?IEM_GUI_COLOR_SELECTED:x->x_gui.x_lcol,
-             strcmp(x->x_gui.x_lab->s_name, "empty")?x->x_gui.x_lab->s_name:"");
+             canvas, x, x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
+             x->x_gui.x_fsf.x_selected ? IEM_GUI_COLOR_SELECTED : x->x_gui.x_lcol,
+             strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : "");
     for(i=0; i<n; i++)
     {
         sys_vgui(".x%lx.c itemconfigure %lxBASE%d -fill #%06x\n", canvas, x, i,
                  x->x_gui.x_bcol);
         sys_vgui(".x%lx.c itemconfigure %lxBUT%d -fill #%06x -outline #%06x\n", canvas, x, i,
-                 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol,
-                 (x->x_on==i)?x->x_gui.x_fcol:x->x_gui.x_bcol);
+                 (x->x_on == i) ? x->x_gui.x_fcol : x->x_gui.x_bcol,
+                 (x->x_on == i) ? x->x_gui.x_fcol : x->x_gui.x_bcol);
     }
 }
 
 void vradio_draw_io(t_vradio* x, t_glist* glist, int old_snd_rcv_flags)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int xpos=text_xpix(&x->x_gui.x_obj, glist);
-    int ypos=text_ypix(&x->x_gui.x_obj, glist);
+    int xpos = text_xpix(&x->x_gui.x_obj, glist);
+    int ypos = text_ypix(&x->x_gui.x_obj, glist);
+    int iow = IOWIDTH * IEMGUI_ZOOM(x), ioh = IEM_GUI_IOHEIGHT * IEMGUI_ZOOM(x);
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if((old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && !x->x_gui.x_fsf.x_snd_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxOUT%d\n",
-                 canvas, xpos,
-                 ypos+(x->x_number*x->x_gui.x_h)-1,
-                 xpos+ IOWIDTH,
-                 ypos+(x->x_number*x->x_gui.x_h), x, 0);
+    {
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxOUT%d\n",
+                 canvas,
+                 xpos, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
+                 xpos + iow, ypos + x->x_gui.x_h,
+                 x, 0);
+        if(x->x_on == 0) /* keep button above outlet */
+            sys_vgui(".x%lx.c raise %lxBUT%d %lxOUT%d\n", canvas, x, x->x_on, x, 0);
+    }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
     if((old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && !x->x_gui.x_fsf.x_rcv_able)
-        sys_vgui(".x%lx.c create rectangle %d %d %d %d -tags %lxIN%d\n",
-                 canvas, xpos, ypos,
-                 xpos+ IOWIDTH, ypos+1,
+    {
+        sys_vgui(".x%lx.c create rectangle %d %d %d %d -fill black -tags %lxIN%d\n",
+                 canvas,
+                 xpos, ypos,
+                 xpos + iow, ypos - IEMGUI_ZOOM(x) + ioh,
                  x, 0);
+        if(x->x_on == 0) /* keep button above inlet */
+            sys_vgui(".x%lx.c raise %lxBUT%d %lxIN%d\n", canvas, x, x->x_on, x, 0);
+    }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
 }
 
 void vradio_draw_select(t_vradio* x, t_glist* glist)
 {
-    t_canvas *canvas=glist_getcanvas(glist);
-    int n=x->x_number, i;
+    int n = x->x_number, i;
+    t_canvas *canvas = glist_getcanvas(glist);
 
     if(x->x_gui.x_fsf.x_selected)
     {
-        for(i=0; i<n; i++)
+        for(i = 0; i < n; i++)
         {
             sys_vgui(".x%lx.c itemconfigure %lxBASE%d -outline #%06x\n", canvas, x, i,
                      IEM_GUI_COLOR_SELECTED);
@@ -200,7 +209,7 @@ void vradio_draw_select(t_vradio* x, t_glist* glist)
     }
     else
     {
-        for(i=0; i<n; i++)
+        for(i = 0; i < n; i++)
         {
             sys_vgui(".x%lx.c itemconfigure %lxBASE%d -outline #%06x\n", canvas, x, i,
                      IEM_GUI_COLOR_NORMAL);
@@ -237,7 +246,7 @@ static void vradio_getrect(t_gobj *z, t_glist *glist, int *xp1, int *yp1, int *x
     *xp1 = text_xpix(&x->x_gui.x_obj, glist);
     *yp1 = text_ypix(&x->x_gui.x_obj, glist);
     *xp2 = *xp1 + x->x_gui.x_w;
-    *yp2 = *yp1 + x->x_gui.x_h*x->x_number;
+    *yp2 = *yp1 + x->x_gui.x_h * x->x_number;
 }
 
 static void vradio_save(t_gobj *z, t_binbuf *b)
@@ -247,12 +256,12 @@ static void vradio_save(t_gobj *z, t_binbuf *b)
     t_symbol *srl[3];
 
     iemgui_save(&x->x_gui, srl, bflcol);
-    binbuf_addv(b, "ssiisiiiisssiiiisssf", gensym("#X"),gensym("obj"),
+    binbuf_addv(b, "ssiisiiiisssiiiisssf", gensym("#X"), gensym("obj"),
                 (int)x->x_gui.x_obj.te_xpix,
                 (int)x->x_gui.x_obj.te_ypix,
                 (pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class ?
                     gensym("vdl") : gensym("vradio")),
-                x->x_gui.x_w,
+                x->x_gui.x_w/IEMGUI_ZOOM(x),
                 x->x_change, iem_symargstoint(&x->x_gui.x_isa), x->x_number,
                 srl[0], srl[1], srl[2],
                 x->x_gui.x_ldx, x->x_gui.x_ldy,
@@ -266,7 +275,7 @@ static void vradio_properties(t_gobj *z, t_glist *owner)
     t_vradio *x = (t_vradio *)z;
     char buf[800];
     t_symbol *srl[3];
-    int hchange=-1;
+    int hchange = -1;
 
     iemgui_properties(&x->x_gui, srl);
     if(pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class)
@@ -279,7 +288,7 @@ static void vradio_properties(t_gobj *z, t_glist *owner)
             %s %d %d \
             %d %d \
             #%06x #%06x #%06x\n",
-            x->x_gui.x_w, IEM_GUI_MINSIZE,
+            x->x_gui.x_w/IEMGUI_ZOOM(x), IEM_GUI_MINSIZE,
             0,/*no_schedule*/
             hchange, x->x_gui.x_isa.x_loadinit, -1, x->x_number,
             srl[0]->s_name, srl[1]->s_name,
@@ -292,15 +301,15 @@ static void vradio_properties(t_gobj *z, t_glist *owner)
 static void vradio_dialog(t_vradio *x, t_symbol *s, int argc, t_atom *argv)
 {
     t_symbol *srl[3];
-    int a = (int)atom_getintarg(0, argc, argv);
-    int chg = (int)atom_getintarg(4, argc, argv);
-    int num = (int)atom_getintarg(6, argc, argv);
+    int a = (int)atom_getfloatarg(0, argc, argv);
+    int chg = (int)atom_getfloatarg(4, argc, argv);
+    int num = (int)atom_getfloatarg(6, argc, argv);
     int sr_flags;
 
     if(chg != 0) chg = 1;
     x->x_change = chg;
     sr_flags = iemgui_dialog(&x->x_gui, srl, argc, argv);
-    x->x_gui.x_w = iemgui_clip_size(a);
+    x->x_gui.x_w = iemgui_clip_size(a) * IEMGUI_ZOOM(x);
     x->x_gui.x_h = x->x_gui.x_w;
     if(x->x_number != num)
     {
@@ -324,14 +333,14 @@ static void vradio_dialog(t_vradio *x, t_symbol *s, int argc, t_atom *argv)
 
 static void vradio_set(t_vradio *x, t_floatarg f)
 {
-    int i=(int)f;
+    int i = (int)f;
     int old;
 
     x->x_fval = f;
     if(i < 0)
         i = 0;
     if(i >= x->x_number)
-        i = x->x_number-1;
+        i = x->x_number - 1;
     if(x->x_on != x->x_on_old)
     {
         old = x->x_on_old;
@@ -349,10 +358,10 @@ static void vradio_set(t_vradio *x, t_floatarg f)
 
 static void vradio_bang(t_vradio *x)
 {
-        /* compatibility with earlier  "vdial" behavior */
-    if (pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class)
+        /* compatibility with earlier "vdial" behavior */
+    if(pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class)
     {
-        if((x->x_change)&&(x->x_on != x->x_on_old))
+        if((x->x_change) && (x->x_on != x->x_on_old))
         {
             SETFLOAT(x->x_at, (t_float)x->x_on_old);
             SETFLOAT(x->x_at+1, 0.0);
@@ -378,18 +387,18 @@ static void vradio_bang(t_vradio *x)
 
 static void vradio_fout(t_vradio *x, t_floatarg f)
 {
-    int i=(int)f;
+    int i = (int)f;
 
     x->x_fval = f;
     if(i < 0)
         i = 0;
     if(i >= x->x_number)
-        i = x->x_number-1;
+        i = x->x_number - 1;
 
-    if (pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class)
+    if(pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class)
     {
             /* compatibility with earlier  "vdial" behavior */
-        if((x->x_change)&&(i != x->x_on_old))
+        if((x->x_change) && (i != x->x_on_old))
         {
             SETFLOAT(x->x_at, (t_float)x->x_on_old);
             SETFLOAT(x->x_at+1, 0.0);
@@ -415,25 +424,25 @@ static void vradio_fout(t_vradio *x, t_floatarg f)
         x->x_on = i;
         (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
         outlet_float(x->x_gui.x_obj.ob_outlet, outval);
-        if (x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
+        if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
             pd_float(x->x_gui.x_snd->s_thing, outval);
     }
 }
 
 static void vradio_float(t_vradio *x, t_floatarg f)
 {
-    int i=(int)f;
+    int i = (int)f;
 
     x->x_fval = f;
     if(i < 0)
         i = 0;
     if(i >= x->x_number)
-        i = x->x_number-1;
+        i = x->x_number - 1;
 
-    if (pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class)
+    if(pd_class(&x->x_gui.x_obj.ob_pd) == vradio_old_class)
     {
-            /* compatibility with earlier  "vdial" behavior */
-        if((x->x_change)&&(i != x->x_on_old))
+            /* compatibility with earlier "vdial" behavior */
+        if((x->x_change) && (i != x->x_on_old))
         {
             if(x->x_gui.x_fsf.x_put_in2out)
             {
@@ -464,7 +473,7 @@ static void vradio_float(t_vradio *x, t_floatarg f)
         x->x_on_old = x->x_on;
         x->x_on = i;
         (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_UPDATE);
-        if (x->x_gui.x_fsf.x_put_in2out)
+        if(x->x_gui.x_fsf.x_put_in2out)
         {
             outlet_float(x->x_gui.x_obj.ob_outlet, outval);
             if(x->x_gui.x_fsf.x_snd_able && x->x_gui.x_snd->s_thing)
@@ -492,13 +501,13 @@ static int vradio_newclick(t_gobj *z, struct _glist *glist,
 
 static void vradio_loadbang(t_vradio *x, t_floatarg action)
 {
-    if (action == LB_LOAD && x->x_gui.x_isa.x_loadinit)
+    if(action == LB_LOAD && x->x_gui.x_isa.x_loadinit)
         vradio_bang(x);
 }
 
 static void vradio_number(t_vradio *x, t_floatarg num)
 {
-    int n=(int)num;
+    int n = (int)num;
 
     if(n < 1)
         n = 1;
@@ -512,12 +521,13 @@ static void vradio_number(t_vradio *x, t_floatarg num)
             x->x_on = x->x_number - 1;
         x->x_on_old = x->x_on;
         (*x->x_gui.x_draw)(x, x->x_gui.x_glist, IEM_GUI_DRAW_MODE_NEW);
+        canvas_fixlinesfor(x->x_gui.x_glist, (t_text*)x);
     }
 }
 
 static void vradio_size(t_vradio *x, t_symbol *s, int ac, t_atom *av)
 {
-    x->x_gui.x_w = iemgui_clip_size((int)atom_getintarg(0, ac, av));
+    x->x_gui.x_w = iemgui_clip_size((int)atom_getfloatarg(0, ac, av)) * IEMGUI_ZOOM(x);
     x->x_gui.x_h = x->x_gui.x_w;
     iemgui_size((void *)x, &x->x_gui);
 }
@@ -547,9 +557,7 @@ static void vradio_label_font(t_vradio *x, t_symbol *s, int ac, t_atom *av)
 {iemgui_label_font((void *)x, &x->x_gui, s, ac, av);}
 
 static void vradio_init(t_vradio *x, t_floatarg f)
-{
-    x->x_gui.x_isa.x_loadinit = (f==0.0)?0:1;
-}
+{x->x_gui.x_isa.x_loadinit = (f == 0.0) ? 0 : 1;}
 
 static void vradio_double_change(t_vradio *x)
 {x->x_change = 1;}
@@ -559,11 +567,11 @@ static void vradio_single_change(t_vradio *x)
 
 static void *vradio_donew(t_symbol *s, int argc, t_atom *argv, int old)
 {
-    t_vradio *x = (t_vradio *)pd_new(old? vradio_old_class : vradio_class);
-    int a=IEM_GUI_DEFAULTSIZE, on = 0, f=0;
-    int ldx=0, ldy=-8, chg=1, num=8;
-    int fs=10;
-    int ftbreak=IEM_BNG_DEFAULTBREAKFLASHTIME, fthold=IEM_BNG_DEFAULTHOLDFLASHTIME;
+    t_vradio *x = (t_vradio *)pd_new(old ? vradio_old_class : vradio_class);
+    int a = IEM_GUI_DEFAULTSIZE, on = 0, f = 0;
+    int ldx = 0, ldy = -8, chg = 1, num = 8;
+    int fs = 10;
+    int ftbreak = IEM_BNG_DEFAULTBREAKFLASHTIME, fthold = IEM_BNG_DEFAULTHOLDFLASHTIME;
     char str[144];
     float fval = 0;
 
@@ -579,26 +587,26 @@ static void *vradio_donew(t_symbol *s, int argc, t_atom *argv, int old)
        &&IS_A_FLOAT(argv,7)&&IS_A_FLOAT(argv,8)
        &&IS_A_FLOAT(argv,9)&&IS_A_FLOAT(argv,10)&&IS_A_FLOAT(argv,14))
     {
-        a = (int)atom_getintarg(0, argc, argv);
-        chg = (int)atom_getintarg(1, argc, argv);
-        iem_inttosymargs(&x->x_gui.x_isa, atom_getintarg(2, argc, argv));
-        num = (int)atom_getintarg(3, argc, argv);
+        a = (int)atom_getfloatarg(0, argc, argv);
+        chg = (int)atom_getfloatarg(1, argc, argv);
+        iem_inttosymargs(&x->x_gui.x_isa, atom_getfloatarg(2, argc, argv));
+        num = (int)atom_getfloatarg(3, argc, argv);
         iemgui_new_getnames(&x->x_gui, 4, argv);
-        ldx = (int)atom_getintarg(7, argc, argv);
-        ldy = (int)atom_getintarg(8, argc, argv);
-        iem_inttofstyle(&x->x_gui.x_fsf, atom_getintarg(9, argc, argv));
-        fs = (int)atom_getintarg(10, argc, argv);
+        ldx = (int)atom_getfloatarg(7, argc, argv);
+        ldy = (int)atom_getfloatarg(8, argc, argv);
+        iem_inttofstyle(&x->x_gui.x_fsf, atom_getfloatarg(9, argc, argv));
+        fs = (int)atom_getfloatarg(10, argc, argv);
         iemgui_all_loadcolors(&x->x_gui, argv+11, argv+12, argv+13);
-        fval = atom_getintarg(14, argc, argv);
+        fval = atom_getfloatarg(14, argc, argv);
     }
     else iemgui_new_getnames(&x->x_gui, 4, 0);
     x->x_gui.x_draw = (t_iemfunptr)vradio_draw;
     x->x_gui.x_fsf.x_snd_able = 1;
     x->x_gui.x_fsf.x_rcv_able = 1;
     x->x_gui.x_glist = (t_glist *)canvas_getcurrent();
-    if (!strcmp(x->x_gui.x_snd->s_name, "empty"))
+    if(!strcmp(x->x_gui.x_snd->s_name, "empty"))
         x->x_gui.x_fsf.x_snd_able = 0;
-    if (!strcmp(x->x_gui.x_rcv->s_name, "empty"))
+    if(!strcmp(x->x_gui.x_rcv->s_name, "empty"))
         x->x_gui.x_fsf.x_rcv_able = 0;
     if(x->x_gui.x_fsf.x_font_style == 1) strcpy(x->x_gui.x_font, "helvetica");
     else if(x->x_gui.x_fsf.x_font_style == 2) strcpy(x->x_gui.x_font, "times");
@@ -620,8 +628,8 @@ static void *vradio_donew(t_symbol *s, int argc, t_atom *argv, int old)
     else
         x->x_on = 0;
     x->x_on_old = x->x_on;
-    x->x_change = (chg==0)?0:1;
-    if (x->x_gui.x_fsf.x_rcv_able)
+    x->x_change = (chg == 0) ? 0 : 1;
+    if(x->x_gui.x_fsf.x_rcv_able)
         pd_bind(&x->x_gui.x_obj.ob_pd, x->x_gui.x_rcv);
     x->x_gui.x_ldx = ldx;
     x->x_gui.x_ldy = ldy;
@@ -631,6 +639,7 @@ static void *vradio_donew(t_symbol *s, int argc, t_atom *argv, int old)
     x->x_gui.x_w = iemgui_clip_size(a);
     x->x_gui.x_h = x->x_gui.x_w;
     iemgui_verify_snd_ne_rcv(&x->x_gui);
+    iemgui_newzoom(&x->x_gui);
     outlet_new(&x->x_gui.x_obj, &s_list);
     return (x);
 }
@@ -658,10 +667,10 @@ void g_vradio_setup(void)
         (t_method)vradio_ff, sizeof(t_vradio), 0, A_GIMME, 0);
     class_addbang(vradio_class, vradio_bang);
     class_addfloat(vradio_class, vradio_float);
-    class_addmethod(vradio_class, (t_method)vradio_click, gensym("click"),
-                    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
-    class_addmethod(vradio_class, (t_method)vradio_dialog, gensym("dialog"),
-                    A_GIMME, 0);
+    class_addmethod(vradio_class, (t_method)vradio_click,
+        gensym("click"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
+    class_addmethod(vradio_class, (t_method)vradio_dialog,
+        gensym("dialog"), A_GIMME, 0);
     class_addmethod(vradio_class, (t_method)vradio_loadbang,
         gensym("loadbang"), A_DEFFLOAT, 0);
     class_addmethod(vradio_class, (t_method)vradio_set,
@@ -692,8 +701,8 @@ void g_vradio_setup(void)
         gensym("single_change"), 0);
     class_addmethod(vradio_class, (t_method)vradio_double_change,
         gensym("double_change"), 0);
-    class_addmethod(vradio_class, (t_method)iemgui_zoom, gensym("zoom"),
-        A_CANT, 0);
+    class_addmethod(vradio_class, (t_method)iemgui_zoom,
+        gensym("zoom"), A_CANT, 0);
     vradio_widgetbehavior.w_getrectfn = vradio_getrect;
     vradio_widgetbehavior.w_displacefn = iemgui_displace;
     vradio_widgetbehavior.w_selectfn = iemgui_select;
@@ -711,27 +720,42 @@ void g_vradio_setup(void)
         (t_method)vradio_ff, sizeof(t_vradio), 0, A_GIMME, 0);
     class_addbang(vradio_old_class, vradio_bang);
     class_addfloat(vradio_old_class, vradio_float);
-    class_addmethod(vradio_old_class, (t_method)vradio_click, gensym("click"),
-                    A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_dialog, gensym("dialog"),
-                    A_GIMME, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_loadbang, gensym("loadbang"), 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_set, gensym("set"), A_FLOAT, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_size, gensym("size"), A_GIMME, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_delta, gensym("delta"), A_GIMME, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_pos, gensym("pos"), A_GIMME, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_color, gensym("color"), A_GIMME, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_send, gensym("send"), A_DEFSYM, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_receive, gensym("receive"), A_DEFSYM, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_label, gensym("label"), A_DEFSYM, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_label_pos, gensym("label_pos"), A_GIMME, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_label_font, gensym("label_font"), A_GIMME, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_init, gensym("init"), A_FLOAT, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_number, gensym("number"), A_FLOAT, 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_single_change, gensym("single_change"), 0);
-    class_addmethod(vradio_old_class, (t_method)vradio_double_change, gensym("double_change"), 0);
-    class_addmethod(vradio_old_class, (t_method)iemgui_zoom, gensym("zoom"),
-        A_CANT, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_click,
+        gensym("click"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_dialog,
+        gensym("dialog"), A_GIMME, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_loadbang,
+        gensym("loadbang"), 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_set,
+        gensym("set"), A_FLOAT, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_size,
+        gensym("size"), A_GIMME, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_delta,
+        gensym("delta"), A_GIMME, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_pos,
+        gensym("pos"), A_GIMME, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_color,
+        gensym("color"), A_GIMME, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_send,
+        gensym("send"), A_DEFSYM, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_receive,
+        gensym("receive"), A_DEFSYM, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_label,
+        gensym("label"), A_DEFSYM, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_label_pos,
+        gensym("label_pos"), A_GIMME, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_label_font,
+        gensym("label_font"), A_GIMME, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_init,
+        gensym("init"), A_FLOAT, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_number,
+        gensym("number"), A_FLOAT, 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_single_change,
+        gensym("single_change"), 0);
+    class_addmethod(vradio_old_class, (t_method)vradio_double_change,
+        gensym("double_change"), 0);
+    class_addmethod(vradio_old_class, (t_method)iemgui_zoom,
+        gensym("zoom"), A_CANT, 0);
     class_setwidget(vradio_old_class, &vradio_widgetbehavior);
     class_sethelpsymbol(vradio_old_class, gensym("vradio"));
 }
