@@ -31,12 +31,15 @@ void glob_midi_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv);
 void glob_midi_setapi(t_pd *dummy, t_floatarg f);
 void glob_start_path_dialog(t_pd *dummy, t_floatarg flongform);
 void glob_path_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv);
+void glob_addtopath(t_pd *dummy, t_symbol *path, t_float saveit);
 void glob_start_startup_dialog(t_pd *dummy, t_floatarg flongform);
 void glob_startup_dialog(t_pd *dummy, t_symbol *s, int argc, t_atom *argv);
 void glob_ping(t_pd *dummy);
 void glob_plugindispatch(t_pd *dummy, t_symbol *s, int argc, t_atom *argv);
 void glob_watchdog(t_pd *dummy);
-void glob_savepreferences(t_pd *dummy);
+void glob_loadpreferences(t_pd *dummy, t_symbol *s);
+void glob_savepreferences(t_pd *dummy, t_symbol *s);
+void glob_forgetpreferences(t_pd *dummy);
 
 static void glob_helpintro(t_pd *dummy)
 {
@@ -166,13 +169,19 @@ void glob_init(void)
         gensym("start-path-dialog"), 0);
     class_addmethod(glob_pdobject, (t_method)glob_path_dialog,
         gensym("path-dialog"), A_GIMME, 0);
+    class_addmethod(glob_pdobject, (t_method)glob_addtopath,
+        gensym("add-to-path"), A_SYMBOL, A_DEFFLOAT, 0);
     class_addmethod(glob_pdobject, (t_method)glob_start_startup_dialog,
         gensym("start-startup-dialog"), 0);
     class_addmethod(glob_pdobject, (t_method)glob_startup_dialog,
         gensym("startup-dialog"), A_GIMME, 0);
     class_addmethod(glob_pdobject, (t_method)glob_ping, gensym("ping"), 0);
+    class_addmethod(glob_pdobject, (t_method)glob_loadpreferences,
+        gensym("load-preferences"), A_DEFSYM, 0);
     class_addmethod(glob_pdobject, (t_method)glob_savepreferences,
-        gensym("save-preferences"), 0);
+        gensym("save-preferences"), A_DEFSYM, 0);
+    class_addmethod(glob_pdobject, (t_method)glob_forgetpreferences,
+        gensym("forget-preferences"), A_DEFSYM, 0);
     class_addmethod(glob_pdobject, (t_method)glob_zoom_open,
         gensym("zoom-open"), A_FLOAT, 0);
     class_addmethod(glob_pdobject, (t_method)glob_version,
