@@ -12,7 +12,9 @@
 #include <string.h>
 #include "m_pd.h"
 #include "s_stuff.h"
+#ifdef __APPLE__
 #include <jack/weakjack.h>
+#endif
 #include <jack/jack.h>
 #include <regex.h>
 
@@ -30,9 +32,11 @@ static jack_port_t *input_port[MAX_JACK_PORTS];
 static jack_port_t *output_port[MAX_JACK_PORTS];
 static int outport_count = 0;
 static jack_client_t *jack_client = NULL;
+static char * desired_client_name = NULL;
 char *jack_client_names[MAX_CLIENTS];
 static int jack_dio_error;
 static t_audiocallback jack_callback;
+static int jack_should_autoconnect = 1;
 pthread_mutex_t jack_mutex;
 pthread_cond_t jack_sem;
 
@@ -52,13 +56,13 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
             'jack_default_audio_sample_t' are actually the same type??? */
         if (sizeof(t_sample)==sizeof(jack_default_audio_sample_t))
         {
-            for (j = 0; j < sys_outchannels;  j++)
+            for (j = 0; j < STUFF->st_outchannels;  j++)
             {
                 if (out = jack_port_get_buffer(output_port[j], nframes))
                     memcpy(out, jack_outbuf + (j * BUF_JACK),
                         sizeof (jack_default_audio_sample_t) * nframes);
             }
-            for (j = 0; j < sys_inchannels; j++)
+            for (j = 0; j < STUFF->st_inchannels; j++)
             {
                 if (in = jack_port_get_buffer(input_port[j], nframes))
                     memcpy(jack_inbuf + (j * BUF_JACK), in,
@@ -69,7 +73,7 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
         {
             unsigned int frame=0;
             t_sample*data;
-            for (j = 0; j < sys_outchannels;  j++)
+            for (j = 0; j < STUFF->st_outchannels;  j++)
             {
                 if (out = jack_port_get_buffer (output_port[j], nframes))
                 {
@@ -78,7 +82,7 @@ static int pollprocess(jack_nframes_t nframes, void *arg)
                         *out++ = *data++;
                 }
             }
-            for (j = 0; j < sys_inchannels; j++)
+            for (j = 0; j < STUFF->st_inchannels; j++)
             {
                 if (in = jack_port_get_buffer( input_port[j], nframes))
                 {
@@ -118,31 +122,31 @@ static int callbackprocess(jack_nframes_t nframes, void *arg)
             nframes, DEFDACBLKSIZE);
         nframes -= (nframes % DEFDACBLKSIZE);
     }
-    for (chan = 0; chan < sys_inchannels; chan++)
+    for (chan = 0; chan < STUFF->st_inchannels; chan++)
         in[chan] = jack_port_get_buffer(input_port[chan], nframes);
-    for (chan = 0; chan < sys_outchannels; chan++)
+    for (chan = 0; chan < STUFF->st_outchannels; chan++)
         out[chan] = jack_port_get_buffer(output_port[chan], nframes);
     for (n = 0; n < nframes; n += DEFDACBLKSIZE)
     {
         t_sample *fp;
-        for (chan = 0; chan < sys_inchannels; chan++)
+        for (chan = 0; chan < STUFF->st_inchannels; chan++)
             if (in[chan])
         {
-            for (fp = sys_soundin + chan*DEFDACBLKSIZE,
+            for (fp = STUFF->st_soundin + chan*DEFDACBLKSIZE,
                 jp = in[chan] + n, j=0; j < DEFDACBLKSIZE; j++)
                     *fp++ = *jp++;
         }
-        for (chan = 0; chan < sys_outchannels; chan++)
+        for (chan = 0; chan < STUFF->st_outchannels; chan++)
         {
-            for (fp = sys_soundout + chan*DEFDACBLKSIZE,
+            for (fp = STUFF->st_soundout + chan*DEFDACBLKSIZE,
                 j = 0; j < DEFDACBLKSIZE; j++)
                     *fp++ = 0;
         }
         (*jack_callback)();
-        for (chan = 0; chan < sys_outchannels; chan++)
+        for (chan = 0; chan < STUFF->st_outchannels; chan++)
             if (out[chan])
         {
-            for (fp = sys_soundout + chan*DEFDACBLKSIZE, jp = out[chan] + n,
+            for (fp = STUFF->st_soundout + chan*DEFDACBLKSIZE, jp = out[chan] + n,
                 j=0; j < DEFDACBLKSIZE; j++)
                     *jp++ = *fp++;
         }
@@ -153,7 +157,7 @@ static int callbackprocess(jack_nframes_t nframes, void *arg)
 static int
 jack_srate (jack_nframes_t srate, void *arg)
 {
-        sys_dacsr = srate;
+        STUFF->st_dacsr = srate;
         return 0;
 }
 
@@ -265,7 +269,7 @@ static int jack_connect_ports(char* client)
                                  NULL, JackPortIsOutput);
     if (jack_ports)
     {
-        for (i=0;jack_ports[i] != NULL && i < sys_inchannels;i++)
+        for (i=0;jack_ports[i] != NULL && i < STUFF->st_inchannels;i++)
             if (jack_connect (jack_client, jack_ports[i],
                jack_port_name (input_port[i])))
                   error ("JACK: cannot connect input ports %s -> %s",
@@ -276,7 +280,7 @@ static int jack_connect_ports(char* client)
                                  NULL, JackPortIsInput);
     if (jack_ports)
     {
-        for (i=0;jack_ports[i] != NULL && i < sys_outchannels;i++)
+        for (i=0;jack_ports[i] != NULL && i < STUFF->st_outchannels;i++)
           if (jack_connect (jack_client, jack_port_name (output_port[i]),
             jack_ports[i]))
               error( "JACK: cannot connect output ports %s -> %s",
@@ -305,13 +309,11 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
     int srate;
     jack_status_t status;
 
-#ifdef __APPLE__
     if (!jack_client_open)
     {
-        error("Can't open Jack (it seems not to be installed on this Mac)");
+        error("Can't open Jack (it seems not to be installed)");
         return 1;
     }
-#endif
 
     jack_dio_error = 0;
 
@@ -334,43 +336,32 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
         whether or not this is desirable; see long Pd list thread started by
         yvan volochine, June 2013) */
     if (!jack_client) {
-        do {
-          sprintf(client_name,"pure_data_%d",client_iterator);
-          client_iterator++;
-          jack_client = jack_client_open (client_name, JackNoStartServer,
-            &status, NULL);
-          if (status & JackServerFailed) {
-            error("JACK: unable to connect to JACK server.  Is JACK running?");
+        if (!desired_client_name || !strlen(desired_client_name))
+            jack_client_name("pure_data");
+        jack_client = jack_client_open (desired_client_name, JackNoStartServer,
+          &status, NULL);
+        if (status & JackFailure) {
+            error("JACK: Failure.  Is JACK running?");
+            verbose(1, "JACK: Returned status is: %d", status);
             jack_client=NULL;
-            break;
-          }
-        } while (status & JackNameNotUnique);
-
-        if(status) {
-          if (status & JackServerStarted) {
-            verbose(1, "JACK: started server");
-          } else {
-            error("JACK: server returned status %d", status);
-          }
-        }
-        verbose(1, "JACK: started server as '%s'", client_name);
-
-        if (!jack_client) {
             /* jack spits out enough messages already, do not warn */
-            sys_inchannels = sys_outchannels = 0;
+            STUFF->st_inchannels = STUFF->st_outchannels = 0;
             return 1;
         }
+        if (status & JackNameNotUnique)
+            jack_client_name(jack_get_client_name(jack_client));
+        verbose(1, "JACK: registered as '%s'", desired_client_name);
 
-        sys_inchannels = inchans;
-        sys_outchannels = outchans;
+        STUFF->st_inchannels = inchans;
+        STUFF->st_outchannels = outchans;
         if (jack_inbuf)
             free(jack_inbuf);
-        if (sys_inchannels)
-            jack_inbuf = calloc(sizeof(t_sample), sys_inchannels * BUF_JACK);
+        if (STUFF->st_inchannels)
+            jack_inbuf = calloc(sizeof(t_sample), STUFF->st_inchannels * BUF_JACK);
         if (jack_outbuf)
             free(jack_outbuf);
-        if (sys_outchannels)
-            jack_outbuf = calloc(sizeof(t_sample), sys_outchannels * BUF_JACK);
+        if (STUFF->st_outchannels)
+            jack_outbuf = calloc(sizeof(t_sample), STUFF->st_outchannels * BUF_JACK);
 
         jack_get_clients();
 
@@ -400,9 +391,9 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
 
         jack_on_shutdown (jack_client, jack_shutdown, 0);
 
-        for (j=0; j<sys_inchannels; j++)
+        for (j=0; j<STUFF->st_inchannels; j++)
              input_port[j]=NULL;
-        for (j=0; j<sys_outchannels; j++)
+        for (j=0; j<STUFF->st_outchannels; j++)
              output_port[j] = NULL;
 
         new_jack = 1;
@@ -414,7 +405,7 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
     */
 
     srate = jack_get_sample_rate (jack_client);
-    sys_dacsr = srate;
+    STUFF->st_dacsr = srate;
 
     /* create the ports */
 
@@ -427,7 +418,7 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
         {
           error("JACK: can only register %d input ports (of %d requested)",
             j, inchans);
-          sys_inchannels = inchans = j;
+          STUFF->st_inchannels = inchans = j;
           break;
         }
     }
@@ -441,7 +432,7 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
         {
           error("JACK: can only register %d output ports (of %d requested)",
             j, outchans);
-          sys_outchannels = outchans = j;
+          STUFF->st_outchannels = outchans = j;
           break;
         }
     }
@@ -453,7 +444,7 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
     {
         if (jack_activate (jack_client)) {
             error("cannot activate client");
-            sys_inchannels = sys_outchannels = 0;
+            STUFF->st_inchannels = STUFF->st_outchannels = 0;
             return 1;
         }
 
@@ -461,7 +452,7 @@ jack_open_audio(int inchans, int outchans, int rate, t_audiocallback callback)
             memset(jack_outbuf + j * BUF_JACK, 0,
                 BUF_JACK * sizeof(t_sample));
 
-        if (jack_client_names[0])
+        if (jack_client_names[0] && jack_should_autoconnect)
             jack_connect_ports(jack_client_names[0]);
 
         pthread_mutex_init(&jack_mutex, NULL);
@@ -499,7 +490,7 @@ int jack_send_dacs(void)
     int timenow;
     int timeref = sys_getrealtime();
     if (!jack_client) return SENDDACS_NO;
-    if (!sys_inchannels && !sys_outchannels) return (SENDDACS_NO);
+    if (!STUFF->st_inchannels && !STUFF->st_outchannels) return (SENDDACS_NO);
     if (jack_dio_error)
     {
         sys_log_error(ERR_RESYNC);
@@ -516,15 +507,15 @@ int jack_send_dacs(void)
     }
     jack_started = 1;
 
-    fp = sys_soundout;
-    for (j = 0; j < sys_outchannels; j++)
+    fp = STUFF->st_soundout;
+    for (j = 0; j < STUFF->st_outchannels; j++)
     {
         memcpy(jack_outbuf + (j * BUF_JACK) + jack_filled, fp,
             DEFDACBLKSIZE*sizeof(t_sample));
         fp += DEFDACBLKSIZE;
     }
-    fp = sys_soundin;
-    for (j = 0; j < sys_inchannels; j++)
+    fp = STUFF->st_soundin;
+    for (j = 0; j < STUFF->st_inchannels; j++)
     {
         memcpy(fp, jack_inbuf + (j * BUF_JACK) + jack_filled,
             DEFDACBLKSIZE*sizeof(t_sample));
@@ -537,7 +528,7 @@ int jack_send_dacs(void)
     {
         rtnval = SENDDACS_SLEPT;
     }
-    memset(sys_soundout, 0, DEFDACBLKSIZE*sizeof(t_sample)*sys_outchannels);
+    memset(STUFF->st_soundout, 0, DEFDACBLKSIZE*sizeof(t_sample)*STUFF->st_outchannels);
     return rtnval;
 }
 
@@ -561,4 +552,21 @@ void jack_listdevs( void)
     post("device listing not implemented for jack yet\n");
 }
 
+void jack_autoconnect(int v)
+{
+    jack_should_autoconnect = v;
+}
+
+void jack_client_name(char *name)
+{
+    if (desired_client_name) {
+      free(desired_client_name);
+      desired_client_name = NULL;
+    }
+    if (name) {
+      desired_client_name = (char*)getbytes(strlen(name) + 1);
+      strcpy(desired_client_name, name);
+    }
+}
+
 #endif /* JACK */
