@@ -1,3 +1,4 @@
+#pragma once
 /* Copyright (c) 1997-1999 Miller Puckette.
 * For information on usage and redistribution, and for a DISCLAIMER OF ALL
 * WARRANTIES, see the file, "LICENSE.txt," in this distribution.  */
@@ -5,7 +6,7 @@
 /* Audio and MIDI I/O, and other scheduling and system stuff. */
 
 /* NOTE: this file describes Pd implementation details which may change
-in future releases.  The public (stable) API is in m_pd.h. */  
+in future releases.  The public (stable) API is in m_pd.h. */
 
 /* in s_path.c */
 
@@ -16,14 +17,11 @@ typedef struct _namelist    /* element in a linked list of stored strings */
 } t_namelist;
 
 t_namelist *namelist_append(t_namelist *listwas, const char *s, int allowdup);
-t_namelist *namelist_append_files(t_namelist *listwas, const char *s);
+EXTERN t_namelist *namelist_append_files(t_namelist *listwas, const char *s);
 void namelist_free(t_namelist *listwas);
 char *namelist_get(t_namelist *namelist, int n);
 void sys_setextrapath(const char *p);
 extern int sys_usestdpath;
-extern t_namelist *sys_externlist;
-extern t_namelist *sys_searchpath;
-extern t_namelist *sys_helppath;
 int sys_open_absolute(const char *name, const char* ext,
     char *dirresult, char **nameresult, unsigned int size, int bin, int *fdp);
 int sys_trytoopenone(const char *dir, const char *name, const char* ext,
@@ -32,8 +30,8 @@ t_symbol *sys_decodedialog(t_symbol *s);
 
 /* s_file.c */
 
-void sys_loadpreferences( void);
-void sys_savepreferences( void);
+void sys_loadpreferences(const char *filename, int startingup);
+void sys_savepreferences(const char *filename);
 extern int sys_defeatrt;
 extern t_symbol *sys_flags;
 
@@ -41,11 +39,10 @@ extern t_symbol *sys_flags;
 extern int sys_debuglevel;
 extern int sys_verbose;
 extern int sys_noloadbang;
-extern int sys_nogui;
+EXTERN int sys_havegui( void);
 extern char *sys_guicmd;
 
 EXTERN int sys_nearestfontsize(int fontsize);
-EXTERN int sys_hostfontsize(int fontsize);
 
 extern int sys_defaultfont;
 EXTERN t_symbol *sys_libdir;    /* library directory for auxilliary files */
@@ -53,28 +50,21 @@ extern t_symbol *sys_guidir;    /* directory holding pd_gui, u_pdsend, etc */
 
 /* s_loader.c */
 
-typedef int (*loader_t)(t_canvas *canvas, char *classname); /* callback type */
-EXTERN int sys_load_lib(t_canvas *canvas, char *filename);
+typedef int (*loader_t)(t_canvas *canvas, const char *classname, const char*path); /* callback type */
+EXTERN int sys_load_lib(t_canvas *canvas, const char *classname);
 EXTERN void sys_register_loader(loader_t loader);
 
 /* s_audio.c */
 
 #define SENDDACS_NO 0           /* return values for sys_send_dacs() */
-#define SENDDACS_YES 1 
+#define SENDDACS_YES 1
 #define SENDDACS_SLEPT 2
 
 #define DEFDACBLKSIZE 64
-extern int sys_schedblocksize;  /* audio block size for scheduler */
 extern int sys_hipriority;      /* real-time flag, true if priority boosted */
-EXTERN t_sample *sys_soundout;
-EXTERN t_sample *sys_soundin;
-extern int sys_inchannels;
-extern int sys_outchannels;
-extern int sys_advance_samples; /* scheduler advance in samples */
-extern int sys_blocksize;       /* audio I/O block size in sample frames */
-extern t_float sys_dacsr;
 extern int sys_schedadvance;
 extern int sys_sleepgrain;
+extern int sys_advance_samples;    /* scheduler advance in samples */
 EXTERN void sys_set_audio_settings(int naudioindev, int *audioindev,
     int nchindev, int *chindev,
     int naudiooutdev, int *audiooutdev, int nchoutdev, int *choutdev,
@@ -89,6 +79,9 @@ EXTERN void sys_close_audio(void);
     /* return true if the interface prefers always being open (ala jack) : */
 EXTERN int audio_shouldkeepopen( void);
 EXTERN int audio_isopen( void);     /* true if audio interface is open */
+EXTERN int sys_audiodevnametonumber(int output, const char *name);
+EXTERN void sys_audiodevnumbertoname(int output, int devno, char *name,
+    int namesize);
 
 int sys_send_dacs(void);
 void sys_reportidle(void);
@@ -99,7 +92,7 @@ void sys_listdevs(void);
 void sys_setblocksize(int n);
 
 EXTERN void sys_get_audio_devs(char *indevlist, int *nindevs,
-                          char *outdevlist, int *noutdevs, int *canmulti, int *cancallback, 
+                          char *outdevlist, int *noutdevs, int *canmulti, int *cancallback,
                           int maxndev, int devdescsize);
 EXTERN void sys_get_audio_apis(char *buf);
 
@@ -117,10 +110,13 @@ EXTERN void sys_open_midi(int nmidiin, int *midiinvec,
 
 EXTERN void sys_get_midi_apis(char *buf);
 EXTERN void sys_get_midi_devs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, 
+    char *outdevlist, int *noutdevs,
    int maxndev, int devdescsize);
 EXTERN void sys_get_midi_params(int *pnmidiindev, int *pmidiindev,
     int *pnmidioutdev, int *pmidioutdev);
+EXTERN int sys_mididevnametonumber(int output, const char *name);
+EXTERN void sys_mididevnumbertoname(int output, int devno, char *name,
+    int namesize);
 
 EXTERN void sys_reopen_midi( void);
 EXTERN void sys_close_midi( void);
@@ -161,13 +157,14 @@ EXTERN void sys_log_error(int type);
 #define ERR_DATALATE 4
 
 #define SCHED_AUDIO_NONE 0
-#define SCHED_AUDIO_POLL 1 
+#define SCHED_AUDIO_POLL 1
 #define SCHED_AUDIO_CALLBACK 2
 void sched_set_using_audio(int flag);
 
 /* s_inter.c */
 
 EXTERN void sys_microsleep(int microsec);
+EXTERN void sys_init_fdpoll(void);
 
 EXTERN void sys_bail(int exitcode);
 EXTERN int sys_pollgui(void);
@@ -206,7 +203,7 @@ void sys_setalarm(int microsec);
     likely to offer a working device takes precedence so that if you
     start up Pd for the first time there's a reasonable chance you'll have
     sound.  (You'd think portaudio would be best but it seems to default
-    to jack on linux, and and on Windows we only use it for ASIO). 
+    to jack on linux, and and on Windows we only use it for ASIO).
     If nobody shows up, define DUMMY and make it the default.*/
 #if defined(USEAPI_MMIO)
 # define API_DEFAULT API_MMIO
@@ -229,13 +226,13 @@ void sys_setalarm(int microsec);
 #elif defined(USEAPI_JACK)
 # define API_DEFAULT API_JACK
 # define API_DEFSTRING "Jack audio connection kit"
-#else 
+#else
 # ifndef USEAPI_DUMMY   /* we need at least one so bring in the dummy */
 # define USEAPI_DUMMY
 # endif /* USEAPI_DUMMY */
 # define API_DEFAULT API_DUMMY
 # define API_DEFSTRING "dummy audio"
-#endif 
+#endif
 
 #define DEFAULTAUDIODEV 0
 
@@ -262,10 +259,9 @@ int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
     int indeviceno, int outdeviceno, t_audiocallback callback);
 void pa_close_audio(void);
 int pa_send_dacs(void);
-void sys_reportidle(void);
 void pa_listdevs(void);
 void pa_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize);
 
 int oss_open_audio(int naudioindev, int *audioindev, int nchindev,
@@ -275,7 +271,7 @@ void oss_close_audio(void);
 int oss_send_dacs(void);
 void oss_reportidle(void);
 void oss_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize);
 
 int alsa_open_audio(int naudioindev, int *audioindev, int nchindev,
@@ -285,7 +281,7 @@ void alsa_close_audio(void);
 int alsa_send_dacs(void);
 void alsa_reportidle(void);
 void alsa_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize);
 
 int jack_open_audio(int wantinchans, int wantoutchans, int srate,
@@ -294,9 +290,11 @@ void jack_close_audio(void);
 int jack_send_dacs(void);
 void jack_reportidle(void);
 void jack_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize);
 void jack_listdevs(void);
+void jack_client_name(char *name);
+void jack_autoconnect(int);
 
 int mmio_open_audio(int naudioindev, int *audioindev,
     int nchindev, int *chindev, int naudiooutdev, int *audiooutdev,
@@ -305,7 +303,7 @@ void mmio_close_audio( void);
 void mmio_reportidle(void);
 int mmio_send_dacs(void);
 void mmio_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize);
 
 int audiounit_open_audio(int naudioindev, int *audioindev, int nchindev,
@@ -315,7 +313,7 @@ void audiounit_close_audio(void);
 int audiounit_send_dacs(void);
 void audiounit_listdevs(void);
 void audiounit_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize);
 
 int esd_open_audio(int naudioindev, int *audioindev, int nchindev,
@@ -325,7 +323,7 @@ void esd_close_audio(void);
 int esd_send_dacs(void);
 void esd_listdevs(void);
 void esd_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize);
 
 int dummy_open_audio(int nin, int nout, int sr);
@@ -362,8 +360,6 @@ extern int sys_printtostderr;
 
 /* jsarlo { */
 
-EXTERN double sys_time;
-EXTERN double sys_time_per_dsp_tick;
 EXTERN int sys_externalschedlib;
 
 EXTERN t_sample* get_sys_soundout(void ) ;
@@ -379,10 +375,8 @@ EXTERN int* get_sys_schedadvance(void ) ;
 EXTERN void sys_clearhist(void );
 EXTERN void sys_initmidiqueue(void );
 EXTERN int sys_addhist(int phase);
-EXTERN void sys_setmiditimediff(double inbuftime, double outbuftime);
-EXTERN void sched_tick(double next_sys_time);
+EXTERN void sched_tick( void);
 EXTERN void sys_pollmidiqueue(void );
-EXTERN int sys_pollgui(void );
 EXTERN void sys_setchsr(int chin, int chout, int sr);
 
 EXTERN void inmidi_realtimein(int portno, int cmd);
@@ -401,4 +395,22 @@ EXTERN void inmidi_polyaftertouch(int portno,
                                   int pitch,
                                   int value);
 /* } jsarlo */
-extern t_widgetbehavior text_widgetbehavior;
+EXTERN int sys_zoom_open;
+
+struct _instancestuff
+{
+    t_namelist *st_externlist;
+    t_namelist *st_searchpath;
+    t_namelist *st_staticpath;
+    t_namelist *st_helppath;
+    int st_schedblocksize;      /* audio block size for scheduler */
+    int st_blocksize;           /* audio I/O block size in sample frames */
+    t_float st_dacsr;           /* I/O sample rate */
+    int st_inchannels;
+    int st_outchannels;
+    t_sample *st_soundout;
+    t_sample *st_soundin;
+    double st_time_per_dsp_tick;    /* obsolete - included for GEM?? */
+};
+
+#define STUFF (pd_this->pd_stuff)
