@@ -489,7 +489,8 @@ typedef struct _oscformat
 static void oscformat_set(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
 {
     char buf[MAXPDSTRING];
-    int i, newsize;
+    int i;
+    unsigned long newsize;
     *x->x_pathbuf = 0;
     buf[0] = '/';
     for (i = 0; i < argc; i++)
@@ -500,7 +501,7 @@ static void oscformat_set(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
         if ((newsize = strlen(buf) + strlen(x->x_pathbuf) + 1) > x->x_pathsize)
         {
             x->x_pathbuf = resizebytes(x->x_pathbuf, x->x_pathsize, newsize);
-            x->x_pathsize = newsize;
+            x->x_pathsize = (int)newsize;
         }
         strcat(x->x_pathbuf, buf);
     }
@@ -574,7 +575,7 @@ static void oscformat_list(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
         j++;
         ndata++;
     }
-    datastart = ROUNDUPTO4(strlen(x->x_pathbuf)+1) + ROUNDUPTO4(ndata + 2);
+    datastart = (int)(ROUNDUPTO4(strlen(x->x_pathbuf)+1) + ROUNDUPTO4(ndata + 2));
     msgsize = datastart + msgindex;
     msg = (t_atom *)alloca(msgsize * sizeof(t_atom));
     putstring(msg, &typeindex, x->x_pathbuf);
@@ -680,6 +681,170 @@ void oscformat_setup(void)
     class_addlist(oscformat_class, oscformat_list);
 }
 
+
+/* ---------- fudiparse - parse bytelists to to FUDI messages ----------------- */
+
+static t_class *fudiparse_class;
+
+typedef struct _fudiparse {
+  t_object  x_obj;
+  t_outlet *x_msgout;
+  char     *x_bytes;
+  size_t    x_numbytes;
+} t_fudiparse;
+
+static void fudiparse_binbufout(t_fudiparse *x, t_binbuf *b)
+{
+  int msg, natom = binbuf_getnatom(b);
+  t_atom *at = binbuf_getvec(b);
+  for (msg = 0; msg < natom;) {
+    int emsg;
+    for (emsg = msg; emsg < natom && at[emsg].a_type != A_COMMA
+           && at[emsg].a_type != A_SEMI; emsg++)
+      ;
+    if (emsg > msg) {
+      int i;
+      /* check for illegal atoms */
+      for (i = msg; i < emsg; i++)
+        if (at[i].a_type == A_DOLLAR || at[i].a_type == A_DOLLSYM) {
+          pd_error(x, "fudiparse: got dollar sign in message");
+          goto nodice;
+        }
+
+      if (at[msg].a_type == A_FLOAT) {
+        if (emsg > msg + 1)
+          outlet_list(x->x_msgout, 0, emsg-msg, at + msg);
+        else outlet_float(x->x_msgout, at[msg].a_w.w_float);
+      }
+      else if (at[msg].a_type == A_SYMBOL) {
+        outlet_anything(x->x_msgout, at[msg].a_w.w_symbol,
+                        emsg-msg-1, at + msg + 1);
+      }
+    }
+  nodice:
+    msg = emsg + 1;
+  }
+}
+static void fudiparse_list(t_fudiparse *x, t_symbol*s, int argc, t_atom*argv) {
+  size_t len = argc;
+  t_binbuf* bbuf = binbuf_new();
+  char*cbuf;
+  if((size_t)argc > x->x_numbytes) {
+    freebytes(x->x_bytes, x->x_numbytes);
+    x->x_numbytes = argc;
+    x->x_bytes = getbytes(x->x_numbytes);
+  }
+  cbuf = x->x_bytes;
+
+  while(argc--) {
+    char b = atom_getfloat(argv++);
+    *cbuf++ = b;
+  }
+  binbuf_text(bbuf, x->x_bytes, len);
+
+  fudiparse_binbufout(x, bbuf);
+
+  binbuf_free(bbuf);
+}
+
+static void fudiparse_free(t_fudiparse *x) {
+  freebytes(x->x_bytes, x->x_numbytes);
+  x->x_bytes = NULL;
+  x->x_numbytes = 0;
+}
+
+static void *fudiparse_new(void) {
+  t_fudiparse *x = (t_fudiparse *)pd_new(fudiparse_class);
+  x->x_msgout = outlet_new(&x->x_obj, 0);
+  x->x_numbytes = 1024;
+  x->x_bytes = getbytes(x->x_numbytes);
+  return (void *)x;
+}
+
+void fudiparse_setup(void) {
+  fudiparse_class = class_new(gensym("fudiparse"),
+                              (t_newmethod)fudiparse_new,
+                              (t_method)fudiparse_free,
+                              sizeof(t_fudiparse), CLASS_DEFAULT,
+                              0);
+  class_addlist(fudiparse_class, fudiparse_list);
+}
+/* --------- oscformat - format Pd (FUDI) messages to bytelists ------------ */
+
+static t_class *fudiformat_class;
+
+typedef struct _fudiformat {
+  t_object  x_obj;
+  t_outlet *x_msgout;
+  t_atom   *x_atoms;
+  size_t    x_numatoms;
+  int       x_udp;
+} t_fudiformat;
+
+static void fudiformat_any(t_fudiformat *x, t_symbol*s, int argc, t_atom*argv) {
+  char *buf;
+  int length;
+  int i;
+  t_atom at;
+  t_binbuf*bbuf = binbuf_new();
+  SETSYMBOL(&at, s);
+  binbuf_add(bbuf, 1, &at);
+
+  binbuf_add(bbuf, argc, argv);
+
+  if(!x->x_udp) {
+    SETSEMI(&at);
+    binbuf_add(bbuf, 1, &at);
+  }
+  binbuf_gettext(bbuf, &buf, &length);
+  binbuf_free(bbuf);
+
+  if((size_t)length>x->x_numatoms) {
+    freebytes(x->x_atoms, sizeof(*x->x_atoms) * x->x_numatoms);
+    x->x_numatoms = length;
+    x->x_atoms = getbytes(sizeof(*x->x_atoms) * x->x_numatoms);
+  }
+
+  for(i=0; i<length; i++) {
+    SETFLOAT(x->x_atoms+i, buf[i]);
+  }
+  freebytes(buf, length);
+  outlet_list(x->x_msgout, 0, length, x->x_atoms);
+}
+
+static void fudiformat_free(t_fudiformat *x) {
+  freebytes(x->x_atoms, sizeof(*x->x_atoms) * x->x_numatoms);
+  x->x_atoms = NULL;
+  x->x_numatoms = 0;
+}
+
+static void *fudiformat_new(t_symbol*s) {
+  t_fudiformat *x = (t_fudiformat *)pd_new(fudiformat_class);
+  x->x_msgout = outlet_new(&x->x_obj, 0);
+  x->x_numatoms = 1024;
+  x->x_atoms = getbytes(sizeof(*x->x_atoms) * x->x_numatoms);
+  if (gensym("-u") == s)
+    x->x_udp = 1;
+  else if (gensym("-t") == s)
+    x->x_udp = 0;
+  else if (gensym("") != s) {
+    pd_error(x, "fudiformat: unsupported mode '%s'", s->s_name);
+  }
+
+  return (void *)x;
+}
+
+static void fudiformat_setup(void) {
+  fudiformat_class = class_new(gensym("fudiformat"),
+                               (t_newmethod)fudiformat_new,
+                               (t_method)fudiformat_free,
+                               sizeof(t_fudiformat), CLASS_DEFAULT,
+                               A_DEFSYMBOL, 0);
+  class_addanything(fudiformat_class, fudiformat_any);
+}
+
+
+
 void x_misc_setup(void)
 {
     random_setup();
@@ -689,4 +854,6 @@ void x_misc_setup(void)
     realtime_setup();
     oscparse_setup();
     oscformat_setup();
+    fudiparse_setup();
+    fudiformat_setup();
 }
