@@ -4,10 +4,12 @@ package provide pd_connect 0.1
 namespace eval ::pd_connect:: {
     variable pd_socket
     variable cmds_from_pd ""
+    variable plugin_dispatch_receivers
 
     namespace export to_pd
     namespace export create_socket
     namespace export pdsend
+    namespace export register_plugin_dispatch_receiver
 }
 
 # TODO figure out how to escape { } properly
@@ -22,9 +24,11 @@ proc ::pd_connect::to_pd {port {host localhost}} {
     variable pd_socket
     ::pdwindow::debug "'pd-gui' connecting to 'pd' on localhost $port ...\n"
     if {[catch {set pd_socket [socket $host $port]}]} {
-        puts stderr "WARNING: connect to pd failed, retrying port $host:$port."
-        after 1000 ::pd_connect::to_pd $port $host
-        return
+        if {$host ne "localhost" || [catch {set pd_socket [socket 127.0.0.1 $port]}]} {
+            puts stderr "WARNING: connect to pd failed, retrying port $host:$port."
+            after 1000 ::pd_connect::to_pd $port $host
+            return
+        }
     }
     ::pd_connect::configure_socket $pd_socket
 }
@@ -58,6 +62,12 @@ proc ::pd_connect::pdsend {message} {
     }
 }
 
+# register a plugin to receive messages from running Pd patches
+proc ::pd_connect::register_plugin_dispatch_receiver { nameatom callback } {
+    variable plugin_dispatch_receivers
+    lappend plugin_dispatch_receivers($nameatom) $callback
+}
+
 proc ::pd_connect::pd_readsocket {} {
      variable pd_socket
      variable cmds_from_pd
