@@ -11,7 +11,7 @@ package require pd_menucommands
 # use {} quotes so that $::focused_window is interpreted when the menu item
 # is called, not when the command is mapped to the menu item.  This is the
 # opposite of the 'bind' commands in pd_bindings.tcl
-    
+
 namespace eval ::pd_menus:: {
     variable accelerator
     variable menubar ".menubar"
@@ -26,7 +26,7 @@ namespace eval ::pd_menus:: {
 }
 
 # ------------------------------------------------------------------------------
-# 
+#
 proc ::pd_menus::create_menubar {} {
     variable accelerator
     variable menubar
@@ -36,14 +36,19 @@ proc ::pd_menus::create_menubar {} {
         set accelerator "Ctrl"
     }
     menu $menubar
+    if {$::windowingsystem eq "aqua"} {create_apple_menu $menubar}
     set menulist "file edit put find media window help"
-    foreach mymenu $menulist {    
+    foreach mymenu $menulist {
+        if {$mymenu eq "find"} {
+            set underlined 3
+        } {
+            set underlined 0
+        }
         menu $menubar.$mymenu
         $menubar add cascade -label [_ [string totitle $mymenu]] \
-            -menu $menubar.$mymenu
+            -underline $underlined -menu $menubar.$mymenu
         [format build_%s_menu $mymenu] $menubar.$mymenu
     }
-    if {$::windowingsystem eq "aqua"} {create_apple_menu $menubar}
     if {$::windowingsystem eq "win32"} {create_system_menu $menubar}
     . configure -menu $menubar
 }
@@ -52,15 +57,17 @@ proc ::pd_menus::configure_for_pdwindow {} {
     variable menubar
     # these are meaningless for the Pd window, so disable them
     # File menu
+    $menubar.file entryconfigure [_ "Close"] -state disabled
     $menubar.file entryconfigure [_ "Save"] -state disabled
     $menubar.file entryconfigure [_ "Save As..."] -state normal
     $menubar.file entryconfigure [_ "Print..."] -state disabled
-    $menubar.file entryconfigure [_ "Close"] -state disabled
+
     # Edit menu
     $menubar.edit entryconfigure [_ "Duplicate"] -state disabled
-    $menubar.edit entryconfigure [_ "Tidy Up"] -state disabled
+    $menubar.edit entryconfigure [_ "Font"] -state normal
     $menubar.edit entryconfigure [_ "Zoom In"] -state disabled
     $menubar.edit entryconfigure [_ "Zoom Out"] -state disabled
+    $menubar.edit entryconfigure [_ "Tidy Up"] -state disabled
     $menubar.edit entryconfigure [_ "Edit Mode"] -state disabled
     pdtk_canvas_editmode .pdwindow 0
     # Undo/Redo change names, they need to have the asterisk (*) after
@@ -71,48 +78,65 @@ proc ::pd_menus::configure_for_pdwindow {} {
         # catch errors that happen when trying to disable separators
         catch {$menubar.put entryconfigure $i -state disabled }
     }
+    # Help menu
+    # make sure "List of objects..." is enabled, it sometimes greys out on Mac
+    $menubar.help entryconfigure [_ "List of objects..."] -state normal
 }
 
 proc ::pd_menus::configure_for_canvas {mytoplevel} {
     variable menubar
     # File menu
+    $menubar.file entryconfigure [_ "Close"] -state normal
     $menubar.file entryconfigure [_ "Save"] -state normal
     $menubar.file entryconfigure [_ "Save As..."] -state normal
     $menubar.file entryconfigure [_ "Print..."] -state normal
-    $menubar.file entryconfigure [_ "Close"] -state normal
     # Edit menu
     $menubar.edit entryconfigure [_ "Duplicate"] -state normal
-    $menubar.edit entryconfigure [_ "Tidy Up"] -state normal
+    $menubar.edit entryconfigure [_ "Font"] -state normal
     $menubar.edit entryconfigure [_ "Zoom In"] -state normal
     $menubar.edit entryconfigure [_ "Zoom Out"] -state normal
+    $menubar.edit entryconfigure [_ "Tidy Up"] -state normal
     $menubar.edit entryconfigure [_ "Edit Mode"] -state normal
     pdtk_canvas_editmode $mytoplevel $::editmode($mytoplevel)
     # Put menu
     for {set i 0} {$i <= [$menubar.put index end]} {incr i} {
         # catch errors that happen when trying to disable separators
         if {[$menubar.put type $i] ne "separator"} {
-            $menubar.put entryconfigure $i -state normal 
+            $menubar.put entryconfigure $i -state normal
         }
     }
     update_undo_on_menu $mytoplevel
+    # Help menu
+    # make sure "List of objects..." is enabled, it sometimes greys out on Mac
+    $menubar.help entryconfigure [_ "List of objects..."] -state normal
 }
 
 proc ::pd_menus::configure_for_dialog {mytoplevel} {
     variable menubar
     # these are meaningless for the dialog panels, so disable them except for
-    # the ones that make senes in the Find dialog panel
+    # the ones that make sense in the Find dialog panel and it's canvas
     # File menu
-    if {$mytoplevel ne ".find"} {
+    $menubar.file entryconfigure [_ "Close"] -state disabled
+    if {$mytoplevel eq ".find"} {
+        # these bindings are passed through Find to it's target search window
+        $menubar.file entryconfigure [_ "Save As..."] -state disabled
+        if {$mytoplevel ne ".pdwindow"} {
+            # these don't do anything in the pdwindow
+            $menubar.file entryconfigure [_ "Save"] -state disabled
+            $menubar.file entryconfigure [_ "Print..."] -state disabled
+        }
+    } else {
         $menubar.file entryconfigure [_ "Save"] -state disabled
         $menubar.file entryconfigure [_ "Save As..."] -state disabled
         $menubar.file entryconfigure [_ "Print..."] -state disabled
     }
-    $menubar.file entryconfigure [_ "Close"] -state disabled
+
     # Edit menu
+    $menubar.edit entryconfigure [_ "Font"] -state disabled
     $menubar.edit entryconfigure [_ "Duplicate"] -state disabled
-    $menubar.edit entryconfigure [_ "Tidy Up"] -state disabled
     $menubar.edit entryconfigure [_ "Zoom In"] -state disabled
     $menubar.edit entryconfigure [_ "Zoom Out"] -state disabled
+    $menubar.edit entryconfigure [_ "Tidy Up"] -state disabled
     $menubar.edit entryconfigure [_ "Edit Mode"] -state disabled
     pdtk_canvas_editmode $mytoplevel 0
     # Undo/Redo change names, they need to have the asterisk (*) after
@@ -123,6 +147,9 @@ proc ::pd_menus::configure_for_dialog {mytoplevel} {
         # catch errors that happen when trying to disable separators
         catch {$menubar.put entryconfigure $i -state disabled }
     }
+    # Help menu
+    # make sure "List of objects..." is enabled, it sometimes greys out on Mac
+    $menubar.help entryconfigure [_ "List of objects..."] -state normal
 }
 
 
@@ -136,7 +163,7 @@ proc ::pd_menus::build_file_menu {mymenu} {
     $mymenu entryconfigure [_ "Save"]       -command {menu_send $::focused_window menusave}
     $mymenu entryconfigure [_ "Save As..."] -command {menu_send $::focused_window menusaveas}
     #$mymenu entryconfigure [_ "Revert*"]    -command {menu_revert $::focused_window}
-    $mymenu entryconfigure [_ "Close"]      -command {menu_send_float $::focused_window menuclose 0}
+    $mymenu entryconfigure [_ "Close"]      -command {::pd_bindings::window_close $::focused_window}
     $mymenu entryconfigure [_ "Message..."] -command {menu_message_dialog}
     $mymenu entryconfigure [_ "Print..."]   -command {menu_print $::focused_window}
     # update recent files
@@ -148,9 +175,9 @@ proc ::pd_menus::build_file_menu {mymenu} {
 proc ::pd_menus::build_edit_menu {mymenu} {
     variable accelerator
     $mymenu add command -label [_ "Undo"]       -accelerator "$accelerator+Z" \
-        -command {menu_undo $::focused_window}
+        -command {menu_undo}
     $mymenu add command -label [_ "Redo"]       -accelerator "Shift+$accelerator+Z" \
-        -command {menu_redo $::focused_window}
+        -command {menu_redo}
     $mymenu add  separator
     $mymenu add command -label [_ "Cut"]        -accelerator "$accelerator+X" \
         -command {menu_send $::focused_window cut}
@@ -163,28 +190,19 @@ proc ::pd_menus::build_edit_menu {mymenu} {
     $mymenu add command -label [_ "Select All"] -accelerator "$accelerator+A" \
         -command {menu_send $::focused_window selectall}
     $mymenu add  separator
-    if {$::windowingsystem eq "aqua"} {
-#        $mymenu add command -label [_ "Text Editor"] \
-#            -command {menu_texteditor}
-        $mymenu add command -label [_ "Font"]  -accelerator "$accelerator+T" \
-            -command {menu_font_dialog}
-    } else {
-#        $mymenu add command -label [_ "Text Editor"] -accelerator "$accelerator+T"\
-#            -command {menu_texteditor}
-        $mymenu add command -label [_ "Font"] \
-            -command {menu_font_dialog}
-    }
-    $mymenu add command -label [_ "Zoom In"] -accelerator "$accelerator++" \
+    $mymenu add command -label [_ "Font"]       -accelerator "$accelerator+T" \
+        -command {menu_font_dialog}
+    $mymenu add command -label [_ "Zoom In"]    -accelerator "$accelerator++" \
         -command {menu_send_float $::focused_window zoom 2}
-    $mymenu add command -label [_ "Zoom Out"] -accelerator "$accelerator+-" \
+    $mymenu add command -label [_ "Zoom Out"]   -accelerator "$accelerator+-" \
         -command {menu_send_float $::focused_window zoom 1}
-    $mymenu add command -label [_ "Tidy Up"] \
+    $mymenu add command -label [_ "Tidy Up"]    -accelerator "$accelerator+Shift+R" \
         -command {menu_send $::focused_window tidy}
     $mymenu add command -label [_ "Clear Console"] \
         -accelerator "Shift+$accelerator+L" -command {menu_clear_console}
     $mymenu add  separator
     #TODO madness! how to set the state of the check box without invoking the menu!
-    $mymenu add check -label [_ "Edit Mode"] -accelerator "$accelerator+E" \
+    $mymenu add check -label [_ "Edit Mode"]    -accelerator "$accelerator+E" \
         -variable ::editmode_button \
         -command {menu_editmode $::editmode_button}
     if {$::windowingsystem ne "aqua"} {
@@ -199,38 +217,40 @@ proc ::pd_menus::build_put_menu {mymenu} {
     # The trailing 0 in menu_send_float basically means leave the object box
     # sticking to the mouse cursor. The iemguis alway do that when created
     # from the menu, as defined in canvas_iemguis()
-    $mymenu add command -label [_ "Object"] -accelerator "$accelerator+1" \
-        -command {menu_send_float $::focused_window obj 0} 
-    $mymenu add command -label [_ "Message"] -accelerator "$accelerator+2" \
+    $mymenu add command -label [_ "Object"]   -accelerator "$accelerator+1" \
+        -command {menu_send_float $::focused_window obj 0}
+    $mymenu add command -label [_ "Message"]  -accelerator "$accelerator+2" \
         -command {menu_send_float $::focused_window msg 0}
-    $mymenu add command -label [_ "Number"] -accelerator "$accelerator+3" \
+    $mymenu add command -label [_ "Number"]   -accelerator "$accelerator+3" \
         -command {menu_send_float $::focused_window floatatom 0}
-    $mymenu add command -label [_ "Symbol"] -accelerator "$accelerator+4" \
+    $mymenu add command -label [_ "Symbol"]   -accelerator "$accelerator+4" \
         -command {menu_send_float $::focused_window symbolatom 0}
-    $mymenu add command -label [_ "Comment"] -accelerator "$accelerator+5" \
+    $mymenu add command -label [_ "Comment"]  -accelerator "$accelerator+5" \
         -command {menu_send_float $::focused_window text 0}
     $mymenu add  separator
-    $mymenu add command -label [_ "Bang"]    -accelerator "Shift+$accelerator+B" \
+    $mymenu add command -label [_ "Bang"]     -accelerator "Shift+$accelerator+B" \
         -command {menu_send $::focused_window bng}
-    $mymenu add command -label [_ "Toggle"]  -accelerator "Shift+$accelerator+T" \
+    $mymenu add command -label [_ "Toggle"]   -accelerator "Shift+$accelerator+T" \
         -command {menu_send $::focused_window toggle}
-    $mymenu add command -label [_ "Number2"] -accelerator "Shift+$accelerator+N" \
+    $mymenu add command -label [_ "Number2"]  -accelerator "Shift+$accelerator+N" \
         -command {menu_send $::focused_window numbox}
-    $mymenu add command -label [_ "Vslider"] -accelerator "Shift+$accelerator+V" \
+    $mymenu add command -label [_ "Vslider"]  -accelerator "Shift+$accelerator+V" \
         -command {menu_send $::focused_window vslider}
-    $mymenu add command -label [_ "Hslider"] -accelerator "Shift+$accelerator+H" \
+    $mymenu add command -label [_ "Hslider"]  -accelerator "Shift+$accelerator+J" \
         -command {menu_send $::focused_window hslider}
-    $mymenu add command -label [_ "Vradio"]  -accelerator "Shift+$accelerator+D" \
+    $mymenu add command -label [_ "Vradio"]   -accelerator "Shift+$accelerator+D" \
         -command {menu_send $::focused_window vradio}
-    $mymenu add command -label [_ "Hradio"]  -accelerator "Shift+$accelerator+I" \
+    $mymenu add command -label [_ "Hradio"]   -accelerator "Shift+$accelerator+I" \
         -command {menu_send $::focused_window hradio}
-    $mymenu add command -label [_ "VU Meter"] -accelerator "Shift+$accelerator+U"\
+    $mymenu add command -label [_ "VU Meter"] -accelerator "Shift+$accelerator+U" \
         -command {menu_send $::focused_window vumeter}
-    $mymenu add command -label [_ "Canvas"]  -accelerator "Shift+$accelerator+C" \
+    $mymenu add command -label [_ "Canvas"]   -accelerator "Shift+$accelerator+C" \
         -command {menu_send $::focused_window mycnv}
     $mymenu add  separator
-    $mymenu add command -label [_ "Graph"] -command {menu_send $::focused_window graph}
-    $mymenu add command -label [_ "Array"] -command {menu_send $::focused_window menuarray}
+    $mymenu add command -label [_ "Graph"]    -accelerator "Shift+$accelerator+G" \
+        -command {menu_send $::focused_window graph}
+    $mymenu add command -label [_ "Array"]    -accelerator "Shift+$accelerator+A" \
+        -command {menu_send $::focused_window menuarray}
 }
 
 proc ::pd_menus::build_find_menu {mymenu} {
@@ -240,21 +260,21 @@ proc ::pd_menus::build_find_menu {mymenu} {
     $mymenu add command -label [_ "Find Again"] -accelerator "$accelerator+G" \
         -command {menu_send $::focused_window findagain}
     $mymenu add command -label [_ "Find Last Error"] \
-        -command {pdsend {pd finderror}} 
+        -command {pdsend {pd finderror}}
 }
 
 proc ::pd_menus::build_media_menu {mymenu} {
     variable accelerator
-    $mymenu add radiobutton -label [_ "DSP On"] -accelerator "$accelerator+/" \
+    $mymenu add radiobutton -label [_ "DSP On"]  -accelerator "$accelerator+/" \
         -variable ::dsp -value 1 -command {pdsend "pd dsp 1"}
     $mymenu add radiobutton -label [_ "DSP Off"] -accelerator "$accelerator+." \
         -variable ::dsp -value 0 -command {pdsend "pd dsp 0"}
 
     $mymenu add  separator
     $mymenu add command -label [_ "Test Audio and MIDI..."] \
-        -command {menu_doc_open doc/7.stuff/tools testtone.pd} 
+        -command {menu_doc_open doc/7.stuff/tools testtone.pd}
     $mymenu add command -label [_ "Load Meter"] \
-        -command {menu_doc_open doc/7.stuff/tools load-meter.pd} 
+        -command {menu_doc_open doc/7.stuff/tools load-meter.pd}
 
     set audio_apilist_length [llength $::audio_apilist]
     if {$audio_apilist_length > 0} {$mymenu add separator}
@@ -264,7 +284,7 @@ proc ::pd_menus::build_media_menu {mymenu} {
             -value [lindex [lindex $::audio_apilist $x] 1]\
             -command {pdsend "pd audio-setapi $::pd_whichapi"}
     }
-    
+
     set midi_apilist_length [llength $::midi_apilist]
     if {$midi_apilist_length > 0} {$mymenu add separator}
     for {set x 0} {$x<$midi_apilist_length} {incr x} {
@@ -284,18 +304,23 @@ proc ::pd_menus::build_media_menu {mymenu} {
 proc ::pd_menus::build_window_menu {mymenu} {
     variable accelerator
     if {$::windowingsystem eq "aqua"} {
-        $mymenu add command -label [_ "Minimize"] -accelerator "$accelerator+M"\
-            -command {menu_minimize $::focused_window}
-        $mymenu add command -label [_ "Zoom"] \
-            -command {menu_maximize $::focused_window}
-        $mymenu add  separator
-        $mymenu add command -label [_ "Bring All to Front"] \
-            -command {menu_bringalltofront}
+        # Tk 8.5+ automatically adds default Mac window menu items
+        if {$::tcl_version < 8.5} {
+            $mymenu add command -label [_ "Minimize"] -accelerator "$accelerator+M" \
+                -command {menu_minimize $::focused_window}
+            $mymenu add command -label [_ "Zoom"] \
+                -command {menu_maximize $::focused_window}
+            $mymenu add  separator
+            $mymenu add command -label [_ "Bring All to Front"] \
+                -command {menu_bringalltofront}
+        }
     } else {
-		$mymenu add command -label [_ "Next Window"] \
+        $mymenu add command -label [_ "Minimize"] -accelerator "$accelerator+M" \
+                -command {menu_minimize $::focused_window}
+        $mymenu add command -label [_ "Next Window"] \
             -command {menu_raisenextwindow} \
             -accelerator [_ "$accelerator+Page Down"]
-		$mymenu add command -label [_ "Previous Window"] \
+        $mymenu add command -label [_ "Previous Window"] \
             -command {menu_raisepreviouswindow} \
             -accelerator [_ "$accelerator+Page Up"]
     }
@@ -308,20 +333,25 @@ proc ::pd_menus::build_window_menu {mymenu} {
 }
 
 proc ::pd_menus::build_help_menu {mymenu} {
+    variable accelerator
     if {$::windowingsystem ne "aqua"} {
-        $mymenu add command -label [_ "About Pd"] -command {menu_aboutpd} 
+        # Tk creates this automatically on Mac
+        $mymenu add command -label [_ "About Pd"] -command {menu_aboutpd}
     }
-    $mymenu add command -label [_ "HTML Manual..."] \
-        -command {menu_doc_open doc/1.manual index.htm}
-    $mymenu add command -label [_ "Browser..."] \
-        -command {menu_helpbrowser} 
+    if {$::windowingsystem ne "aqua" || $::tcl_version < 8.5} {
+        # TK 8.5+ on Mac creates this automatically, other platforms do not
+        $mymenu add command -label [_ "HTML Manual..."] \
+                -command {menu_manual}
+    }
+    $mymenu add command -label [_ "Browser..."] -accelerator "$accelerator+B" \
+        -command {menu_helpbrowser}
     $mymenu add command -label [_ "List of objects..."] \
-        -command {pdsend "pd help-intro"} 
+        -command {menu_objectlist}
     $mymenu add  separator
     $mymenu add command -label [_ "puredata.info"] \
-        -command {menu_openfile {http://puredata.info}} 
+        -command {menu_openfile {http://puredata.info}}
     $mymenu add command -label [_ "Report a bug"] -command {menu_openfile \
-        {http://sourceforge.net/tracker/?func=add&group_id=55736&atid=478070}} 
+        {http://bugs.puredata.info}}
 }
 
 #------------------------------------------------------------------------------#
@@ -355,10 +385,9 @@ proc ::pd_menus::update_recentfiles_menu {{write true}} {
 }
 
 proc ::pd_menus::clear_recentfiles_menu {} {
+    # empty recentfiles in preferences (write empty array)
     set ::recentfiles_list {}
     ::pd_menus::update_recentfiles_menu
-    # empty recentfiles in preferences (write empty array)
-    ::pd_guiprefs::write_recentfiles
 }
 
 proc ::pd_menus::update_openrecent_menu_aqua {mymenu {write}} {
@@ -394,15 +423,17 @@ proc ::pd_menus::update_recentfiles_on_menu {mymenu {write}} {
     }
     # insert the list from the end because we insert each element on the top
     set i [llength $::recentfiles_list]
-    while {[incr i -1] > 0} {
-
+    while {[incr i -1] > -1} {
         set filename [lindex $::recentfiles_list $i]
+        set j [expr $i + 1]
+        if {$::windowingsystem eq "aqua"} {
+            set label [file tail $filename]
+        } else {
+            set label [concat "$j. " [file tail $filename]]
+        }
         $mymenu insert [expr $top_separator+1] command \
-            -label [file tail $filename] -command "open_file {$filename}"
+            -label $label -command "open_file {$filename}" -underline 0
     }
-    set filename [lindex $::recentfiles_list 0]
-    $mymenu insert [expr $top_separator+1] command \
-        -label [file tail $filename] -command "open_file {$filename}"
 
     # write to config file
     if {$write == true} { ::pd_guiprefs::write_recentfiles }
@@ -464,6 +495,12 @@ proc ::pd_menus::add_list_to_menu {mymenu window parentlist} {
 # update the list of windows on the Window menu. This expects run on the
 # Window menu, and to insert below the last separator
 proc ::pd_menus::update_window_menu {} {
+
+    # TK 8.5+ Cocoa on Mac handles the window list for us
+    if {$::windowingsystem eq "aqua" && $::tcl_version >= 8.5} {
+        return 0
+    }
+
     set mymenu $::patch_menubar.window
     # find the last separator and delete everything after that
     for {set i 0} {$i <= [$mymenu index end]} {incr i} {
@@ -482,6 +519,26 @@ proc ::pd_menus::update_window_menu {} {
 
 # ------------------------------------------------------------------------------
 # submenu for Preferences, now used on all platforms
+proc ::pd_menus::savepreferences {} {
+    set filename [tk_getSaveFile \
+                  -initialdir $::fileopendir \
+                  -defaultextension .pdsettings \
+                  -initialfile Untitled.pdsettings]
+    if {$filename ne ""} {pdsend "pd save-preferences $filename"}
+}
+
+proc ::pd_menus::loadpreferences {} {
+    set filename [tk_getOpenFile \
+                  -initialdir $::fileopendir \
+                  -defaultextension .pdsettings]
+    if {$filename ne ""} {pdsend "pd load-preferences $filename"}
+}
+
+proc ::pd_menus::forgetpreferences {} {
+    pdtk_check .pdwindow \
+        [_ "Delete all preferences?\n(takes effect when Pd is restarted)"] \
+        {pd forget-preferences} yes
+}
 
 proc ::pd_menus::create_preferences_menu {mymenu} {
     menu $mymenu
@@ -489,16 +546,22 @@ proc ::pd_menus::create_preferences_menu {mymenu} {
         -command {pdsend "pd start-path-dialog"}
     $mymenu add command -label [_ "Startup..."] \
         -command {pdsend "pd start-startup-dialog"}
-    $mymenu add command -label [_ "Audio Settings..."] \
+    $mymenu add command -label [_ "Audio..."] \
         -command {pdsend "pd audio-properties"}
-    $mymenu add command -label [_ "MIDI Settings..."] \
+    $mymenu add command -label [_ "MIDI..."] \
         -command {pdsend "pd midi-properties"}
-    $mymenu add check -label [_ "Zoom new windows"] \
+    $mymenu add check -label [_ "Zoom New Windows"] \
         -variable ::zoom_open \
         -command {pdsend "pd zoom-open $zoom_open"}
     $mymenu add  separator
-    $mymenu add command -label [_ "Save All Settings"] \
+    $mymenu add command -label [_ "Save All Preferences"] \
         -command {pdsend "pd save-preferences"}
+    $mymenu add command -label [_ "Save to..."] \
+        -command {::pd_menus::savepreferences}
+    $mymenu add command -label [_ "Load from..."] \
+        -command {::pd_menus::loadpreferences}
+    $mymenu add command -label [_ "Forget All..."] \
+        -command {::pd_menus::forgetpreferences}
 }
 
 # ------------------------------------------------------------------------------
@@ -509,30 +572,29 @@ proc ::pd_menus::create_apple_menu {mymenu} {
     # TODO this should open a Pd patch called about.pd
     menu $mymenu.apple
     $mymenu.apple add command -label [_ "About Pd"] -command {menu_aboutpd}
-    $mymenu.apple add  separator
+    $mymenu.apple add separator
     create_preferences_menu $mymenu.apple.preferences
     $mymenu.apple add cascade -label [_ "Preferences"] \
         -menu $mymenu.apple.preferences
     # this needs to be last for things to function properly
     $mymenu add cascade -label "Apple" -menu $mymenu.apple
-    
 }
 
 proc ::pd_menus::build_file_menu_aqua {mymenu} {
     variable accelerator
-    $mymenu add command -label [_ "New"]       -accelerator "$accelerator+N"
-    $mymenu add command -label [_ "Open"]      -accelerator "$accelerator+O"
+    $mymenu add command -label [_ "New"]        -accelerator "$accelerator+N"
+    $mymenu add command -label [_ "Open"]       -accelerator "$accelerator+O"
     # this is now done in main ::pd_menus::build_file_menu
     #::pd_menus::update_openrecent_menu_aqua .openrecent
     $mymenu add cascade -label [_ "Open Recent"] -menu .openrecent
     $mymenu add  separator
-    $mymenu add command -label [_ "Close"]     -accelerator "$accelerator+W"
-    $mymenu add command -label [_ "Save"]      -accelerator "$accelerator+S"
+    $mymenu add command -label [_ "Close"]      -accelerator "$accelerator+W"
+    $mymenu add command -label [_ "Save"]       -accelerator "$accelerator+S"
     $mymenu add command -label [_ "Save As..."] -accelerator "$accelerator+Shift+S"
     #$mymenu add command -label [_ "Save All"]
     #$mymenu add command -label [_ "Revert to Saved"]
     $mymenu add  separator
-    $mymenu add command -label [_ "Message..."]
+    $mymenu add command -label [_ "Message..."] -accelerator "$accelerator+Shift+M"
     $mymenu add  separator
     $mymenu add command -label [_ "Print..."]   -accelerator "$accelerator+P"
 }
@@ -546,28 +608,28 @@ proc ::pd_menus::build_window_menu_aqua {mymenu} {
 }
 
 # the "Help" does not have cross-platform differences
- 
+
 # ------------------------------------------------------------------------------
 # menu building functions for UNIX/X11
 
 proc ::pd_menus::build_file_menu_x11 {mymenu} {
     variable accelerator
-    $mymenu add command -label [_ "New"]        -accelerator "$accelerator+N"
-    $mymenu add command -label [_ "Open"]       -accelerator "$accelerator+O"
+    $mymenu add command -label [_ "New"]         -accelerator "$accelerator+N"
+    $mymenu add command -label [_ "Open"]        -accelerator "$accelerator+O"
     $mymenu add  separator
-    $mymenu add command -label [_ "Save"]       -accelerator "$accelerator+S"
-    $mymenu add command -label [_ "Save As..."] -accelerator "Shift+$accelerator+S"
+    $mymenu add command -label [_ "Save"]        -accelerator "$accelerator+S"
+    $mymenu add command -label [_ "Save As..."]  -accelerator "Shift+$accelerator+S"
     #    $mymenu add command -label "Revert"
     $mymenu add  separator
-    $mymenu add command -label [_ "Message..."]    -accelerator "$accelerator+M"
+    $mymenu add command -label [_ "Message..."]  -accelerator "$accelerator+M"
     create_preferences_menu $mymenu.preferences
     $mymenu add cascade -label [_ "Preferences"] -menu $mymenu.preferences
-    $mymenu add command -label [_ "Print..."]   -accelerator "$accelerator+P"
+    $mymenu add command -label [_ "Print..."]    -accelerator "$accelerator+P"
     $mymenu add  separator
     # the recent files get inserted in here by update_recentfiles_on_menu
     $mymenu add  separator
-    $mymenu add command -label [_ "Close"]      -accelerator "$accelerator+W"
-    $mymenu add command -label [_ "Quit"]       -accelerator "$accelerator+Q" \
+    $mymenu add command -label [_ "Close"]       -accelerator "$accelerator+W"
+    $mymenu add command -label [_ "Quit"]        -accelerator "$accelerator+Q" \
         -command {pdsend "pd verifyquit"}
 }
 
@@ -599,22 +661,22 @@ proc ::pd_menus::create_system_menu {mymenubar} {
 
 proc ::pd_menus::build_file_menu_win32 {mymenu} {
     variable accelerator
-    $mymenu add command -label [_ "New"]      -accelerator "$accelerator+N"
-    $mymenu add command -label [_ "Open"]     -accelerator "$accelerator+O"
+    $mymenu add command -label [_ "New"]         -accelerator "$accelerator+N"
+    $mymenu add command -label [_ "Open"]        -accelerator "$accelerator+O"
     $mymenu add  separator
-    $mymenu add command -label [_ "Save"]      -accelerator "$accelerator+S"
-    $mymenu add command -label [_ "Save As..."] -accelerator "Shift+$accelerator+S"
+    $mymenu add command -label [_ "Save"]        -accelerator "$accelerator+S"
+    $mymenu add command -label [_ "Save As..."]  -accelerator "Shift+$accelerator+S"
     #    $mymenu add command -label "Revert"
     $mymenu add  separator
     $mymenu add command -label [_ "Message..."]  -accelerator "$accelerator+M"
     create_preferences_menu $mymenu.preferences
     $mymenu add cascade -label [_ "Preferences"] -menu $mymenu.preferences
-    $mymenu add command -label [_ "Print..."] -accelerator "$accelerator+P"
+    $mymenu add command -label [_ "Print..."]    -accelerator "$accelerator+P"
     $mymenu add  separator
     # the recent files get inserted in here by update_recentfiles_on_menu
     $mymenu add  separator
-    $mymenu add command -label [_ "Close"]    -accelerator "$accelerator+W"
-    $mymenu add command -label [_ "Quit"]     -accelerator "$accelerator+Q"\
+    $mymenu add command -label [_ "Close"]       -accelerator "$accelerator+W"
+    $mymenu add command -label [_ "Quit"]        -accelerator "$accelerator+Q" \
         -command {pdsend "pd verifyquit"}
 }
 
